# Pupitre d'EAD
#
# History:
# 2019/02/24: Delmas-Begue Ulysse: adaptation du reseau pour s'interfcaer avec D17
#                                  configuration par defaut pour D17
# 2017/03/22: Delmas-Begue Ulysse: rajout du reseau pour s'interfacer avec fdcc2017
# 2016/12/19: Delmas-Begue Ulysse: rajout DCC et phares
# 2016/11/24: Delmas-Begue Ulysse: creation
#
# Notes:
# License: GPLv2
# Photo du pupitre: http://projetsesk.perso.sfr.fr/XBDnuit.jpg
# This application works on Linux/Windows and Python3.x/2.x
#
# TBD
# - pente + (vent) ?
# - sonds (+klaxon)
# - compteur de vitesse un peu plus realiste (celui ci monte a 200km/h)

TITLE = "FDCC Pupitre EAD v20190226 UDB"

use_serial    = 0
SZ_SERIALPORT = "\\\\.\\COM4"

use_network   = 1
use_local     = 0
server_ip_adr = "192.168.4.1"
server_port   = 1234


# caracteristique des differentes familles (surement qq erreurs)
#
# type         |   x4300 |   x4500 |   x4630 |     x4900 | x4750/4790/94750 |
# caisses      |       2 |       2 |       2 |         3 |                2 |
# mot          | P 295kW | S 330kW | S 330kW | S 2x330kW |          S 440kW |
# transmission |     BV8 |     BV8 |    CPH2 |      CPH2 |             CPH2 |
# vmax         | 120kmph | 120kmph | 120kmph |   140kmph |          140kmph |
# ralenti      |  550rpm |  550rpm |  550rpm |    550rpm |           550rpm |
# nominal max  | 1800rpm | 1500rpm | 1500rpm |   1500rpm |          1500rpm |
# reservoire   |    800L |    800L |    800L |    2x800L |             800L |
# conso km     |    0.8L |    0.8L |    0.8L |    2x0.8L |               1L |
# masse        |     57t |     57t |     64t |      104t |              64t |
# nb           |     151 |     126 |     115 |       13  |               54 |
#
# *P=Poyaud / S=Saurer / BV8=Boite 8vit / CPH2=CouPleur Hydraulque 2 vit
# couplage = 6 caisses max
# modernise = +6t

# pupitre:
# temperature bat                                                 ?
# daat                                                   ?        ?
# defaut inverseur                                       arret    marche
# cabine | eclairage pupitre | general | signalis | chauff cabine | anti-buee | essai-VA
#
#* signalisation ou encore feux
#* general coupe tous les autres inters

# parametres pwm pour commander une locomotive via carte Arduino
pwm_vit_max    = 140  # vitesse correspondante a pwm_max
pwm_max        = 255  # valeur de la pwm(0-255) a la vitesse max
pwm_min        = 1    # valeur de la pwm(0-255) pour demarrer la loco
pwm_inv_sens   = 0    # invertion du sens (0=ne pas inverser, 1=inverser)

# parametres dcc pour commander une locomotive via carte Arduino
dcc_adr        = 3    # adresse de la locomotive
dcc_vit_max    = 140  # vitesse correspondante a pwm_max
dcc_max        = 28   # valeur du cran DCC 0-28 a la vitesse max
dcc_min        = 1    # valeur du cran DCC 0-28 pour demarrer la loco
dcc_inv_sens   = 0    # invertion du sens (0=ne pas inverser, 1=inverser)
net_max        = 100  # valeur a envoyer par le reseau a la vitesse max (100 pour D17, 28 pour fdcc2017)
net_min        = 1    # valeur a envoyer par le reseau pour demarrer la loco


# parametres de simulation (a adapter suivant les besoins)
vmax = 140.0               # kmph
t_0_100_mot  = 10.0        # s
t_100_0_mot  = 10.0        # s (frein moteur)
t_100_0_fr   = 5.0         # s
t_100_0_erre = 30.0        # s
t_press_3b5_5b    = 4.0    # s
t_press_au_5b_3b5 = 1.0    # s
use_bv_0_cph_1 = 0         # 0=BV8 1=CPH2
rpm_ralenti  = 500.0       # tr/min
rpm_nominal = 1500.0       # tr/min
slow_aig_trm_inc = 40.0    # tr/min /tic

# a l'init
motor_on = 0
vit = 0.0                  # kmph (a l'init)
pression_rp_0_10  = 8.0    # bars (a l'init mais bouge pas)
pression_cg_0_5   = 0.0    # bars (a l'init)
pression_cf_0_5   = 5.0    # bars (a l'init)
fr_0_100          = 100.0  # %    (a l'init)
robinet_au = 0             # 0=ferme, 1=ouvert donc AU
manette_acc_0_100 = 0      # %

RETIRE = -1
ISO = 0
SERVICE = 1
NEUTRE = 2
palette_service = RETIRE

AV = 1
ARRET = 0
AR = 2
palette_sens = ARRET
mem_sens = AV

DESS = 2
RIEN = 0
SERR = 1
manette_frein  = RIEN
manette_freind = RIEN   #non supporte

KL_AV  = 1
KL_0   = 0
KL_AR  = 2
manette_klaxon = KL_0

LSSF_0   = 0
LSSF_1   = 1
LSSF_CLI = 2
lssf = LSSF_0
lssf_dcpt = LSSF_0

bv_0_8 = 0
cph_0_2 = 0

# over write pour aller plus vite pendant le debug
palette_service   = SERVICE
pression_cg_0_5   = 5.0
pression_cf_0_5   = 0.0
fr_0_100          = 0.0



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# IMPORT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

import os
import time
import sys
import subprocess
import threading
import datetime
import socket
try:
    # for Python 3
    from tkinter import *
    print("Python 3.x interpreter detected")
except ImportError:
    # for Python2
    from Tkinter import *
    print("Python 2.x interpreter detected")
from math import *

ser = 0
serial_ok = 1
if(use_serial):
    try:
        import serial
    except ImportError:
        serial_ok = 0
        print("Cannot load serial lib")
else:
    serial_ok = 0



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# DRAW FCTs
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

# parametres du pupitre
#image_du_pupitre = "C:\\Users\\udelmasb\\Desktop\\python\\try\\ead\\ead.png"
image_du_pupitre = "ead.png"
#image_du_pupitre = "ead_nuit.png"

x_aig_vit = 772; y_aig_vit = 111; d_aig_vit = 53
x_aig_trm = 507; y_aig_trm = 107; d_aig_trm = 53

x_aig_rp = 139; y_aig_rp = 144; d_aig_rp = 31
x_aig_cg = 139; y_aig_cg = 144; d_aig_cg = 31
x_aig_cf = 252; y_aig_cf = 157; d_aig_cf = 31

xc_boule_acc   = 797; yc_boule_acc   = 300
x0_boule_acc   = 860; y0_boule_acc   = 300
x100_boule_acc = 780; y100_boule_acc = 200

xc_boule_fr    = 258; yc_boule_fr    = 284
xdess_boule_fr = 270; ydess_boule_fr = 192
x0_boule_fr    = 250; y0_boule_fr    = 222
xserr_boule_fr = 230; yserr_boule_fr = 252

xc_sens   = 746; yc_sens   = 244
xav_sens  = 719; yav_sens  = 193
x0_sens   = 747; y0_sens   = 188
xarr_sens = 752; yarr_sens = 229

xc_srv   = 286; yc_srv   = 240
x_neutre = 299; y_neutre = 200
x_srv    = 286; y_srv    = 193
x_iso    = 277; y_iso    = 248

xc_klaxon   = 890; yc_klaxon   = 340
xav_klaxon  = 870; yav_klaxon  = 235
x0_klaxon   = 890; y0_klaxon   = 265
xarr_klaxon = 910; yarr_klaxon = 295

xc_au = 41 ; yc_au = 287
x_au  = 121; y_au  = 243
x0_au = 52 ; y0_au = 175

x_lssf      = 274; y_lssf      = 30 ; d_lssf     = 15
xc_ack_lssf = 940; yc_ack_lssf = 350; d_ack_lssf = 20

line_aig_vit=0; line_aig_trm=0; line_aig_rp=0; line_aig_cg=0; line_aig_cf=0
line_au=0; oval_lssf = 0
line_cmd_srv=0; line2_cmd_srv=0
line_cmd_sens=0; line2_cmd_sens=0
line_cmd_fr=0; oval_cmd_fr=0
line_cmd_acc = 0; oval_cmd_acc = 0
line_cmd_klaxon=0; oval_cmd_klaxon=0

xc_phares = 505; yc_phares = 165; rect_phares = 0; phares = 0; old_phares = -1

zefont = ("Times New Roman", 6)
filename_pupitre = 0
dbg_txt = ""
x_press = 0; y_press = 0


#creation du pupitre (image, aiguilles, manettes)
def draw_first():
    global filename_pupitre   #necessaire car si local, image perdue a la sortie de la fonction
    global dbg_txt
    global line_aig_vit; global line_aig_trm; global line_aig_rp; global line_aig_cg; global line_aig_cf
    global line_au;
    global line_cmd_srv; global line2_cmd_srv
    global line_cmd_sens; global line2_cmd_sens
    global line_cmd_fr; global oval_cmd_fr
    global line_cmd_acc; global oval_cmd_acc
    global line_cmd_klaxon; global oval_cmd_klaxon
    global oval_lssf
    global rect_phares

    #my_canvas.create_rectangle(x, y, x + 100, y +200 , fill = "#000000"  , outline = "#ffffff")

    dbg_txt = my_canvas.create_text(10, 380, text = "DBG: .............", fill = "#ff0000", anchor = "nw")

    # image pupitre
    filename_pupitre = PhotoImage(file = image_du_pupitre)
    image_pupitre = my_canvas.create_image(0, 0, image=filename_pupitre, anchor = "nw")   #anchor=NE,

    # aiguilles
    line_aig_vit = my_canvas.create_line(0,0,0,0,width=5,fill="#ffffff")
    line_aig_trm = my_canvas.create_line(0,0,0,0,width=4,fill="#ffffff")
    line_aig_rp  = my_canvas.create_line(0,0,0,0,width=4,fill="#ff0000")
    line_aig_cg  = my_canvas.create_line(0,0,0,0,width=3,fill="#ffffff")
    line_aig_cf  = my_canvas.create_line(0,0,0,0,width=3,fill="#ffffff")

    # avertisseur
    line_cmd_klaxon = my_canvas.create_line(0,0,0,0,width=7,fill="#444444")
    oval_cmd_klaxon = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    line_dbg_klaxon = my_canvas.create_line(x0_klaxon,y0_klaxon,xav_klaxon,yav_klaxon,width=1,fill="#00ff00")
    line2_dbg_klaxon= my_canvas.create_line(x0_klaxon,y0_klaxon,xarr_klaxon,yarr_klaxon,width=1,fill="#00ff00")

    # manette de sens
    line_cmd_sens  = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    line2_cmd_sens = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    line_dbg_fr    = my_canvas.create_line(x0_sens,y0_sens,xav_sens,yav_sens,width=1,fill="#00ff00")
    line2_dbg_fr   = my_canvas.create_line(x0_sens,y0_sens,xarr_sens,yarr_sens,width=1,fill="#00ff00")

    # manette de mise en service
    line_cmd_srv  = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    line2_cmd_srv = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    line_dbg_srv  = my_canvas.create_line(x_srv,y_srv,x_neutre,y_neutre,width=1,fill="#00ff00")
    line2_dbg_srv = my_canvas.create_line(x_srv,y_srv,x_iso,y_iso,width=1,fill="#00ff00")

    # manette d'acceleration
    line_cmd_acc = my_canvas.create_line(0,0,0,0,width=7,fill="#888888")
    oval_cmd_acc = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    line_dbg_acc = my_canvas.create_line(x0_boule_acc,y0_boule_acc,x100_boule_acc,y100_boule_acc,width=1,fill="#00ff00")

    # manette de freinage
    line_cmd_fr = my_canvas.create_line(0,0,0,0,width=7,fill="#888888")
    oval_cmd_fr = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    line_dbg_fr = my_canvas.create_line(x0_boule_fr,y0_boule_fr,xserr_boule_fr,yserr_boule_fr,width=1,fill="#00ff00")
    line2_dbg_fr= my_canvas.create_line(x0_boule_fr,y0_boule_fr,xdess_boule_fr,ydess_boule_fr,width=1,fill="#00ff00")

    # robinet d'arret d'urgence
    line_au = my_canvas.create_line(xc_au,yc_au,x_au,y_au,width=10,fill="#ff0000")

    #lssf
    oval_lssf = my_canvas.create_oval(x_lssf-d_lssf,y_lssf-d_lssf,x_lssf+d_lssf,y_lssf+d_lssf,fill="#1f1820")

    #bouton signalisation (phares)
    rect_phares = my_canvas.create_rectangle(xc_phares-5, yc_phares-5, xc_phares+5, yc_phares+20, fill='#888888', outline='#000000', width=1)

    return

# valeurs
aig_val_vit_0_210 = 0.0
aig_val_trm_0_2500 = 0.0
aig_val_rp_0_10 = 8.0
aig_val_cg_0_10 = 0.0
aig_val_cf_0_10 = 5.0
slow_aig_val_trm_0_2500 = 0.0

old_aig_val_vit_0_210 = -1.0   #sauvegarde des valeurs entre 2 appels pour
old_aig_val_trm_0_2500 = -1.0  #ne pas redessiner et ainsi gagner en performances
old_aig_val_rp_0_10 = -1.0
old_aig_val_cg_0_10 = -1.0
old_aig_val_cf_0_10 = -1.0
old_robinet_au = -1
old_palette_sens = -1
old_palette_service=-1
old_manette_frein = -1
old_manette_acc_0_100 = -1.0
old_manette_klaxon = -1
old_lssf = -1
draw_update_cpt = 0


#mise a jour des aiguilles et position des manettes
def draw_update():
    global my_canvas
    global line_aig_vit; global line_aig_trm; global line_aig_rp; global line_aig_cg; global line_aig_cf
    global line_cmd_srv; global line2_cmd_srv
    global line_cmd_fr; global oval_cmd_fr
    global line_cmd_acc; global oval_cmd_acc
    global line_cmd_srv; global line2_cmd_srv
    global old_aig_val_vit_0_210; global old_aig_val_trm_0_2500
    global old_aig_val_rp_0_10; global old_aig_val_cg_0_10; global old_aig_val_cf_0_10
    global old_robinet_au;
    global old_palette_sens; global old_palette_service; global old_manette_frein; global old_manette_acc_0_100
    global old_manette_klaxon; global line_cmd_klaxon; global oval_cmd_klaxon
    global old_lssf; global oval_lssf
    global old_phares; global rect_phares
    global draw_update_cpt

    draw_update_cpt += 1

    # mise a jour des aiguilles
    if(aig_val_vit_0_210 != old_aig_val_vit_0_210):
        angle = 213.0 - (213.0-(-23.0))*(aig_val_vit_0_210/210.0)
        x = int( float(x_aig_vit) + float(d_aig_vit) * cos(radians(angle)) )
        y = int( float(y_aig_vit) - float(d_aig_vit) * sin(radians(angle)) )
        my_canvas.coords(line_aig_vit, x_aig_vit, y_aig_vit, x, y)
        old_aig_val_vit_0_210 = aig_val_vit_0_210

    if(aig_val_trm_0_2500 != old_aig_val_trm_0_2500):
        angle = 245.0 - (245.0-(-75.0))*(aig_val_trm_0_2500/2500.0)
        x = int( float(x_aig_trm) + float(d_aig_trm) * cos(radians(angle)) )
        y = int( float(y_aig_trm) - float(d_aig_trm) * sin(radians(angle)) )
        my_canvas.coords(line_aig_trm, x_aig_trm, y_aig_trm, x, y)
        old_aig_val_trm_0_2500 = aig_val_trm_0_2500

    if(aig_val_rp_0_10 != old_aig_val_rp_0_10):
        angle = 225.0 - (225.0-(-60.0))*(aig_val_rp_0_10/10.0)
        x = int( float(x_aig_rp) + float(d_aig_rp) * cos(radians(angle)) )
        y = int( float(y_aig_rp) - float(d_aig_rp) * sin(radians(angle)) )
        my_canvas.coords(line_aig_rp, x_aig_rp, y_aig_rp, x, y)
        old_aig_val_rp_0_10 = aig_val_rp_0_10

    if(aig_val_cg_0_10 != old_aig_val_cg_0_10):
        angle = 225.0 - (225.0-(-60.0))*(aig_val_cg_0_10/10.0)
        x = int( float(x_aig_cg) + float(d_aig_cg) * cos(radians(angle)) )
        y = int( float(y_aig_cg) - float(d_aig_cg) * sin(radians(angle)) )
        my_canvas.coords(line_aig_cg, x_aig_cg, y_aig_cg, x, y)
        old_aig_val_cg_0_10 = aig_val_cg_0_10

    if(aig_val_cf_0_10 != old_aig_val_cf_0_10):
        angle = 220.0 - (220.0-(-45.0))*(aig_val_cf_0_10/10.0)
        x = int( float(x_aig_cf) + float(d_aig_cf) * cos(radians(angle)) )
        y = int( float(y_aig_cf) - float(d_aig_cf) * sin(radians(angle)) )
        my_canvas.coords(line_aig_cf, x_aig_cf, y_aig_cf, x, y)
        old_aig_val_cf_0_10 = aig_val_cf_0_10

    # robinet AU
    if(robinet_au != old_robinet_au):
        if(robinet_au==0):
            my_canvas.itemconfigure(line_au,state=HIDDEN)
        else:
            my_canvas.itemconfigure(line_au,state=NORMAL)
        old_robinet_au = robinet_au

    # avertisseur
    if(manette_klaxon != old_manette_klaxon):
        old_manette_klaxon = manette_klaxon
        if(manette_klaxon==KL_AV):
            x = xav_klaxon;
            y = yav_klaxon
        elif(manette_klaxon==KL_AR):
            x = xarr_klaxon
            y = yarr_klaxon
        else:
            x = x0_klaxon
            y = y0_klaxon
        my_canvas.coords(line_cmd_klaxon, xc_klaxon, yc_klaxon, x, y)
        my_canvas.coords(oval_cmd_klaxon, x-20, y-20, x+20, y+20)

    #lssf
    if(old_lssf!=lssf or lssf==LSSF_CLI):
        old_lssf=lssf
        aff = 1     #inverse des aures cabines car on masque
        if(lssf==LSSF_1):
            aff = 0
        if(lssf==LSSF_CLI and draw_update_cpt&4):
            aff = 0
        if(aff == 0):
            my_canvas.itemconfigure(oval_lssf,state=HIDDEN)
        else:
            my_canvas.itemconfigure(oval_lssf,state=NORMAL)

    # palette service
    if(palette_service != old_palette_service):
        old_palette_service = palette_service
        if(palette_service == RETIRE):
            my_canvas.itemconfigure(line_cmd_srv,state=HIDDEN)
            my_canvas.itemconfigure(line2_cmd_srv,state=HIDDEN)
        else:
            my_canvas.itemconfigure(line_cmd_srv,state=NORMAL)
            my_canvas.itemconfigure(line2_cmd_srv,state=NORMAL)

        if(palette_service == NEUTRE):
            my_canvas.coords(line_cmd_srv, xc_srv, yc_srv, x_neutre-5, y_neutre)
            my_canvas.coords(line2_cmd_srv, xc_srv, yc_srv, x_neutre+5, y_neutre)
        if(palette_service == SERVICE):
            my_canvas.coords(line_cmd_srv, xc_srv, yc_srv, x_srv-5, y_srv)
            my_canvas.coords(line2_cmd_srv, xc_srv, yc_srv, x_srv+5, y_srv)
        if(palette_service == ISO):
            my_canvas.coords(line_cmd_srv, xc_srv, yc_srv, x_iso-5, y_iso)
            my_canvas.coords(line2_cmd_srv, xc_srv, yc_srv, x_iso+5, y_iso)

    # palette sens
    if(palette_sens != old_palette_sens):
        old_palette_sens = palette_sens
        if(palette_sens == AV):
            my_canvas.coords(line_cmd_sens, xc_sens, yc_sens, xav_sens-5, yav_sens)
            my_canvas.coords(line2_cmd_sens, xc_sens, yc_sens, xav_sens+5, yav_sens)
        if(palette_sens == ARRET):
            my_canvas.coords(line_cmd_sens, xc_sens, yc_sens, x0_sens-5, y0_sens)
            my_canvas.coords(line2_cmd_sens, xc_sens, yc_sens, x0_sens+5, y0_sens)
        if(palette_sens == AR):
            my_canvas.coords(line_cmd_sens, xc_sens, yc_sens, xarr_sens-5, yarr_sens)
            my_canvas.coords(line2_cmd_sens, xc_sens, yc_sens, xarr_sens+5, yarr_sens)

    # manette frein automatique
    if(manette_frein != old_manette_frein):
        old_manette_frein = manette_frein
        if(manette_frein==DESS):
            my_canvas.coords(line_cmd_fr, xc_boule_fr, yc_boule_fr, xdess_boule_fr, ydess_boule_fr)
            my_canvas.coords(oval_cmd_fr, xdess_boule_fr-20, ydess_boule_fr-20, xdess_boule_fr+20, ydess_boule_fr+20)
        if(manette_frein==RIEN):
            my_canvas.coords(line_cmd_fr, xc_boule_fr, yc_boule_fr, x0_boule_fr, y0_boule_fr)
            my_canvas.coords(oval_cmd_fr, x0_boule_fr-20, y0_boule_fr-20, x0_boule_fr+20, y0_boule_fr+20)
        if(manette_frein==SERR):
            my_canvas.coords(line_cmd_fr,  xc_boule_fr, yc_boule_fr, xserr_boule_fr, yserr_boule_fr)
            my_canvas.coords(oval_cmd_fr, xserr_boule_fr-20, yserr_boule_fr-20, xserr_boule_fr+20, yserr_boule_fr+20)

    # manette acceleration
    if(old_manette_acc_0_100 != manette_acc_0_100):
        old_manette_acc_0_100 = manette_acc_0_100
        x = x0_boule_acc - (x0_boule_acc - x100_boule_acc) * manette_acc_0_100 / 100
        y = y0_boule_acc - (y0_boule_acc - y100_boule_acc) * manette_acc_0_100 / 100
        my_canvas.coords(line_cmd_acc, xc_boule_acc, yc_boule_acc, x, y)
        my_canvas.coords(oval_cmd_acc, x-20, y-20, x+20, y+20)

    # bouton signalisation (phares)
    if(old_phares != phares):
        old_phares = phares
        if(phares == 0):
            my_canvas.itemconfigure(rect_phares,state=HIDDEN)
        else:
            my_canvas.itemconfigure(rect_phares,state=NORMAL)

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# SIMU FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

# les parametres de simulations sont maintenant au debut du programme
vit_sens = 0.0    #vit mais avec un signe +=AV, -=AR

def simu():
    global vit; global vit_sens; global mem_sens
    global aig_val_vit_0_210
    global pression_cg_0_5; global pression_cf_0_5; global fr_0_100; global fr_0_100
    global aig_val_rp_0_10; global aig_val_cg_0_10; global aig_val_cf_0_10
    global robinet_au; global lssf_dcpt
    global dbg_txt

    if(lssf == LSSF_CLI):
        if(lssf_dcpt==0):
            robinet_au = 1
        else:
            lssf_dcpt -= 1

    v_cible = vmax * float(manette_acc_0_100) / 100.0
    #vit = v_cible
    t_cible = 0

    #marche sur l'erre
    if(manette_acc_0_100==0 or motor_on==0):  #debraye, marche sur l'erre
        dv = (100.0/t_100_0_erre)/10
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        aig_val_vit_0_210 = vit
        t_cible = (vit / 100.0) * t_100_0_erre

    #acceleration
    elif(v_cible>vit and palette_sens!=ARRET):
        dv = (100.0/t_0_100_mot)/10.0     * float(manette_acc_0_100) / 100.0
        v=vit+dv
        if(v>v_cible):
            vit=v_cible
        else:
            vit=v
        aig_val_vit_0_210 = vit
        t_cible = (abs(vit-v_cible) / 100.0) * t_0_100_mot * float(manette_acc_0_100) / 100.0

    #frein moteur
    elif(v_cible<vit):
        dv = (100.0/t_100_0_mot)/10.0     #* float(manette_acc_0_100) / 100.0
        v=vit-dv
        if(v<v_cible):
            vit=v_cible
        else:
            vit=v
        aig_val_vit_0_210 = vit
        t_cible = (abs(vit-v_cible) / 100.0) * t_100_0_mot

    #freinage pneumatique
    #manette frein -> cg
    if(manette_frein == SERR):
        if(palette_service == SERVICE or palette_service == NEUTRE):
            dp = (1.5 / t_press_3b5_5b) / 10.0
            p = pression_cg_0_5 - dp
            if(p < 0.0):
                p = 0.0
            pression_cg_0_5 = p
    if(manette_frein == DESS):
        if(palette_service == SERVICE):
            dp = (1.5 / t_press_3b5_5b) / 10.0
            p = pression_cg_0_5 + dp
            if(p > 5.0):
                p = 5.0
            pression_cg_0_5 = p

    #robinet au -> cg
    if(robinet_au):
        dp = (1.5 / t_press_au_5b_3b5) / 10.0
        p = pression_cg_0_5 - dp
        if(p < 0.0):
            p = 0.0
        pression_cg_0_5 = p

    #cg=5 -> cf=0, cg=3.5 -> cf=5
    p =  (5.0 / 1.5) * (5.0 - pression_cg_0_5)
    if(p > 5.0):
        p = 5.0
    if(p < 0.0):
        p = 0.0
    pression_cf_0_5 = p

    fr_0_100 = 100.0 * (pression_cf_0_5 / 5.0)
    aig_val_rp_0_10 = pression_rp_0_10
    aig_val_cg_0_10 = pression_cg_0_5
    aig_val_cf_0_10 = pression_cf_0_5

    if(fr_0_100>0.0):
        dv = (100.0/t_100_0_fr)/10.0    *   fr_0_100 / 100.0
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        aig_val_vit_0_210 = vit
        t_cible = (vit / 100.0) * t_100_0_fr * fr_0_100 / 100.0
        v_cible = 0

    #vitesse combinee avec sens
    if(palette_sens==AV):
        vit_sens = vit
        mem_sens = AV
    if(palette_sens==AR):
        vit_sens = -vit
        mem_sens = AR
    if(palette_sens==ARRET):
        vit_sens = 0

    zetxt = "vit=" + str(round(vit_sens,2)) + " manette_acc=" + str(round(manette_acc_0_100,0)) + "% manette_fr=" + str(round(fr_0_100,0)) +"%"
    if(fr_0_100==0.0):
        zetxt += " vcible=" + str(round(v_cible,2)) + " tcible=" + str(round(t_cible,2))
    elif(manette_acc_0_100==0):
        zetxt += " vcible=" + str(round(v_cible,2)) + " tcible=" + str(round(t_cible,2))

    if(use_bv_0_cph_1==0):
        zetxt += " BV8=" + str(bv_0_8)
    else:
        zetxt += " CPH2=" + str(cph_0_2)
    zetxt += " RPM=" + str(round(aig_val_trm_0_2500,0))
    zetxt += " X=" + str(x_press)
    zetxt += " Y=" + str(y_press)

    print(zetxt)
    my_canvas.itemconfigure(dbg_txt,text=zetxt)

    return


# coupleur hydraulique:
#   transition entre basse et hautes vitesses 72km/h
#   cph_0_2 = 0  pas de couplage
#   cph_0_2 = 1  couplage basses vitesses
#   cph_0_2 = 2  couplage hautes vitesses
# boite vitesse 8 rapports
#   bv_0_8 = 0
#   bv_0_8 = 1   1er rapport
#   ...
#   bv_0_8 = 8   8eme rapport
#   pour simplifier les rapports sont constants
#   alors qu'en realite ils respectent la table suivante
#   bv_0_8 = 1    0-15   au lieu de 0-20
#   bv_0_8 = 2   15-30   au lieu de 20-26
#   bv_0_8 = 3   30-45   au lieu de 26-34
#   bv_0_8 = 4   45-60   au lieu de 34-45
#   bv_0_8 = 5   60-75   au lieu de 45-58
#   bv_0_8 = 6   75-90   au lieu de 58-74
#   bv_0_8 = 7   90-105  au lieu de 74-94   (pas sure du 94)
#   bv_0_8 = 8  105-120  au lieu de 94-120
def simu_rpm():
    global aig_val_trm_0_2500; global slow_aig_val_trm_0_2500
    global bv_0_8
    global cph_0_2

    if(motor_on==0):
        slow_aig_val_trm_0_2500 = 0
        cph_0_2 = 0
        bv_0_8 = 0

    elif(palette_sens == ARRET):
        slow_aig_val_trm_0_2500 = rpm_ralenti
        cph_0_2 = 0
        bv_0_8 = 0

    elif(manette_acc_0_100==0):
        slow_aig_val_trm_0_2500 = rpm_ralenti
        cph_0_2 = 0
        bv_0_8 = 0

    elif(use_bv_0_cph_1 == 1):
        if(vit<72.0):
            cph_0_2 = 1
            rpm = (vit/72.0) * rpm_nominal
        else:
            cph_0_2 = 2
            rpm = (vit/vmax) * rpm_nominal
        if(rpm < rpm_ralenti):   #ds ce cas, le couplage n'est pas optimal
            rpm = rpm_ralenti
        slow_aig_val_trm_0_2500 = rpm

    elif(use_bv_0_cph_1 == 0):
        bv_0_8 = int(vit / (vmax/8)) + 1     #simplification de la table des vitesses
        if(bv_0_8 > 8):
            bv_0_8 = 8
        if(bv_0_8 < 1):
            bv_0_8 = 1
        rpm = vit / (float(bv_0_8) * (vmax/8)) * rpm_nominal
        if(rpm < rpm_ralenti):   #ds ce cas, l'embrayage patine
            rpm = rpm_ralenti
        slow_aig_val_trm_0_2500 = rpm

    #pour que l'aiguille bouge lentement surtaout au l'allumage et arret du moteur
    if(slow_aig_val_trm_0_2500 > aig_val_trm_0_2500):
        aig_val_trm_0_2500 += slow_aig_trm_inc
        if(aig_val_trm_0_2500 > slow_aig_val_trm_0_2500):
            aig_val_trm_0_2500 = slow_aig_val_trm_0_2500
    if(slow_aig_val_trm_0_2500 < aig_val_trm_0_2500):
        aig_val_trm_0_2500 -= slow_aig_trm_inc
        if(aig_val_trm_0_2500 < slow_aig_val_trm_0_2500):
            aig_val_trm_0_2500 = slow_aig_val_trm_0_2500

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# SERIAL FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

#mise a jour de l'arduino avec les nouvelles valeurs
#
# protocol:
# s s s s s s s s p <0..sens> <vit0-255> k <0..KL_AV.KL_AR> 0         e
#                                                           |<ana0>   |<e0> <e1>
#
#  e0:   0 AU FRD-DESS FRD-SERR FRA-DESS FRA-SERR SENS_AV SENS-AR
MSK_AU       = 64
MSK_FRD_DESS = 32
MSK_FRD_SERR = 16
MSK_FRA_DESS = 8
MSK_FRA_SERR = 4
MSK_SENS_AV  = 2
MSK_SENS_AR  = 1
#  e1:   0  0        0        0        0        0  KL_AV    KL_AR
MSK_KL_AV = 2
MSK_KL_AR = 1
#  a0: 0-255 -> acc

first = 1; old_e1 = 0; old_e0 = 0; old_a0 = 0

def serial_update():
    global ser; global serial_ok
    global manette_acc_0_100; global palette_sens; global manette_klaxon
    global robinet_au; global manette_freind; global manette_frein;
    global first; global old_e1; global old_e0; global old_a0

    if(serial_ok==0):
        return

    try:
        nb_rx=0
        while(ser.inWaiting()):
            nb_rx+=1
            rb = ser.read(1)
            if(nb_rx==1): a0 = rb[0]
            if(nb_rx==2): e0 = rb[0]
            if(nb_rx==3): e1 = rb[0]
            #print("RX:" + str(rb[0]))

        if(nb_rx==3 and first==0):

            if(abs(old_a0 - a0)>5):
                manette_acc_0_100 = 100 * a0 / 255
                if(a0<5):
                    manette_acc_0_100 = 0
                if(a0>250):
                    manette_acc_0_100 = 100
                old_a0 = a0

            e0 ^= 255; e1 ^= 255;

            if((old_e0 & MSK_AU)==0 and (e0 & MSK_AU)):
                robinet_au ^= 1

            if(e0 & MSK_SENS_AV):
                palette_sens = AV
            elif(e0 & MSK_SENS_AR):
                palette_sens = AR
            elif(old_e0 & MSK_SENS_AV):
                palette_sens = ARRET
            elif(old_e0 & MSK_SENS_AR):
                palette_sens = ARRET

            if(e0 & MSK_FRA_DESS):
                manette_frein = DESS
            elif(e0 & MSK_FRA_SERR):
                manette_frein = SERR
            elif(old_e0 & MSK_FRA_DESS):
                manette_frein = RIEN
            elif(old_e0 & MSK_FRA_SERR):
                manette_frein = RIEN

            if(e0 & MSK_FRD_DESS):
                manette_freind = DESS
            elif(e0 & MSK_FRD_SERR):
                manette_freind = SERR
            elif(old_e0 & MSK_FRD_DESS):
                manette_freind = RIEN
            elif(old_e0 & MSK_FRD_SERR):
                manette_freind = RIEN

            if(e1 & MSK_KL_AV):
                manette_klaxon = KL_AV
            elif(e1 & MSK_KL_AR):
                manette_klaxon = KL_AR
            elif(old_e1 & MSK_KL_AV):
                manette_klaxon = KL_0
            elif(old_e1 & MSK_KL_AR):
                manette_klaxon = KL_0

            old_e1 = e1
            old_e0 = e0

        first = 0

        # 1) synchros
        data = ord('s')
        for i in range(0,8):
            ser.write(bytes([data]))

        # 2) pwm command (sens + vit)
        ser.write(bytes([ord('p')]))
        #sens
        if(mem_sens == AV):
            sens = 1      #sens +
        else:
            sens = 0      #sens -
        sens ^= pwm_inv_sens
        ser.write(bytes([sens]))
        #print("TX:"+str(sens))
        #vit
        if(vit==0.0):
            v = 0
        else:
            v = pwm_min + int(float(pwm_max - pwm_min) * (vit / pwm_vit_max))
            if(v > pwm_max):
                v = pwm_max
        ser.write(bytes([v]))
        #print("TX:"+str(v))

        # 3) dcc command (adr + sens + vit + fct)
        ser.write(bytes([ord('d')]))
        #adresse
        ser.write(bytes([dcc_adr]))
        #sens
        if(mem_sens == AV):
            sens = 1      #sens +
        else:
            sens = 0      #sens -
        sens ^= dcc_inv_sens
        ser.write(bytes([sens]))
        #vit
        if(vit==0.0):
            v = 0
        else:
            v = dcc_min + int(float(dcc_max - dcc_min) * (vit / dcc_vit_max))
            if(v > dcc_max):
                v = dcc_max
        ser.write(bytes([v]))
        # fct (F0 si le bouton signalisation est actif)
        fct = 0
        if(phares):
            fct = 1
        ser.write(bytes([fct]))

        # 3) cmd klaxon (plus utilise car le timer 2 est utilise par le DCC)
        # ser.write(bytes([ord('k')]))
        # data = 0
        # if(manette_klaxon == KL_AV):  # 0.0.0.0.0.0.KLAV.KLAR
        #    data |= 2
        # if(manette_klaxon == KL_AR):  # 0.0.0.0.0.0.KLAV.KLAR
        #     data |= 1
        # ser.write(bytes([data]))
        #print("TX:"+str(data))

        # 4) read analog 0
        ser.write(bytes([ord('0')]))

        # 5) read boutons
        ser.write(bytes([ord('e')]))

    except:
        serial_ok=0
        print("ERROR: Perte de la liaison serie")

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# NETWORK FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

def net_connect():
    if(use_network==0):
        return

    if(use_local):
        adr = socket.gethostbyname(socket.gethostname())
    else:
        adr = server_ip_adr

    s.connect((adr, server_port))           #uu: mettre une touche d'inversion ex: I
    print("NETWORK: connected")
    welcome = s.recv(1024).decode()
    print("NETWORK: server string: \"" + welcome + "\"")
    s.send("cabine netv1 ead".encode())
    return

net_connect()

def net_disconnect():
    if(use_network==0):
        return
    s.close()
    return

old_msg = ""

def net_update():
    global old_msg

    if(use_network==0):
        return

    # adr
    msg = "a" + str(dcc_adr)

    # sens
    if(mem_sens == AV):
        sens = 1      #sens +
    else:
        sens = 0      #sens -
    sens ^= dcc_inv_sens
    if(sens):
        msg += " s+"
    else:
        msg += " s-"

    # vit
    if(vit==0.0):
        v = 0
    else:
        v = net_min + int(float(net_max - net_min) * (vit / dcc_vit_max))
        if(v > net_max):
            v = net_max
    msg += " v" + str(v)

    # fct (F0 si au moins un des boutons des phares est actif)
    fct = 0
    if(phares):
        fct = 1
    if(fct):
        msg += " f0+"
    else:
        msg += " f0-"

    msg += "\n"

    # send msg
    if(msg != old_msg):
        print("NETWORK: send: " + msg)
        s.send(msg.encode())
        old_msg = msg

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# CALLBACKS
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

cpt_avr = 0
cpt_err = 0

def timer_1s_cb():
    global cpt_avr
    global aig_val_vit_0_210; global aig_val_trm_0_2500
    global aig_val_rp_0_10; global aig_val_cg_0_10; global aig_val_cf_0_10
    cpt_avr += 1
    ##print("cpt_avr = " + str(cpt_avr));
    #aig_val_vit_0_210 = float(s_pot.get())

    simu()
    simu_rpm()

    serial_update()
    net_update()

    draw_update()


mouse_owner = ""
mem_manette_acc_0_100 = 0


def callback_my_canvas_mouse_press(event):
    global mouse_owner; global x_press; global y_press
    global motor_on
    global slow_aig_val_trm_0_2500
    global mem_manette_acc_0_100
    global lssf; global phares

    m_x = int(event.x); x_press = m_x
    m_y = int(event.y); y_press = m_y
    #print("mouse press at " + str(m_x) + " " + str(m_y))

    if(m_x>0 and m_x<150 and m_y>150 and m_y<300):
        mouse_owner = "robinet_au"

    if(m_x>x0_klaxon-20 and m_x<x0_klaxon+20 and m_y>y0_klaxon-20 and m_y<y0_klaxon+20):
        mouse_owner = "klaxon"

    if(m_x>276 and m_x<306 and m_y>189 and m_y<252):
        mouse_owner = "service"

    if(m_x>720 and m_x<760 and m_y>175 and m_y<264):
        mouse_owner = "sens"

    if(m_x>200 and m_x<274 and m_y>176 and m_y<284):
        mouse_owner = "frein"

    if(m_x>760 and m_x<884 and m_y>175 and m_y<321):
        mouse_owner = "acc"
        mem_manette_acc_0_100 = manette_acc_0_100

    if(m_x>588 and m_x<618 and m_y>135 and m_y<165):
        motor_on = 0
    if(m_x>628 and m_x<658 and m_y>135 and m_y<165):
        if(manette_acc_0_100==0):
            if(palette_sens == ARRET):
                if(palette_service != RETIRE):
                    motor_on = 1

    if(lssf == LSSF_CLI):
        if(m_x>xc_ack_lssf-d_ack_lssf and m_x<xc_ack_lssf+d_ack_lssf and m_y>yc_ack_lssf-d_ack_lssf and m_y<yc_ack_lssf+d_ack_lssf):
            lssf = LSSF_1

    if(m_x>xc_phares-10 and m_x<xc_phares+10 and m_y>yc_phares-10 and m_y<xc_phares+30):
        phares ^= 1

    return


def callback_my_canvas_mouse_move(event):
    global manette_frein
    global manette_acc_0_100
    global manette_klaxon
    m_x = int(event.x);
    m_y = int(event.y);

    if(mouse_owner=="frein"):
        if(m_y < y_press-10):     #desserage
            manette_frein=DESS
        elif(m_y > y_press+10):   #serrage
            manette_frein=SERR
        else:
            manette_frein=RIEN

    elif(mouse_owner=="acc"):
        dy = y_press - m_y   #positif ds le sens des accelerations
        dy_100 = y0_boule_acc - y100_boule_acc
        v = mem_manette_acc_0_100 + 100 * dy / dy_100
        if(v<0):
            v=0
        if(v>100):
            v=100
        manette_acc_0_100=v

    elif(mouse_owner == "klaxon"):
        if(m_y < y_press-10):
            manette_klaxon = KL_AV
        elif(m_y > y_press+10):
            manette_klaxon = KL_AR
        else:
            manette_klaxon = KL_0

    return


def callback_my_canvas_mouse_release(event):
    global robinet_au
    global palette_service; global palette_sens
    global mouse_owner
    global manette_frein
    global manette_acc_0_100
    global manette_klaxon

    m_x = int(event.x);
    m_y = int(event.y);

    if(mouse_owner=="robinet_au"):
        #robinet_au ^= 1
        if(m_x>x_press):
            robinet_au = 1
        else:
            robinet_au = 0

    elif(mouse_owner=="service"):
        if(m_y<y_press):
            v = palette_service
            if(v==RETIRE):
                v=ISO
            elif(v==ISO):
                v=SERVICE
            elif(v==SERVICE):
                v=NEUTRE
            palette_service = v
        else:
            v = palette_service
            if(v==NEUTRE):
                v=SERVICE
            elif(v==SERVICE):
                v=ISO
            elif(v==ISO):
                v=RETIRE
            palette_service = v

    elif(mouse_owner=="sens"):
        if(vit==0.0 and manette_acc_0_100==0):
            if(m_y<y_press):
                if(palette_sens == AR):
                    palette_sens = ARRET
                elif(palette_sens == ARRET):
                    palette_sens = AV
            else:
                if(palette_sens == AV):
                    palette_sens = ARRET
                elif(palette_sens == ARRET):
                    palette_sens = AR

    elif(mouse_owner=="frein"):
        manette_frein=RIEN

    elif(mouse_owner=="klaxon"):
        manette_klaxon=KL_0

    mouse_owner=""
    return


def callback_my_canvas_mouse_press_right(event):
    global palette_service;

    m_x = int(event.x);
    m_y = int(event.y);

    #if(m_x>276 and m_x<306 and m_y>189 and m_y<252):
    #    if(palette_service==RETIRE):
    #        palette_service=ISO
    #    else:
    #        palette_service=RETIRE

    return


def callback_my_canvas_mouse_release_right(event):
    return


# mapping du pave numerique
# 7 8 9  -->  DESS MON  ACC
# 4 5 6  -->  SERR MOFF DEC
# 1 2 3  -->  SRV^ AU   S^
# 0 0 .  -->  SRVv SRVv Sv
def cb_key(event):
    global manette_frein
    global manette_acc_0_100
    global palette_service
    global palette_sens
    global robinet_au
    global motor_on
    global slow_aig_val_trm_0_2500
    global vit
    global lssf; global lssf_dcpt
    global pwm_inv_sens
    global dcc_inv_sens

    k = repr(event.char)
    #print("pressed " + k)  #return 'a' with '

    if(k[1]=="8"):
        if(manette_acc_0_100==0):
            if(palette_sens == ARRET):
                if(palette_service != RETIRE):
                    motor_on = 1

    if(k[1]=="5"):
        motor_on = 0

    if(k[1]=="2"):
        robinet_au ^= 1

    if(k[1]=="3"):
        if(vit==0.0 and manette_acc_0_100==0):
            v = palette_sens
            if(v==AR):
                v=ARRET
            elif(v==ARRET):
                v=AV
            palette_sens = v

    if(k[1]=="."):
        if(vit==0.0 and manette_acc_0_100==0):
            v = palette_sens
            if(v==AV):
                v=ARRET
            elif(v==ARRET):
                v=AR
            palette_sens = v

    if(k[1]=="1"):
        v = palette_service
        if(v==RETIRE):
            v=ISO
        elif(v==ISO):
            v=SERVICE
        elif(v==SERVICE):
            v=NEUTRE
        palette_service = v

    if(k[1]=="0"):
        v = palette_service
        if(v==NEUTRE):
            v=SERVICE
        elif(v==SERVICE):
            v=ISO
        elif(v==ISO):
            v=RETIRE
        palette_service = v

    if(k[1]=="9"):
        v = manette_acc_0_100
        v = (v//5) * 5 + 5    #(v//5)*5 pour ne garder que multiples de 5
        if(v > 100):
            v = 100
        manette_acc_0_100 = v
    if(k[1]=="6"):
        v = manette_acc_0_100
        v = (v//5) * 5 - 5
        if(v < 0):
            v = 0
        manette_acc_0_100 = v

    if(k[1]=="7"):
        manette_frein=DESS
    if(k[1]=="4"):
        manette_frein=SERR

    if(k[1]=="l"):
        if(lssf == LSSF_0):
            lssf_dcpt = 50
            lssf = LSSF_CLI
        if(lssf == LSSF_1):
            lssf = LSSF_0

    if(k[1]==" "):    #arret immediat loco miniature avec la barre d'espace
        robinet_au = 1
        vit = 0

    if(k[1]=="i"):
        pwm_inv_sens ^= 1
        dcc_inv_sens ^= 1

    return


def cb_key_release(event):
    global manette_frein

    k = repr(event.char)
    #print("released " + k)

    if(k[1]=="7" or k[1]=="4"):
        manette_frein = RIEN

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# GUI management
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

top = Tk()
top.title(TITLE)

row = 0

#e_v_div_mbps = Entry(top)
#b_reset      = Button(top, text = "reset", command = reset_cb)
#l_sec        = Label(top, text = "0 sec", font=("Helvetica", 16), fg = "#800000")
#l_v_div.grid(row = row, column = 0, sticky = W + E + N + S)
#e_v_div_mbps.grid(row = row, column = 1, sticky = W + E + N + S)
#b_reset.grid(row = row, column = 3, sticky = W + E + N + S)
#l_sec.grid(row = row, column = 1, sticky = W + E + N + S)
#e_v_div_mbps.delete(0, 1000); e_v_div_mbps.insert(0, "100")

my_canvas = Canvas(top, bg = "#cccccc", height = 400, width = 960)
my_canvas.grid(row = row, column = 0, columnspan = 4)
row=row+1

#---------------------------------------------------------------------------------------

#l_pot = Label(top, text ="Pot")
##s_pot = Scale(top, from_=0, to=210, orient=HORIZONTAL, tickinterval=10)
##s_pot = Scale(top, from_=0, to=2500, orient=HORIZONTAL, tickinterval=100)
#s_pot = Scale(top, from_=0, to=10, orient=HORIZONTAL, tickinterval=1)
#s_pot.grid(row=row,column=0,sticky=W+E+N+S)
#s_pot.grid(row=row,column=1,columnspan=4,sticky=W+E+N+S)
#s_pot.set(0) #s_pot.get()
#row=row+1

#---------------------------------------------------------------------------------------

#l_0 = Label(top, text ="----------------------")
#l_1 = Label(top, text ="----------------------")
#l_2 = Label(top, text ="----------------------")
#l_3 = Label(top, text ="----------------------")

#l_0.grid(row = row, column = 0)
#l_1.grid(row = row, column = 1)
#l_2.grid(row = row, column = 2)
#l_3.grid(row = row, column = 3)
#row = row + 1

my_canvas.bind("<Button-1>"       , callback_my_canvas_mouse_press)
my_canvas.bind("<ButtonRelease-1>", callback_my_canvas_mouse_release)
my_canvas.bind("<B1-Motion>"       , callback_my_canvas_mouse_move)
my_canvas.bind("<Button-3>"        , callback_my_canvas_mouse_press_right)
my_canvas.bind("<ButtonRelease-3>", callback_my_canvas_mouse_release_right)
top.bind("<Key>", cb_key)
top.bind("<KeyRelease>", cb_key_release)



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# MAIN
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

if(serial_ok):
    try:
        ser = serial.Serial(SZ_SERIALPORT,115200,timeout=10,stopbits=serial.STOPBITS_TWO)
        ser.setDTR(False)
        print("INFO:  Ouverture du port serie " + SZ_SERIALPORT + " OK")
    except:
        serial_ok=0
        print("ERROR: Ne peut pas ouvrir le port serie " + SZ_SERIALPORT)

#myFont = Font(family="Times New Roman", size=12)
#text.configure(font=myFont)
#myFont.configure(size=14)
draw_first()
draw_update()

# Thread on enumeration checking
daemon_bool = True
def daemon_func():
    global cpt_err
    next_call = time.time()
    while daemon_bool:
        print("time="+str(datetime.datetime.now())+" cpt_err="+str(cpt_err)+" x="+str(x_press)+" y="+str(y_press))
        try:
            timer_1s_cb()
            next_call += 0.1
            sleep_duration = next_call - time.time()
            if(sleep_duration <= 0):
                print("ERROR: non accurate time !!")
                cpt_err += 1
                #time.sleep(1)
                next_call = time.time()
            else:
                time.sleep(sleep_duration)
        except RuntimeError:
            None


daemon_thread = threading.Thread(target=daemon_func)
daemon_thread.daemon = True
daemon_thread.start()

top.mainloop()

if(serial_ok):
    ser.close()
daemon_bool = False
print("ByeBye")
time.sleep(1)



# QUELQUES NOTES SUR LA PWM POUR ARDUINO UNO R3:

# /*
# * T0A --> PWM D6
# * T0B --> PWM D5
# * T1A --> PWM D9
# * T1B --> PWM D10
# * T2A --> PWM D11
# * T2B --> PWM D3
# *
# * analogWrite(pin, value);  // 0->0% 1->2/256% 255->256/256%
# * T0 aussi utilise pour millis ... (pas de pb pour utiliser pwm en //)
# * T2 peut etre utilise par tone (dans ce cas pas de pwm avec T2)
# *     tone(pin,value,[duration ms])  //31Hz-65kHz
# *     notone()
# * T1 peut etre utilise par servo (dans ce cas pas de pwm avec T1) servos sur 12 pins 2-13
# *     #include <Servo.h>
# *     Servo aServo  or   Servo tServo[12]
# *     aServo.attach(pin_number)
# *     aServo.writeMicroseconds(uS) // aServo.write(0-180)
# *     plus besoin d'utiliser refresh
# */

# EXEMPLE DE CODE ARDUINO DE BASE supportant juste PWM (sens et vitesse)
# QUI FONCTIONNE AVEC CETTE CABINE:
# Pour le code plus complet avec DCC et manette, regarder le fichier cabine.ino le plus recent
#
# #define PIN_PWM  6
# #define PIN_SENS 5
#
# void setup()
# {
#    pinMode(PIN_PWM , OUTPUT); digitalWrite(PIN_PWM , LOW);
#    pinMode(PIN_SENS, OUTPUT); digitalWrite(PIN_SENS, LOW);
#
#    pinMode(13, OUTPUT);
#    digitalWrite(13, HIGH);
#    Serial.begin(115200);
#    while (!Serial) { }
#    digitalWrite(13, LOW);
# }
#
# int step=0;
# int sens;
# int pwm_vit;
#
# void loop()
# {
#     int rx;
#
#     rx = Serial.read();
#     if(rx==-1) return;
#
#     if(step==0)
#     {
#         if(rx=='s') {}
#         if(rx=='p') step=1;
#         if(rx=='0') Serial.write(analogRead(A0)/4);   //0-1023->0-255
#         if(rx=='1') Serial.write(analogRead(A1)/4);   //0-1023->0-255
#         if(rx=='e') { Serial.write(0xff); Serial.write(0xff); } //not supported
#         if(rx=='k') step=3;
#         if(rx=='d') step=4;
#     }
#     else if(step==1)
#     {
#         sens = rx;
#         step = 2;
#     }
#     else if(step==2)
#     {
#         pwm_vit = rx;
#         step = 0;
#
#         if(sens)
#             digitalWrite(PIN_SENS, HIGH);
#         else
#             digitalWrite(PIN_SENS, LOW);
#
#         analogWrite(PIN_PWM, pwm_vit);
#     }
#     else if(step==3)
#     {
#         step = 0;
#     }
#     else if(step==4)
#     {
#          step = 5;
#     }
#     else if(step==5)
#     {
#          step = 6;
#     }
#     else if(step==6)
#     {
#          step = 7;
#     }
#     else if(step==7)
#     {
#          step = 0;
#     }
# }
