/* fdcc_cabine

    programme de la carte arduino pour les programmes fdcc_cabine
    
    2016-15-20: Ulysse Delmas-Begue: Ajout DCC prog
    2016-15-19: Ulysse Delmas-Begue: Creation (PWM & DCC)
*/

// connections:
#define PIN_AU        13
#define PIN_KL_AV     12
#define PIN_KL_AR     11
#define PIN_FRA_DESS  10
#define PIN_FRA_SERR   9
#define PIN_FRD_DESS   8
#define PIN_FRD_SERR   7
#define PIN_PWM        6
#define PIN_SENS       5
#define PIN_KL         4 //le buzzer du klaxon ne peut plus etre utilise car le timer 2 est utilise pour le DCC
#define PIN_SENS_AV    3
#define PIN_SENS_AR    2
// A0-A3: entrees analogiques
// A5: DCC
// A4: DCC#


/*--------------------------------------------------------------------------
1. Gestion du DCC
--------------------------------------------------------------------------*/

#include <avr/io.h>
#include <avr/interrupt.h> 

#define DCC_PHASE_0_L   0 // 0: __##
#define DCC_PHASE_0_L2  1
#define DCC_PHASE_0_H   2
#define DCC_PHASE_0_H2  3
#define DCC_PHASE_0_NEW 4            // pour calcul du prochain 1
#define DCC_PHASE_1_L   5 // 1: _#
#define DCC_PHASE_1_H   6
#define DCC_PHASE_1_NEW 7            // pour calcul du prochain 0

unsigned char dcc_next_port_val = 0;
unsigned char dcc_phase = DCC_PHASE_0_L;
unsigned char dcc_val = 0;
unsigned char dcc_val8 = 0;
unsigned char dcc_cpt_bit = 0;
unsigned char dcc_cpt_field = 0;
unsigned char dcc_bytes[4]; // 0-111111111-0-adr-0-cmd-0-xor-1

// ne pas oublier d'utiliser volatile si une variable est partagee entre loop et it
volatile unsigned char dcc_au    = 0; // non utilise
volatile unsigned char loc_adr   = 0; // 0 = pas de loco, transmission idle 
volatile unsigned char loc_vit   = 0; 
volatile unsigned char loc_fct   = 0;
volatile unsigned char frame_cpt = 0;

volatile unsigned char dcc_prog = 0; //mettre a 1 pour initier la prog (se remet a 0 automatiquement)
volatile unsigned int  cv_adr = 0;   //1-1024
volatile unsigned char cv_dat = 0;   //0-255
volatile unsigned char cv_pag = 0;   //1-255-0
volatile unsigned char cv_reg = 0;   //0-3

#define DCC_PROG_FRAME_IDLE  0
#define DCC_PROG_FRAME_RESET 1
#define DCC_PROG_FRAME_PAGE  2
#define DCC_PROG_FRAME_DAT   3
//22x idle 3x reset 6x page 10x reset 6x dat 10x reset 0xidle
unsigned char dcc_prog_seq[] = { \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
\
DCC_PROG_FRAME_PAGE, \
DCC_PROG_FRAME_PAGE, \
DCC_PROG_FRAME_PAGE, \
DCC_PROG_FRAME_PAGE, \
DCC_PROG_FRAME_PAGE, \
DCC_PROG_FRAME_PAGE, \
\
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
\
DCC_PROG_FRAME_DAT, \
DCC_PROG_FRAME_DAT, \
DCC_PROG_FRAME_DAT, \
DCC_PROG_FRAME_DAT, \
DCC_PROG_FRAME_DAT, \
DCC_PROG_FRAME_DAT, \
\
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
DCC_PROG_FRAME_RESET, \
};
#define PROG_SEQ_SIZE 35

int seq_ind = 0;
unsigned char preambule_len = 10;

inline void compute_new_frame_prog(void)
{
    dcc_bytes[0] = 0xff;
    preambule_len = 22;

    switch(dcc_prog_seq[seq_ind])
    {
        case DCC_PROG_FRAME_IDLE:        
                    dcc_bytes[1] = 0xff;
                    dcc_bytes[2] = 0;
                    break;
        case DCC_PROG_FRAME_RESET:        
                    dcc_bytes[1] = 0;
                    dcc_bytes[2] = 0;
                    break;
        case DCC_PROG_FRAME_PAGE:        
                    dcc_bytes[1] = 0x7D;  //0111CRRR C=1=WR RRR=5=page
                    //dcc_bytes[2] = ((cv_adr-1)/4)+1;  // cv1-4 = page1 ...
                    dcc_bytes[2] = cv_pag;
                    //dcc_bytes[1] = 0x7d;
                    //dcc_bytes[2] = 1;
                    break;
        case DCC_PROG_FRAME_DAT:        
                    //dcc_bytes[1] = 0x78 + (cv_adr-1)&3;  //0111CRRR C=1=WR RRR=0(cv1),1(cv2),2(cv3),3(cv4) ...
                    //dcc_bytes[2] = cv_dat;
                    dcc_bytes[1] = (unsigned char)0x78 + cv_reg;  //0111CRRR C=1=WR RRR=0(cv1),1(cv2),2(cv3),3(cv4) ...
                    dcc_bytes[2] = cv_dat;
                    //dcc_bytes[1] = 0x78;
                    //dcc_bytes[2] = 10;
                    break;
    }

    dcc_bytes[3] = dcc_bytes[1] ^ dcc_bytes[2]; 
              
    seq_ind++;
    if(seq_ind == PROG_SEQ_SIZE) 
    {
        dcc_prog--;
        seq_ind = 0; 
    }
}

inline void compute_new_frame(void)
{
                dcc_bytes[0] = 0xff;
                preambule_len = 14;
                
                if(dcc_au) // Arret d'urgence
                {
                    dcc_bytes[1] = 0;    // broadcast: toutes les locos
                    dcc_bytes[2] = 0x41; // arret d'urgence
                }

                else if(loc_adr==0)   // idle (pas de loco selectionee)
                {
                    dcc_bytes[1] = 0xff; //bourrage
                    dcc_bytes[2] = 0;        
                }
                
                else if((frame_cpt&1)==0)  // vit
                {
                     dcc_bytes[1] = loc_adr;
                     dcc_bytes[2] = loc_vit;
                }

                else // fct
                {
                     dcc_bytes[1] = loc_adr;
                     dcc_bytes[2] = loc_fct;
                }

                dcc_bytes[3] = dcc_bytes[1] ^ dcc_bytes[2];
                
                frame_cpt += 1;
                
                //Serial.println("");
                //Serial.print((int)dcc_bytes[0]); Serial.print("-");
                //Serial.print((int)dcc_bytes[1]); Serial.print("-");
                //Serial.print((int)dcc_bytes[2]); Serial.print("-");
                //Serial.print((int)dcc_bytes[3]); Serial.print("-");
}


inline void dcc_it(void)
{
    // Mise a jour du port avec l'etat DCC calcul precedement
    // Fait ici pour avoir un timing precis
    //Serial.print((int)dcc_next_port_val);
    //PORTC = port_mouse | dcc_next_port_val;
    PORTC = dcc_next_port_val;

    // Calcul du prochin etat
    dcc_phase++;
    if(dcc_phase == DCC_PHASE_0_H || dcc_phase == DCC_PHASE_1_H) dcc_next_port_val = 0x20;

    if(dcc_phase == DCC_PHASE_0_NEW || dcc_phase == DCC_PHASE_1_NEW)
    {
        // calcul du prochain bit
        if(dcc_cpt_bit==0)
        {
            if(dcc_cpt_field == 0)
            {
                dcc_val = 1;   //pr transmettre 1 final (entre les 2 trames)

                // calcul de la nouvelle trame
                if(dcc_prog == 0)
                    compute_new_frame();
                else
                    compute_new_frame_prog();

                dcc_val8=0xff;
                dcc_cpt_bit = preambule_len;
                dcc_cpt_field = 1;
            }
            else
            {
                dcc_val = 0;  //inter trames
                dcc_cpt_bit = 8;
                dcc_val8 = dcc_bytes[dcc_cpt_field]; //0=premabule; 1=adr; 2=cmd; 3=xor
                dcc_cpt_field++;
                if(dcc_cpt_field == 4) { dcc_cpt_field = 0; /*dcc_cpt_bit++;*/ }
            }
            //Serial.println("");
            //Serial.print((int)dcc_val8);
            //Serial.print(">");

        }
        else
        {
            //dcc_val = dcc_val8 & 1;    //LSB transmit first
            //dcc_val8 = dcc_val8 >> 1;
            if(dcc_val8 & 0x80) dcc_val = 1; else dcc_val = 0;   //MSB transmit first
            dcc_val8 = (dcc_val8 << 1) | 1;  // |1 pour les preambules > 10 bits et apres CC 
            dcc_cpt_bit--;
        }

        if(dcc_val) dcc_phase = DCC_PHASE_1_L; else dcc_phase = DCC_PHASE_0_L;
        dcc_next_port_val = 0x10;
   }    
    
}


// sens 1=AV, 0=AR
// vit 0--28
void update_loc_sens_vit_fct(unsigned char loc, unsigned char sens, unsigned char vit, unsigned char f43210)
{
    unsigned char data;
    
    loc_adr = loc;
    
    // 0 1 D V0 V4 V3 V2 V1
    // 0 1 x  0  0  0  0  0  = cran0
    // 0 1 x  0  0  0  0  1  = AU
    // 0 1 x  0  0  0  1  0  = cran1
    // ...
    // 0 1 x  1  1  1  1  1  = cran28
    data = 0x40;  
    if(sens)
    {
        data |= 0x20;
    }    
    if(vit >= 1)
    {
        data |= ((vit + 3) >> 1);
        if((vit & 1) == 0)
            data |= 0x10;
    }
    loc_vit = data;

    // 1 0 0 F0 F4 F3 F2 F1
    data = 0x80;
    if(f43210 & 1)
        data |= 0x10;
    data |= (f43210 >> 1);
    loc_fct = data;
   
}


/*--------------------------------------------------------------------------
2. Gestion du timer 2 (pour le DCC)
--------------------------------------------------------------------------*/

unsigned int cpt58 = 0;
unsigned int cpt100ms = 0;
volatile unsigned int flag100ms = 0;
unsigned int in_it = 0;
volatile unsigned int err_it = 0;

// timer 2 interrupt (58us)
ISR( TIMER2_COMPA_vect ){
  TCNT2 = 0; // Reset Timer2 counter ...
  
  if(in_it) err_it++;
  
  dcc_it();
  
  cpt58++;
  if(cpt58 == (100000 / 58)) { cpt58 = 0; /*cpt100ms++;*/ flag100ms = 1; /*Serial.print("*");*/}
  in_it = 0;
}

#define TIMER2_TARGET 114
byte timer2_target = TIMER2_TARGET;  //UU mettre ds la fct

// init du timer pour generer une IT toute les 58us
//  Prescaler: set to divide-by-8 (B'010)
//  Compare target: 58us / ( 1 / ( 16MHz/8) ) = 116
void timer_init() 
{
  // Set prescaler to div-by-8
  bitClear(TCCR2B, CS22);
  bitSet(TCCR2B, CS21);
  bitClear(TCCR2B, CS20);
  
  // Set counter target
  OCR2A = timer2_target;
  //OCR2A = (byte)TIMER2_TARGET;   //uu: verifier si ca marche !
    
  // Enable Timer2 interrupt
  bitSet(TIMSK2, OCIE2A); 
}


/*--------------------------------------------------------------------------
3. programme principal: setup et boucle PWM & DCC
--------------------------------------------------------------------------*/

// init
void setup()
{
   // entrees (ecriture a 1 pour activer les pull-up)
   pinMode(PIN_KL_AV   , INPUT); digitalWrite(PIN_KL_AV   , HIGH);
   pinMode(PIN_KL_AR   , INPUT); digitalWrite(PIN_KL_AR   , HIGH);
   pinMode(PIN_AU      , INPUT); digitalWrite(PIN_AU      , HIGH);
   pinMode(PIN_FRA_SERR, INPUT); digitalWrite(PIN_FRA_SERR, HIGH);
   pinMode(PIN_FRA_DESS, INPUT); digitalWrite(PIN_FRA_DESS, HIGH);
   pinMode(PIN_FRD_SERR, INPUT); digitalWrite(PIN_FRD_SERR, HIGH);
   pinMode(PIN_FRD_DESS, INPUT); digitalWrite(PIN_FRD_DESS, HIGH);
   pinMode(PIN_SENS_AV , INPUT); digitalWrite(PIN_SENS_AV , HIGH);
   pinMode(PIN_SENS_AR , INPUT); digitalWrite(PIN_SENS_AR , HIGH);

   // sorties PWM
   pinMode(PIN_PWM , OUTPUT); digitalWrite(PIN_PWM , LOW);
   pinMode(PIN_SENS, OUTPUT); digitalWrite(PIN_SENS, LOW);
   //pinMode(PIN_KL  , OUTPUT); digitalWrite(PIN_KL  , LOW);

    // sorties DCC 
    PORTC = 0;
    DDRC = 0x30; //PORTC 0.0.DCC.DCC#.0.0.0.0
    timer_init();
    sei();  // Enable interrupts 

   // attente de l'ouverture du port par le PC   
   Serial.begin(115200);
   while (!Serial) 
   {
   }
   //tone(PIN_KL,440,50);  //pin, 31Hz-65kHz, ms
}

// envoie l'etat des entrees
void tx_entrees(void)
{
    int data;
    /*    0 AU FRD-DESS FRD-SERR FRA-DESS FRA-SERR SENS_AV SENS-AR
     *    0  0        0        0        0        0  KL_AV    KL_AR
     */
    data = 0;
    if(digitalRead(PIN_SENS_AR )) data|=1;
    if(digitalRead(PIN_SENS_AV )) data|=2;
    if(digitalRead(PIN_FRA_SERR)) data|=4;
    if(digitalRead(PIN_FRA_DESS)) data|=8;
    if(digitalRead(PIN_FRD_SERR)) data|=16;
    if(digitalRead(PIN_FRD_DESS)) data|=32;
    if(digitalRead(PIN_AU      )) data|=64;
    Serial.write(data);

    data = 0;
    if(digitalRead(PIN_KL_AR )) data|=1;
    if(digitalRead(PIN_KL_AV )) data|=2;
    Serial.write(data);
}

// boucle qui gere les requetes USB
int step=0;
int pwm_sens;
int pwm_vit;
int dcc_adr;
int dcc_sens;
int dcc_vit;
int dcc_fct;

void loop()
{
    int rx;
    
    rx = Serial.read();
    if(rx==-1) return;

    if(step==0)
    {
        if(rx=='s') {}
        if(rx=='p') step=1;
        if(rx=='0') Serial.write(analogRead(A0)/4);
        if(rx=='1') Serial.write(analogRead(A1)/4);
        if(rx=='2') Serial.write(analogRead(A2)/4);
        if(rx=='3') Serial.write(analogRead(A3)/4);
        if(rx=='4') Serial.write(analogRead(A4)/4);
        if(rx=='5') Serial.write(analogRead(A5)/4);
        if(rx=='e') tx_entrees();
        if(rx=='k') step=3;
        if(rx=='d') step=4;
        if(rx=='!') step=8;
    }
    else if(step==1)
    {
        pwm_sens = rx;
        step = 2; 
    }
    else if(step==2)     // 0.0.0.0.0.0.0.sens
    {
        pwm_vit = rx;
        step = 0; 

        if(pwm_sens&1)
            digitalWrite(PIN_SENS, HIGH);
        else
            digitalWrite(PIN_SENS, LOW);
            
        analogWrite(PIN_PWM, pwm_vit);
    }
    else if(step==3)    // 0.0.0.0.0.0.KLAV.KLAR
    {
        step = 0;   //plus de klaxon, le timer 2 est utilise pour le DCC
        //if(rx==0) noTone(PIN_KL);
        //if(rx==1) tone(PIN_KL,370);  //+/-10% en realite
        //if(rx==2) tone(PIN_KL,660);  //+/-15% en realite
    }
    else if(step==4)    // adr (0-99)
    {
        step = 5; 
        dcc_adr = rx;
    }
    else if(step==5)    // sens (0-1)
    {
        step = 6; 
        dcc_sens = rx;
    }
    else if(step==6)    // vit (0-28)
    {
        step = 7; 
        dcc_vit = rx;
    }
    else if(step==7)    // fct 0.0.0.F4.F3.F2.F1.F0
    {
        step = 0; 
        dcc_fct = rx;
        update_loc_sens_vit_fct((unsigned char) dcc_adr, (unsigned char)dcc_sens, (unsigned char)dcc_vit, (unsigned char)dcc_fct);
    }
    else if(step==8)
    {
        if(rx == 0x55) step=9; else step=0; 
    }
    else if(step==9)
    {
        if(rx == 0xAA) step=10; else step=0; 
    }
    else if(step==10)
    {
        cv_adr = rx;
        cv_adr = cv_adr << 8;
        cv_adr &= 0x300;
        step=11; 
    }
    else if(step==11)
    {
        cv_adr |= rx;
        step=12; 
    }
    else if(step==12)
    {
        cv_pag = ((cv_adr-1)/4)+1;
        cv_reg = (cv_adr-1)&3;
        cv_dat = rx;
        dcc_prog = 1;  //10 programing seq
        step=0; 
        //Serial.write(0x90);
        //Serial.write(cv_pag);
        //Serial.write(cv_reg);
    }        
}
