# Pupitre de RTG
#
# History:
# 2019/02/24: Delmas-Begue Ulysse: adaptation du reseau pour s'interfcaer avec D17
#                                  configuration par defaut pour D17
# 2017/03/22: Delmas-Begue Ulysse: rajout du reseau pour s'interfacer avec fdcc2017
# 2016/12/19: Delmas-Begue Ulysse: rajout DCC et phares
# 2016/12/10: Delmas-Begue Ulysse: sync avec etg prise en compte de la turmo12
# 2016/11/24: Delmas-Begue Ulysse: creation
#
# Notes:
# License: GPLv2
# Photo du pupitre: http://www.cheminots.net/forum/topic/25024-rtg-t-2000-turbotrain-sujet-officiel/?page=2
# This application works on Linux/Windows and Python3.x/2.x
#
# TBD
# - pente + (vent) ?
# - FDCC network
# - sonds
# - - S +
# - merger etg et rtg

TITLE = "FDCC Pupitre RTG v20190226 UDB"

use_serial    = 0
SZ_SERIALPORT = "\\\\.\\COM4"

use_network   = 1
use_local     = 0
server_ip_adr = "192.168.4.1"
server_port   = 1234

# caracteristiques (surement qq erreurs)
#
# type         | rtg
# caisses      | 5
# mot          | 2x Turmo III F1  (2x850kW=1700kW) / Turmo III F1 + Turmo XII  850kW+1200kW=2050kW (la TurmoIII est utilisee que lors de la mise en vitesse)
# aux          | 2x Astazou IV A  (2x320kW 400gr/kW) genere l'alimentation 380V (une seule turbine a la fois)
# transmission | Hydraulique (convertisseur de couple puis coupleur entre 65 et 150km/h, on prendra 140km/h)
# vmax         | 200kmph (160kmph en service commerciel)
# ralenti      | 20 000rpm (generateur de gaz)
# nominal max  | 32 000rpm
# reservoire   | 9400l (4x2350l)
# conso km     | 400gr/kW (~0.4L/kW) (TurmoIII, Aztazou IV), ~285gr/kW (si conso pour Puissance nominale == entre les 2 turbines)
# masse        | 225t  (approximation:5x45t)
# nb           | 41
# freinage     | sabot / disque / patins (sous2.5bars, +40km/h) / hydraulique(je sais pas ou est la manette sur le pupitre)
#
# couplage = 2 max

# pupitre:
#
# on/off:
# TurmoIII 1 ON, OFF, TurmoIII 2 ON, OFF, AstazouIV 1 ON, OFF, AstazouIV 2 ON, OFF
#
# pictos:
#  TurmoIII 1 | patinage 1 | patinage 2 | TurmoIII 2
#  700°C      | QT1        | QT2        | 700°C
#  380V       | CB         | PB INV     | volets clim
#
# boite a boutons:
# armement QT1 | QT2     | compresseur auto | direct | x     | x          | glaces chauffantes | volets clim | eclairage rame
# cabine       | horaire | pupitre          | x      | fanal | projecteur | projecteur         | essai VA    | maintient eclairage

# parametres pwm pour commander une locomotive via carte Arduino
pwm_vit_max    = 200  # vitesse correspondante a pwm_max
pwm_max        = 255  # valeur de la pwm(0-255) a la vitesse max
pwm_min        = 1    # valeur de la pwm(0-255) pour demarrer la loco
pwm_inv_sens   = 0    # invertion du sens (0=ne pas inverser, 1=inverser)

# parametres dcc pour commander une locomotive via carte Arduino
dcc_adr        = 3    # adresse de la locomotive
dcc_vit_max    = 200  # vitesse correspondante a pwm_max
dcc_max        = 28   # valeur du cran DCC 0-28 a la vitesse max
dcc_min        = 1    # valeur du cran DCC 0-28 pour demarrer la loco
dcc_inv_sens   = 0    # invertion du sens (0=ne pas inverser, 1=inverser)
net_max        = 100  # valeur a envoyer par le reseau a la vitesse max (100 pour D17, 28 pour fdcc2017)
net_min        = 1    # valeur a envoyer par le reseau pour demarrer la loco

# parametres de simulation (a adapter suivant les besoins)
vmax              = 200.0   # kmph
use_turmo12       = 1       # 0=2xturmo2 1=turmo12(mot1)+turmo3(mot2)
vmax_turmo12      = 260     # kmph (pour pouvoir atteindre 160kmph juste avec la turmo12)
t_0_100_mot       = 10.0    # s
t_100_0_mot       = 0.0     # s (frein moteur non disponible sur les turbines)
t_100_0_fr        = 5.0     # s
t_100_0_frd       = 10.0    # s
t_100_0_frhydro   = 0.0     # s (pas de frein hydraulique sur etg)
t_100_0_fr_patins = 2.5     # s (cg<2.5bars et v>50kmph)
t_100_0_erre      = 30.0    # s
t_press_3b5_5b    = 4.0     # s
t_press_au_5b_3b5 = 1.0     # s
t_press_0b_5b_fd  = 3.0     # s
rpm_ralenti       = 20000.0 # tr/min
rpm_nominal       = 32000.0 # tr/min
slow_aig_trm_inc  = 500.0   # tr/min /tic
rpm2_ralenti      = 20000.0 # tr/min
rpm2_nominal      = 32000.0 # tr/min
slow_aig_trm2_inc = 500.0   # tr/min /tic

# a l'init
motor_on  = 0       #turmo3 ou turmo12
motor2_on = 0       #turmo3
vit = 0.0                  # kmph (a l'init)
pression_rp_0_10  = 8.0    # bars (a l'init mais bouge pas)
pression_cg_0_5   = 0.0    # bars (a l'init)
pression_cfd_0_5  = 0.0    # bars (a l'init) frein direct
pression_cf_0_5   = 5.0    # bars (a l'init)
robinet_au = 0             # 0=ferme, 1=ouvert donc AU
manette_acc_0_100 = 0      # %
manette_acc2_0_100 = 0     # %

RETIRE   = 0
RETRAIT  = 1
ISO      = 2
NEUTRE   = 3
SERVICE  = 4
palette_service = RETIRE

RETIRE = 0 #redefine
CLE_OFF = 1
CLE_ON  = 2
cle = RETIRE

AV    = 1
ARRET = 0
AR    = 2
palette_sens = ARRET
mem_sens = AV

DESS = 2
RIEN = 0
SERR = 1
manette_frein = RIEN
manette_freind= RIEN

KL_AV  = 1
KL_0   = 0
KL_AR  = 2
manette_klaxon = KL_0

LSSF_0   = 0
LSSF_1   = 1
LSSF_CLI = 2
lssf = LSSF_0
lssf_dcpt = 0

phares = 0

# over write pour aller plus vite pendant le debug
palette_service   = SERVICE
pression_cg_0_5   = 5.0
pression_cf_0_5   = 0.0
cle               = CLE_ON


#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# IMPORT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

import os
import time
import sys
import subprocess
import threading
import datetime
import socket
try:
    # for Python 3
    from tkinter import *
    print("Python 3.x interpreter detected")
except ImportError:
    # for Python2
    from Tkinter import *
    print("Python 2.x interpreter detected")
from math import *

ser = 0
serial_ok = 1
if(use_serial):
    try:
        import serial
    except ImportError:
        serial_ok = 0
        print("Cannot load serial lib")
else:
    serial_ok = 0




#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# DRAW FCTs
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

# parametres du pupitre
fichier_pupitre = "rtg.png"
img_pupitre = 0
img_dx = 0
img_dy = 0

x_aig_vit  = 748; y_aig_vit  = 86; d_aig_vit  = 53
x_aig_trm  = 360; y_aig_trm  = 71; d_aig_trm  = 40
x_aig_trm2 = 611; y_aig_trm2 = 65; d_aig_trm2 = 40

x_aig_rp = 234; y_aig_rp = 72; d_aig_rp = 31
x_aig_cg = 234; y_aig_cg = 72; d_aig_cg = 31
x_aig_cf = 239; y_aig_cf = 172; d_aig_cf = 31

xc_boule_acc   = 712; yc_boule_acc   = 354
x0_boule_acc   = 850; y0_boule_acc   = 414
x100_boule_acc = 820; y100_boule_acc = 255

xc_boule_acc2   = 712; yc_boule_acc2   = 354
x0_boule_acc2   = 770; y0_boule_acc2   = 414
x100_boule_acc2 = 743; y100_boule_acc2 = 255

xc_boule_fr    = 110; yc_boule_fr    = 400
xdess_boule_fr = 120; ydess_boule_fr = 290
x0_boule_fr    = 100; y0_boule_fr    = 340
xserr_boule_fr = 80 ; yserr_boule_fr = 390

xc_boule_frd    = 30; yc_boule_frd    = 440
xdess_boule_frd = 40; ydess_boule_frd = 295
x0_boule_frd    = 20; y0_boule_frd    = 345
xserr_boule_frd = 0 ; yserr_boule_frd = 395

xc_sens   = 700; yc_sens   = 295
xav_sens  = 690; yav_sens  = 221
x0_sens   = 700; y0_sens   = 245
xarr_sens = 710; yarr_sens = 289

xc_srv   = 158; yc_srv   = 286; d_srv = 40
x_neutre = 299; y_neutre = 200
x_srv    = 286; y_srv    = 193
x_iso    = 277; y_iso    = 248

xc_klaxon   = 812; yc_klaxon   = 295
xav_klaxon  = 810; yav_klaxon  = 130
x0_klaxon   = 820; y0_klaxon   = 170
xarr_klaxon = 830; yarr_klaxon = 210

xc_au = 0; yc_au = 275
x_au = 60; y_au  = 325

xc_cle = 616; yc_cle = 490  #(la cle est en dehors du cadre et positionnee par defaut sur ON)
x0_cle = 660; y0_cle = 525
x1_cle = 616; y1_cle = 540

x_lssf       = 433; y_lssf = 45
xc_ack_lssf  = 433; yc_ack_lssf  = 93;
xc_ack2_lssf = 890; yc_ack2_lssf = 457;

line_aig_vit=0; line_aig_trm=0; line_aig_trm2=0; line_aig_rp=0; line_aig_cg=0; line_aig_cf=0
oval_au=0
line_cmd_srv=0; oval_cmd_srv=0; line_cmd_cle=0
line_cmd_sens=0; line2_cmd_sens=0
line_cmd_fr=0; oval_cmd_fr=0
line_cmd_frd=0; oval_cmd_frd=0
line_cmd_acc = 0; oval_cmd_acc = 0
line_cmd_acc2 = 0; oval_cmd_acc2 = 0
line_cmd_klaxon=0; oval_cmd_klaxon=0
oval_lssf = 0

zefont = ("Times New Roman", 6)
filename_pupitre = 0
dbg_txt = ""

rpm_txt=0; rpm2_txt=0

#creation du pupitre (image, aiguilles, manettes)
def draw_first():
    global filename_pupitre   #necessaire car si local, image perdue a la sortie de la fonction
    global dbg_txt
    global line_aig_vit; global line_aig_trm; global line_aig_trm2; global line_aig_rp; global line_aig_cg; global line_aig_cf
    global oval_au;
    global line_cmd_srv; global oval_cmd_srv; global line_cmd_cle
    global line_cmd_sens; global line2_cmd_sens
    global line_cmd_fr; global oval_cmd_fr
    global line_cmd_frd; global oval_cmd_frd
    global line_cmd_acc; global oval_cmd_acc
    global line_cmd_acc2; global oval_cmd_acc2
    global rpm_txt; global rpm2_txt
    global line_cmd_klaxon; global oval_cmd_klaxon
    global oval_lssf

    #my_canvas.create_rectangle(x, y, x + 100, y +200 , fill = "#000000"  , outline = "#ffffff")

    dbg_txt = my_canvas.create_text(10, img_dy+10, text = "DBG: .............", fill = "#ff0000", anchor = "nw")

    # image pupitre
    #filename_pupitre = PhotoImage(file = image_du_pupitre)
    image_pupitre = my_canvas.create_image(0, 0, image=img_pupitre, anchor = "nw")   #anchor=NE,

    # aiguilles
    line_aig_vit = my_canvas.create_line(0,0,0,0,width=5,fill="#ffffff")
    line_aig_trm = my_canvas.create_line(0,0,0,0,width=4,fill="#ffffff")
    line_aig_trm2= my_canvas.create_line(0,0,0,0,width=4,fill="#ffffff")
    line_aig_rp  = my_canvas.create_line(0,0,0,0,width=4,fill="#ff0000")
    line_aig_cg  = my_canvas.create_line(0,0,0,0,width=3,fill="#ffffff")
    line_aig_cf  = my_canvas.create_line(0,0,0,0,width=3,fill="#ffffff")
    # x10000 rpm
    rpm_txt = my_canvas.create_text(345, 72, text = "0", fill = "#ffffff", anchor = "center")
    rpm2_txt= my_canvas.create_text(596, 66, text = "0", fill = "#ffffff", anchor = "center")

    # avertisseur
    line_cmd_klaxon = my_canvas.create_line(0,0,0,0,width=7,fill="#444444")
    oval_cmd_klaxon = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    line_dbg_klaxon = my_canvas.create_line(x0_klaxon,y0_klaxon,xav_klaxon,yav_klaxon,width=1,fill="#00ff00")
    line2_dbg_klaxon= my_canvas.create_line(x0_klaxon,y0_klaxon,xarr_klaxon,yarr_klaxon,width=1,fill="#00ff00")

    # manette d'acceleration 1 (rouge)
    my_canvas.create_line(xc_boule_acc,yc_boule_acc,xc_boule_acc+40,yc_boule_acc,width=7,fill="#888888")
    line_cmd_acc = my_canvas.create_line(0,0,0,0,width=7,fill="#888888")
    oval_cmd_acc = my_canvas.create_oval(0,0,0,0, fill = "#880000", outline = "black")
    my_canvas.create_line(x0_boule_acc,y0_boule_acc,x100_boule_acc,y100_boule_acc,width=1,fill="#00ff00")

    # manette d'acceleration 2 (noire)
    line_cmd_acc2 = my_canvas.create_line(0,0,0,0,width=7,fill="#888888")
    oval_cmd_acc2 = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    my_canvas.create_line(x0_boule_acc2,y0_boule_acc2,x100_boule_acc2,y100_boule_acc2,width=1,fill="#00ff00")

    # manette de freinage
    line_cmd_fr = my_canvas.create_line(0,0,0,0,width=7,fill="#888888")
    oval_cmd_fr = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    my_canvas.create_line(x0_boule_fr,y0_boule_fr,xserr_boule_fr,yserr_boule_fr,width=1,fill="#00ff00")
    my_canvas.create_line(x0_boule_fr,y0_boule_fr,xdess_boule_fr,ydess_boule_fr,width=1,fill="#00ff00")

    # manette de freinage direct
    line_cmd_frd = my_canvas.create_line(0,0,0,0,width=27,fill="#222222")
    oval_cmd_frd = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    my_canvas.create_line(x0_boule_frd,y0_boule_frd,xserr_boule_frd,yserr_boule_frd,width=1,fill="#00ff00")
    my_canvas.create_line(x0_boule_frd,y0_boule_frd,xdess_boule_frd,ydess_boule_frd,width=1,fill="#00ff00")

    # manette de sens
    line_cmd_sens  = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    line2_cmd_sens = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    my_canvas.create_line(x0_sens,y0_sens,xav_sens,yav_sens,width=1,fill="#00ff00")
    my_canvas.create_line(x0_sens,y0_sens,xarr_sens,yarr_sens,width=1,fill="#00ff00")

    # manette de mise en service
    line_cmd_srv = my_canvas.create_line(0,0,0,0,width=7,fill="#888888", capstyle=ROUND)
    oval_cmd_srv = my_canvas.create_oval(0,0,0,0,fill="#111111")

    #cle
    line_cmd_cle = my_canvas.create_line(0,0,0,0,width=20,fill="#888888", capstyle=ROUND)

    # robinet d'arret d'urgence
    #line_au = my_canvas.create_line(xc_au,yc_au,x_au,y_au,width=10,fill="#ff0000")  #a masquer si pas au
    oval_au = my_canvas.create_oval(xc_au,yc_au,x_au,y_au,fill="#ff0000")  #a masquer si pas au

    #lssf
    oval_lssf = my_canvas.create_oval(x_lssf-12,y_lssf-12,x_lssf+12,y_lssf+12,fill="#ffdd00")

    return

# valeurs
aig_val_vit_0_210 = 0.0
aig_val_trm_0_32000 = 0.0
aig_val_trm2_0_32000 = 0.0
aig_val_rp_0_10 = 8.0
aig_val_cg_0_10 = 0.0
aig_val_cf_0_10 = 5.0

slow_aig_val_trm_0_32000 = 0.0
slow_aig_val_trm2_0_32000 = 0.0

old_aig_val_vit_0_210 = -1.0   #sauvegarde des valeurs entre 2 appels pour
old_aig_val_trm_0_32000 = -1.0 #ne pas redessiner et ainsi gagner en performances
old_aig_val_trm2_0_32000 = -1.0
old_aig_val_rp_0_10 = -1.0
old_aig_val_cg_0_10 = -1.0
old_aig_val_cf_0_10 = -1.0
old_robinet_au = -1
old_palette_sens = -1
old_palette_service=-1
old_manette_frein = -1
old_manette_freind= -1
old_manette_acc_0_100 = -1.0
old_manette_acc2_0_100 = -1.0
old_cle = -1
old_manette_klaxon = -1
old_lssf = -1
draw_update_cpt = 0

#mise a jour des aiguilles et position des manettes
def draw_update():
    global my_canvas
    global line_aig_vit; global line_aig_trm; global line_aig_trm2; global line_aig_rp; global line_aig_cg; global line_aig_cf
    global line_cmd_fr; global oval_cmd_fr; global oval_au
    global line_cmd_frd; global oval_cmd_frd;
    global line_cmd_acc; global oval_cmd_acc
    global line_cmd_acc2; global oval_cmd_acc2
    global line_cmd_srv; global oval_cmd_srv; global line_cmd_cle
    global old_aig_val_vit_0_210; global old_aig_val_trm_0_32000; global old_aig_val_trm2_0_32000
    global old_aig_val_rp_0_10; global old_aig_val_cg_0_10; global old_aig_val_cf_0_10
    global old_robinet_au;
    global old_palette_sens; global old_palette_service; global old_manette_frein; global old_manette_freind; global old_cle
    global old_manette_acc_0_100; global old_manette_acc2_0_100
    global rpm_txt; global rpm2_txt
    global old_manette_klaxon; global line_cmd_klaxon; global oval_cmd_klaxon
    global old_lssf
    global draw_update_cpt

    draw_update_cpt += 1

    #mise a jour des aiguilles
    if(aig_val_vit_0_210 != old_aig_val_vit_0_210):
        angle = 205.0 - (205.0-(-10.0))*(aig_val_vit_0_210/205.0)
        x = int( float(x_aig_vit) + float(d_aig_vit) * cos(radians(angle)) )
        y = int( float(y_aig_vit) - float(d_aig_vit) * sin(radians(angle)) )
        my_canvas.coords(line_aig_vit, x_aig_vit, y_aig_vit, x, y)
        old_aig_val_vit_0_210 = aig_val_vit_0_210

    if(aig_val_trm_0_32000 != old_aig_val_trm_0_32000):
        angle = 90 - 360 * (aig_val_trm_0_32000/10000)
        x = int( float(x_aig_trm) + float(d_aig_trm) * cos(radians(angle)) )
        y = int( float(y_aig_trm) - float(d_aig_trm) * sin(radians(angle)) )
        my_canvas.coords(line_aig_trm, x_aig_trm, y_aig_trm, x, y)
        old_aig_val_trm_0_32000 = aig_val_trm_0_32000
        my_canvas.itemconfigure(rpm_txt,text=int(aig_val_trm_0_32000/10000))

    if(aig_val_trm2_0_32000 != old_aig_val_trm2_0_32000):
        angle = 90 - 360 * (aig_val_trm2_0_32000/10000)
        x = int( float(x_aig_trm2) + float(d_aig_trm2) * cos(radians(angle)) )
        y = int( float(y_aig_trm2) - float(d_aig_trm2) * sin(radians(angle)) )
        my_canvas.coords(line_aig_trm2, x_aig_trm2, y_aig_trm2, x, y)
        old_aig_val_trm2_0_32000 = aig_val_trm2_0_32000
        my_canvas.itemconfigure(rpm2_txt,text=int(aig_val_trm2_0_32000/10000))

    if(aig_val_rp_0_10 != old_aig_val_rp_0_10):
        angle = 225.0 - (225.0-(-60.0))*(aig_val_rp_0_10/10.0)
        x = int( float(x_aig_rp) + float(d_aig_rp) * cos(radians(angle)) )
        y = int( float(y_aig_rp) - float(d_aig_rp) * sin(radians(angle)) )
        my_canvas.coords(line_aig_rp, x_aig_rp, y_aig_rp, x, y)
        old_aig_val_rp_0_10 = aig_val_rp_0_10

    if(aig_val_cg_0_10 != old_aig_val_cg_0_10):
        angle = 225.0 - (225.0-(-60.0))*(aig_val_cg_0_10/10.0)
        x = int( float(x_aig_cg) + float(d_aig_cg) * cos(radians(angle)) )
        y = int( float(y_aig_cg) - float(d_aig_cg) * sin(radians(angle)) )
        my_canvas.coords(line_aig_cg, x_aig_cg, y_aig_cg, x, y)
        old_aig_val_cg_0_10 = aig_val_cg_0_10

    if(aig_val_cf_0_10 != old_aig_val_cf_0_10):
        angle = 220.0 - (220.0-(-45.0))*(aig_val_cf_0_10/10.0)
        x = int( float(x_aig_cf) + float(d_aig_cf) * cos(radians(angle)) )
        y = int( float(y_aig_cf) - float(d_aig_cf) * sin(radians(angle)) )
        my_canvas.coords(line_aig_cf, x_aig_cf, y_aig_cf, x, y)
        old_aig_val_cf_0_10 = aig_val_cf_0_10

    #robinet AU
    if(robinet_au != old_robinet_au):
        if(robinet_au==0):
            my_canvas.itemconfigure(oval_au,state=HIDDEN)
        else:
            my_canvas.itemconfigure(oval_au,state=NORMAL)
        old_robinet_au = robinet_au

    #selecteur frein (service)
    if(palette_service != old_palette_service):
        old_palette_service = palette_service
        if(palette_service == RETIRE):
            my_canvas.itemconfigure(line_cmd_srv,state=HIDDEN)
            my_canvas.itemconfigure(oval_cmd_srv,state=HIDDEN)
        else:
            my_canvas.itemconfigure(line_cmd_srv,state=NORMAL)
            my_canvas.itemconfigure(oval_cmd_srv,state=NORMAL)

        if(palette_service == RETRAIT):
            my_canvas.coords(line_cmd_srv, xc_srv+5, yc_srv-d_srv/3, xc_srv-15, yc_srv+d_srv)
            my_canvas.coords(oval_cmd_srv, xc_srv-20-15, yc_srv+d_srv-20, xc_srv+20-15, yc_srv+d_srv+20)
        if(palette_service == SERVICE):
            my_canvas.coords(line_cmd_srv, xc_srv-5, yc_srv+d_srv/3, xc_srv+15, yc_srv-d_srv)
            my_canvas.coords(oval_cmd_srv, xc_srv-20+15, yc_srv-d_srv-20, xc_srv+20+15, yc_srv-d_srv+20)
        if(palette_service == ISO):
            my_canvas.coords(line_cmd_srv, xc_srv-d_srv/3, yc_srv, xc_srv+d_srv, yc_srv)
            my_canvas.coords(oval_cmd_srv, xc_srv+d_srv-20, yc_srv-20, xc_srv+d_srv+20, yc_srv+20)
        if(palette_service == NEUTRE):
            my_canvas.coords(line_cmd_srv, xc_srv+d_srv/3, yc_srv, xc_srv-d_srv, yc_srv)
            my_canvas.coords(oval_cmd_srv, xc_srv-d_srv-20, yc_srv-20, xc_srv-d_srv+20, yc_srv+20)

    #palette sens
    if(palette_sens != old_palette_sens):
        old_palette_sens = palette_sens
        if(palette_sens == AV):
            my_canvas.coords(line_cmd_sens, xc_sens, yc_sens, xav_sens-5, yav_sens)
            my_canvas.coords(line2_cmd_sens, xc_sens, yc_sens, xav_sens+5, yav_sens)
        if(palette_sens == ARRET):
            my_canvas.coords(line_cmd_sens, xc_sens, yc_sens, x0_sens-5, y0_sens)
            my_canvas.coords(line2_cmd_sens, xc_sens, yc_sens, x0_sens+5, y0_sens)
        if(palette_sens == AR):
            my_canvas.coords(line_cmd_sens, xc_sens, yc_sens, xarr_sens-5, yarr_sens)
            my_canvas.coords(line2_cmd_sens, xc_sens, yc_sens, xarr_sens+5, yarr_sens)

    #cle
    if(cle != old_cle):
        old_cle = cle
        if(cle == RETIRE):
            my_canvas.itemconfigure(line_cmd_cle,state=HIDDEN)
        else:
            my_canvas.itemconfigure(line_cmd_cle,state=NORMAL)

        if(cle == CLE_OFF):
            my_canvas.coords(line_cmd_cle, xc_cle, yc_cle, x0_cle, y0_cle)
        if(cle == CLE_ON):
            my_canvas.coords(line_cmd_cle, xc_cle, yc_cle, x1_cle, y1_cle)

    #manette frein
    if(manette_frein != old_manette_frein):
        old_manette_frein = manette_frein
        if(manette_frein==DESS):
            my_canvas.coords(line_cmd_fr, xc_boule_fr, yc_boule_fr, xdess_boule_fr, ydess_boule_fr)
            my_canvas.coords(oval_cmd_fr, xdess_boule_fr-20, ydess_boule_fr-20, xdess_boule_fr+20, ydess_boule_fr+20)
        if(manette_frein==RIEN):
            my_canvas.coords(line_cmd_fr, xc_boule_fr, yc_boule_fr, x0_boule_fr, y0_boule_fr)
            my_canvas.coords(oval_cmd_fr, x0_boule_fr-20, y0_boule_fr-20, x0_boule_fr+20, y0_boule_fr+20)
        if(manette_frein==SERR):
            my_canvas.coords(line_cmd_fr,  xc_boule_fr, yc_boule_fr, xserr_boule_fr, yserr_boule_fr)
            my_canvas.coords(oval_cmd_fr, xserr_boule_fr-20, yserr_boule_fr-20, xserr_boule_fr+20, yserr_boule_fr+20)

    #manette frein direct
    if(manette_freind != old_manette_freind):
        old_manette_freind = manette_freind
        if(manette_freind==DESS):
            x = xdess_boule_frd
            y = ydess_boule_frd
        elif(manette_freind==SERR):
            x = xserr_boule_frd
            y = yserr_boule_frd
        else:
            x = x0_boule_frd
            y = y0_boule_frd
        my_canvas.coords(line_cmd_frd, xc_boule_frd, yc_boule_frd, x, y)
        my_canvas.coords(oval_cmd_frd, x-20, y-20, x+20, y+20)

    #manettes d'acceleration
    if(old_manette_acc_0_100 != manette_acc_0_100):
        old_manette_acc_0_100 = manette_acc_0_100
        x = x0_boule_acc - (x0_boule_acc - x100_boule_acc) * manette_acc_0_100 / 100
        y = y0_boule_acc - (y0_boule_acc - y100_boule_acc) * manette_acc_0_100 / 100
        my_canvas.coords(line_cmd_acc, xc_boule_acc+40, yc_boule_acc, x, y)
        my_canvas.coords(oval_cmd_acc, x-20, y-20, x+20, y+20)

    if(old_manette_acc2_0_100 != manette_acc2_0_100):
        old_manette_acc2_0_100 = manette_acc2_0_100
        x = x0_boule_acc2 - (x0_boule_acc2 - x100_boule_acc2) * manette_acc2_0_100 / 100
        y = y0_boule_acc2 - (y0_boule_acc2 - y100_boule_acc2) * manette_acc2_0_100 / 100
        my_canvas.coords(line_cmd_acc2, xc_boule_acc2, yc_boule_acc2, x, y)
        my_canvas.coords(oval_cmd_acc2, x-20, y-20, x+20, y+20)

    #avertisseur
    if(manette_klaxon != old_manette_klaxon):
        old_manette_klaxon = manette_klaxon
        if(manette_klaxon==KL_AV):
            x = xav_klaxon;
            y = yav_klaxon
        elif(manette_klaxon==KL_AR):
            x = xarr_klaxon
            y = yarr_klaxon
        else:
            x = x0_klaxon
            y = y0_klaxon
        my_canvas.coords(line_cmd_klaxon, xc_klaxon, yc_klaxon, x, y)
        my_canvas.coords(oval_cmd_klaxon, x-20, y-20, x+20, y+20)

    #lssf
    if(old_lssf!=lssf or lssf==LSSF_CLI):
        old_lssf=lssf
        aff = 0
        if(lssf==LSSF_1):
            aff = 1
        if(lssf==LSSF_CLI and draw_update_cpt&4):
            aff = 1
        if(aff == 0):
            my_canvas.itemconfigure(oval_lssf,state=HIDDEN)
        else:
            my_canvas.itemconfigure(oval_lssf,state=NORMAL)

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# SIMU FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

# les parametres de simulations sont maintenant au debut du programme
vit_sens = 0.0    #vit mais avec un signe +=AV, -=ARR


def simu():
    global vit; global vit_sens; global mem_sens
    global aig_val_vit_0_210
    global pression_cg_0_5; global pression_cf_0_5; global pression_cfd_0_5
    global aig_val_rp_0_10; global aig_val_cg_0_10; global aig_val_cf_0_10
    global dbg_txt
    global lssf_dcpt
    global robinet_au
    mode = ""

    #entrees
    if(use_turmo12 == 0):
        traction_0_100 =  0.5 * float(manette_acc_0_100) * float(motor_on) \
                        + 0.5 * float(manette_acc2_0_100) * float(motor2_on)
        v_cible = vmax * traction_0_100 / 100.0
    else:
        traction_0_100 =  0.6154 * float(manette_acc_0_100) * float(motor_on) \
                        + 0.3846 * float(manette_acc2_0_100) * float(motor2_on)
        v_cible = vmax_turmo12 * traction_0_100 / 100.0

    t_cible=0

    if(lssf == LSSF_CLI):
        if(lssf_dcpt==0):
            robinet_au = 1
        else:
            lssf_dcpt -= 1

    #arret
    if(traction_0_100==0 and vit==0.0):
        mode = "ARRET "
        print("ARRET")

    #marche sur l'erre
    elif(traction_0_100==0 or motor_on+motor2_on==0 or v_cible<vit or robinet_au==1):
        mode = "ERRE "
        print("ERRE");
        dv = (100.0/t_100_0_erre)/10
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        t_cible = vit / (10*dv)

    #acceleration
    elif(v_cible>vit and palette_sens!=ARRET):
        dv = (100.0/t_0_100_mot)/10.0     * traction_0_100 / 100.0
        v=vit+dv
        if(v>v_cible):
            vit=v_cible
        else:
            vit=v
        mode = "ACC "
        print("ACC dv="+str(dv))
        t_cible = (v_cible-vit) / (10*dv)

    #precalcul des pressions
    #manette frein -> cg
    if(manette_frein == SERR):
        if(palette_service == SERVICE or palette_service == NEUTRE):
            dp = (1.5 / t_press_3b5_5b) / 10.0
            p = pression_cg_0_5 - dp
            if(p < 0.0):
                p = 0.0
            pression_cg_0_5 = p
    if(manette_frein == DESS):
        if(palette_service == SERVICE):
            dp = (1.5 / t_press_3b5_5b) / 10.0
            p = pression_cg_0_5 + dp
            if(p > 5.0):
                p = 5.0
            pression_cg_0_5 = p

    #manette frein direct -> cfd
    if(manette_freind == SERR):
        if(palette_service == SERVICE or palette_service == NEUTRE):
            dp = (5.0 / t_press_0b_5b_fd) / 10.0
            p = pression_cfd_0_5 + dp
            if(p > 5.0):
                p = 5.0
            pression_cfd_0_5 = p
    if(manette_freind == DESS):
        if(palette_service == SERVICE):
            dp = (5.0 / t_press_0b_5b_fd) / 10.0
            p = pression_cfd_0_5 - dp
            if(p < 0.0):
                p = 0.0
            pression_cfd_0_5 = p

    #robinet au -> cg
    if(robinet_au):
        dp = (1.5 / t_press_au_5b_3b5) / 10.0
        p = pression_cg_0_5 - dp
        if(p < 0.0):
            p = 0.0
        pression_cg_0_5 = p

    #cg=5 -> cf=0, cg=3.5 -> cf=5
    p =  (5.0 / 1.5) * (5.0 - pression_cg_0_5)
    if(p > 5.0):
        p = 5.0
    if(p < 0.0):
        p = 0.0
    pression_cf_0_5 = p

    fr_0_100 = 100.0 * (pression_cf_0_5 / 5.0)
    aig_val_rp_0_10 = pression_rp_0_10
    aig_val_cg_0_10 = pression_cg_0_5
    aig_val_cf_0_10 = pression_cf_0_5

    #freinage pneumatique (sabots + disques)
    if(fr_0_100>0.0):
        mode += "FR-AUTO "
        dv = (100.0/t_100_0_fr)/10.0    *   fr_0_100 / 100.0
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        t_cible = vit / (10*dv)
        v_cible = 0

    #freinage avec les patins
    if(pression_cg_0_5 < 2.5 and vit > 40.0):
        mode += "FR-PAT "
        dv = (100.0/t_100_0_fr_patins)/10.0
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        t_cible = vit / (10*dv)
        v_cible = 0

    #frein_direct
    if(pression_cfd_0_5 > pression_cf_0_5):
        mode += "FR-DIR "
        dv = (100.0/t_100_0_frd)/10.0    *   (pression_cfd_0_5 - pression_cf_0_5) / 5.0
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        t_cible = vit / (10*dv)
        v_cible = 0
        aig_val_cf_0_10 = pression_cfd_0_5


    #vitesse combinee avec sens
    if(palette_sens==AV):
        vit_sens = vit
        mem_sens = AV
    if(palette_sens==AR):
        vit_sens = -vit
        mem_sens = AR
    if(palette_sens==ARRET):
        vit_sens = 0

    #sorties
    aig_val_vit_0_210 = vit

    if(mode==""):
        mode = "STABLE"

    zetxt = "VIT=" + str(round(vit_sens,2)) + "kmph"
    zetxt += " VCIBLE=" + str(round(v_cible,2)) + "kmph"
    zetxt += " TCIBLE=" + str(round(t_cible,2)) + "s"
    zetxt += " ACC1=" + str(round(manette_acc_0_100,0)) + "%"
    zetxt += " ACC2=" + str(round(manette_acc2_0_100,0)) + "%"
    zetxt += " ACC=" + str(round(traction_0_100,0)) + "%"
    zetxt += " RPM=" + str(round(aig_val_trm_0_32000,0)) + "trpm"
    zetxt += " RPM2=" + str(round(aig_val_trm2_0_32000,0)) + "trpm"
    zetxt += " MODE=" + mode

    print(zetxt)
    my_canvas.itemconfigure(dbg_txt,text=zetxt)

    return




def simu_rpm():
    global aig_val_trm_0_32000; global slow_aig_val_trm_0_32000


    if(motor_on==0):
        slow_aig_val_trm_0_32000 = 0

    elif(palette_sens == ARRET or manette_acc_0_100==0 or robinet_au==1):
        slow_aig_val_trm_0_32000 = rpm_ralenti

    else:
        #rpm = rpm_ralenti + (vit/200.0) * (rpm_nominal - rpm_ralenti)                #en fonction de la vitesse
        rpm = rpm_ralenti + (manette_acc_0_100/100.0) * (rpm_nominal - rpm_ralenti)   #en fonction de la manette
        slow_aig_val_trm_0_32000 = rpm


    #pour que l'aiguille bouge lentement surtaout au l'allumage et arret du moteur
    if(slow_aig_val_trm_0_32000 > aig_val_trm_0_32000):
        aig_val_trm_0_32000 += slow_aig_trm_inc
        if(aig_val_trm_0_32000 > slow_aig_val_trm_0_32000):
            aig_val_trm_0_32000 = slow_aig_val_trm_0_32000
    if(slow_aig_val_trm_0_32000 < aig_val_trm_0_32000):
        aig_val_trm_0_32000 -= slow_aig_trm_inc
        if(aig_val_trm_0_32000 < slow_aig_val_trm_0_32000):
            aig_val_trm_0_32000 = slow_aig_val_trm_0_32000

    return

def simu_rpm2():
    global aig_val_trm2_0_32000; global slow_aig_val_trm2_0_32000

    if(motor2_on==0):
        slow_aig_val_trm2_0_32000 = 0

    elif(palette_sens == ARRET or manette_acc2_0_100==0 or robinet_au==1):
        slow_aig_val_trm2_0_32000 = rpm2_ralenti

    else:
        #rpm2 = rpm2_ralenti + (vit/200.0) * (rpm2_nominal - rpm2_ralenti)                #en fonction de la vitesse
        rpm2 = rpm2_ralenti + (manette_acc2_0_100/100.0) * (rpm2_nominal - rpm2_ralenti)  #en fonction de la manette
        slow_aig_val_trm2_0_32000 = rpm2

    #pour que l'aiguille bouge lentement surtaout au l'allumage et arret du moteur
    if(slow_aig_val_trm2_0_32000 > aig_val_trm2_0_32000):
        aig_val_trm2_0_32000 += slow_aig_trm2_inc
        if(aig_val_trm2_0_32000 > slow_aig_val_trm2_0_32000):
            aig_val_trm2_0_32000 = slow_aig_val_trm2_0_32000
    if(slow_aig_val_trm2_0_32000 < aig_val_trm2_0_32000):
        aig_val_trm2_0_32000 -= slow_aig_trm2_inc
        if(aig_val_trm2_0_32000 < slow_aig_val_trm2_0_32000):
            aig_val_trm2_0_32000 = slow_aig_val_trm2_0_32000

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# SERIAL FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

#mise a jour de l'arduino avec les nouvelles valeurs
#
# protocol:
# s s s s s s s s p <0..sens> <vit0-255> k <0..KL_AV.KL_AR> 0     1     e
#                                                           |<a0> |<a1> |<e0> <e1>
#
#  e0:   0 AU FRD-DESS FRD-SERR FRA-DESS FRA-SERR SENS_AV SENS-AR
MSK_AU       = 64
MSK_FRD_DESS = 32
MSK_FRD_SERR = 16
MSK_FRA_DESS = 8
MSK_FRA_SERR = 4
MSK_SENS_AV  = 2
MSK_SENS_AR  = 1
#  e1:   0  0        0        0        0        0  KL_AV    KL_AR
MSK_KL_AV = 2
MSK_KL_AR = 1
#  a0: 0-255 -> acc  (diesel)
#  a1: 0-255 -> acc2 (turbine)

first = 1; old_e1 = 0; old_e0 = 0; old_a0 = 0; old_a1 = 0

def serial_update():
    global ser; global serial_ok
    global manette_acc_0_100; global manette_acc2_0_100;
    global palette_sens; global manette_klaxon
    global robinet_au; global manette_freind; global manette_frein;
    global first; global old_e1; global old_e0; global old_a0; global old_a1

    if(serial_ok==0):
        return

    try:
    #if(1):
        nb_rx=0
        while(ser.inWaiting()):
            nb_rx+=1
            rb = ser.read(1)
            if(nb_rx==1): a0 = rb[0]
            if(nb_rx==2): a1 = rb[0]
            if(nb_rx==3): e0 = rb[0]
            if(nb_rx==4): e1 = rb[0]
            #print("RX:" + str(rb[0]))

        if(nb_rx==4 and first==0):

            if(abs(old_a0 - a0)>5):
                manette_acc_0_100 = 100 * a0 / 255
                if(a0<5):
                    manette_acc_0_100 = 0
                if(a0>250):
                    manette_acc_0_100 = 100
                old_a0 = a0

            if(abs(old_a1 - a1)>5):
                manette_acc2_0_100 = 100 * a1 / 255
                if(a1<5):
                    manette_acc2_0_100 = 0
                if(a1>250):
                    manette_acc2_0_100 = 100
                old_a1 = a1

            e0 ^= 255; e1 ^= 255;

            if((old_e0 & MSK_AU)==0 and (e0 & MSK_AU)):
                robinet_au ^= 1

            if(e0 & MSK_SENS_AV):
                palette_sens = AV
            elif(e0 & MSK_SENS_AR):
                palette_sens = AR
            elif(old_e0 & MSK_SENS_AV):
                palette_sens = ARRET
            elif(old_e0 & MSK_SENS_AR):
                palette_sens = ARRET

            if(e0 & MSK_FRA_DESS):
                manette_frein = DESS
            elif(e0 & MSK_FRA_SERR):
                manette_frein = SERR
            elif(old_e0 & MSK_FRA_DESS):
                manette_frein = RIEN
            elif(old_e0 & MSK_FRA_SERR):
                manette_frein = RIEN

            if(e0 & MSK_FRD_DESS):
                manette_freind = DESS
            elif(e0 & MSK_FRD_SERR):
                manette_freind = SERR
            elif(old_e0 & MSK_FRD_DESS):
                manette_freind = RIEN
            elif(old_e0 & MSK_FRD_SERR):
                manette_freind = RIEN

            if(e1 & MSK_KL_AV):
                manette_klaxon = KL_AV
            elif(e1 & MSK_KL_AR):
                manette_klaxon = KL_AR
            elif(old_e1 & MSK_KL_AV):
                manette_klaxon = KL_0
            elif(old_e1 & MSK_KL_AR):
                manette_klaxon = KL_0

            old_e1 = e1
            old_e0 = e0

        first = 0

        # 1) synchros
        data = ord('s')
        for i in range(0,8):
            ser.write(bytes([data]))

        # 2) pwm command (sens + vit)
        ser.write(bytes([ord('p')]))
        #sens
        if(mem_sens == AV):
            sens = 1      #sens +
        else:
            sens = 0      #sens -
        sens ^= pwm_inv_sens
        ser.write(bytes([sens]))
        #print("TX:"+str(sens))
        #vit
        if(vit==0.0):
            v = 0
        else:
            v = pwm_min + int(float(pwm_max - pwm_min) * (vit / pwm_vit_max))
            if(v > pwm_max):
                v = pwm_max
        ser.write(bytes([v]))
        #print("TX:"+str(v))

        # 3) dcc command (adr + sens + vit + fct)
        ser.write(bytes([ord('d')]))
        #adresse
        ser.write(bytes([dcc_adr]))
        #sens
        if(mem_sens == AV):
            sens = 1      #sens +
        else:
            sens = 0      #sens -
        sens ^= dcc_inv_sens
        ser.write(bytes([sens]))
        #vit
        if(vit==0.0):
            v = 0
        else:
            v = dcc_min + int(float(dcc_max - dcc_min) * (vit / dcc_vit_max))
            if(v > dcc_max):
                v = dcc_max
        ser.write(bytes([v]))
        # fct (F0 pour les phares)
        fct = 0
        if(phares):
            fct = 1
        ser.write(bytes([fct]))

        # 3) cmd klaxon (plus utilise car le timer 2 est utilise par le DCC)
        # ser.write(bytes([ord('k')]))
        # data = 0
        # if(manette_klaxon == KL_AV):  # 0.0.0.0.0.0.KLAV.KLAR
        #    data |= 2
        #if(manette_klaxon == KL_AR):  # 0.0.0.0.0.0.KLAV.KLAR
        #    data |= 1
        #ser.write(bytes([data]))
        #print("TX:"+str(data))

        # 4) read analog 0 & 1
        ser.write(bytes([ord('0')]))
        ser.write(bytes([ord('1')]))

        # 5) read boutons
        ser.write(bytes([ord('e')]))

    except:
        serial_ok=0
        print("ERROR: Perte de la liaison serie")

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# NETWORK FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

def net_connect():
    if(use_network==0):
        return

    if(use_local):
        adr = socket.gethostbyname(socket.gethostname())
    else:
        adr = server_ip_adr

    s.connect((adr, server_port))           #uu: mettre une touche d'inversion ex: I
    print("NETWORK: connected")
    welcome = s.recv(1024).decode()
    print("NETWORK: server string: \"" + welcome + "\"")
    s.send("cabine netv1 rtg".encode())
    return

net_connect()

def net_disconnect():
    if(use_network==0):
        return
    s.close()
    return

old_msg = ""

def net_update():
    global old_msg

    if(use_network==0):
        return

    # adr
    msg = "a" + str(dcc_adr)

    # sens
    if(mem_sens == AV):
        sens = 1      #sens +
    else:
        sens = 0      #sens -
    sens ^= dcc_inv_sens
    if(sens):
        msg += " s+"
    else:
        msg += " s-"

    # vit
    if(vit==0.0):
        v = 0
    else:
        v = net_min + int(float(net_max - net_min) * (vit / dcc_vit_max))
        if(v > net_max):
            v = net_max
    msg += " v" + str(v)

    # fct (F0 si au moins un des boutons des phares est actif)
    fct = 0
    if(phares):
        fct = 1
    if(fct):
        msg += " f0+"
    else:
        msg += " f0-"

    msg += "\n"

    # send msg
    if(msg != old_msg):
        print("NETWORK: send: " + msg)
        s.send(msg.encode())
        old_msg = msg

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# CALLBACKS
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

cpt_avr = 0
cpt_err = 0

def timer_1s_cb():
    global cpt_avr
    #global aig_val_vit_0_210;
    cpt_avr += 1
    ##print("----------------------------------")
    ##print("cpt_avr = " + str(cpt_avr));
    #aig_val_vit_0_210 = float(s_pot.get())

    simu()
    simu_rpm()
    simu_rpm2()

    serial_update()
    net_update()

    draw_update()


mouse_owner = ""
x_press = 0
y_press = 0
mem_manette_acc_0_100 = 0
mem_manette_acc2_0_100 = 0


def callback_my_canvas_mouse_press(event):
    global mouse_owner; global x_press; global y_press
    global motor_on; global motor2_on;
    global mem_manette_acc_0_100
    global mem_manette_acc2_0_100
    global lssf; global phares

    m_x = int(event.x); x_press = m_x
    m_y = int(event.y); y_press = m_y
    #print("mouse press at " + str(m_x) + " " + str(m_y))

    if(m_x>xc_au and m_x<x_au and m_y>yc_au and m_y<y_au):
        mouse_owner = "robinet_au"

    if(m_x>95 and m_x<243 and m_y>220 and m_y<346):
        mouse_owner = "service"

    if(m_x>xc_cle-10 and m_x<x0_cle+10 and m_y>yc_cle-10 and m_y<x1_cle+10):
        mouse_owner = "cle"

    if(m_x>680 and m_x<722 and m_y>221 and m_y<306):
        mouse_owner = "sens"

    if(m_x>x0_boule_fr-20 and m_x<x0_boule_fr+20 and m_y>y0_boule_fr-20 and m_y<y0_boule_fr+20):
        mouse_owner = "frein"

    if(m_x>x0_boule_frd-20 and m_x<x0_boule_frd+20 and m_y>y0_boule_frd-20 and m_y<y0_boule_frd+20):
        mouse_owner = "freind"

    if(m_x>720 and m_x<875 and m_y>230 and m_y<430):
        x1 = x0_boule_acc - (x0_boule_acc - x100_boule_acc) * manette_acc_0_100 / 100
        y1 = y0_boule_acc - (y0_boule_acc - y100_boule_acc) * manette_acc_0_100 / 100
        x2 = x0_boule_acc2 - (x0_boule_acc2 - x100_boule_acc2) * manette_acc2_0_100 / 100
        y2 = y0_boule_acc2 - (y0_boule_acc2 - y100_boule_acc2) * manette_acc2_0_100 / 100
        if(m_x>x1-20 and m_x<x1+20 and m_y>y1-20 and m_y<y1+20):
            mouse_owner = "acc"
            mem_manette_acc_0_100 = manette_acc_0_100
        elif(m_x>x2-20 and m_x<x2+20 and m_y>y2-20 and m_y<y2+20):
            mouse_owner = "acc2"
            mem_manette_acc2_0_100 = manette_acc2_0_100
        else:
            mouse_owner = "acc12"
            mem_manette_acc_0_100 = manette_acc_0_100
            mem_manette_acc2_0_100 = manette_acc2_0_100

    if(m_x>x0_klaxon-20 and m_x<x0_klaxon+20 and m_y>y0_klaxon-20 and m_y<y0_klaxon+20):
        mouse_owner = "klaxon"

    # motor y 130-175    x 329 ON1 365 OFF1 408 ON2 443 OFF2 486 ONAUX1 526 OFFAUX1 568 ONAUX2 609 OFFAUX2 648
    if(m_y>130 and m_y<175):
        if(m_x>365 and m_x<408):
            motor_on = 0
        if(m_x>329 and m_x<365):
            if(manette_acc_0_100==0):
                #if(palette_sens == ARRET):
                if(cle == CLE_ON):
                    motor_on = 1

        if(m_x>443 and m_x<486):
            motor2_on = 0
        if(m_x>408 and m_x<443):
            if(manette_acc2_0_100==0):
                #if(palette_sens == ARRET):
                if(cle == CLE_ON):
                    motor2_on = 1


    if(lssf == LSSF_CLI):
        if(m_x>xc_ack_lssf-20 and m_x<xc_ack_lssf+20 and m_y>yc_ack_lssf-20 and m_y<yc_ack_lssf+20):
            lssf = LSSF_1
        if(m_x>xc_ack2_lssf-20 and m_x<xc_ack2_lssf+20 and m_y>yc_ack2_lssf-20 and m_y<yc_ack2_lssf+20):
            lssf = LSSF_1

    if(m_x>275 and m_x<295 and m_y>460 and m_y<480):
        phares ^= 1

    return


def callback_my_canvas_mouse_move(event):
    global manette_frein; global manette_freind
    global manette_acc_0_100; global manette_acc2_0_100
    global manette_klaxon
    m_x = int(event.x);
    m_y = int(event.y);

    print("mouse move at " + str(m_x) + " " + str(m_y) + " owner=" + mouse_owner)

    if(mouse_owner=="frein"):
        if(m_y < y_press-10):   #desserage
            manette_frein=DESS
        elif(m_y > y_press+10):   #serrage
            manette_frein=SERR
        else:
            manette_frein=RIEN

    elif(mouse_owner=="freind"):
        if(m_y < y_press-10):   #desserage
            manette_freind=DESS
        elif(m_y > y_press+10): #serrage
            manette_freind=SERR
        else:
            manette_freind=RIEN

    elif(mouse_owner=="acc"):
        dy = y_press - m_y   #positif ds le sens des accelerations
        dy_100 = y0_boule_acc - y100_boule_acc
        v = mem_manette_acc_0_100 + 100 * dy / dy_100
        if(v<0):
            v=0
        if(v>100):
            v=100
        manette_acc_0_100=v

    elif(mouse_owner=="acc2"):
        dy = y_press - m_y
        dy_100 = y0_boule_acc2 - y100_boule_acc2
        v = mem_manette_acc2_0_100 + 100 * dy / dy_100
        if(v<0):
            v=0
        if(v>100):
            v=100
        manette_acc2_0_100=v

    elif(mouse_owner=="acc12"):
        dy = y_press - m_y   #positif ds le sens des accelerations
        dy_100 = y0_boule_acc - y100_boule_acc
        v = mem_manette_acc_0_100 + 100 * dy / dy_100
        if(v<0):
            v=0
        if(v>100):
            v=100
        manette_acc_0_100=v
        manette_acc2_0_100=v

    elif(mouse_owner == "klaxon"):
        if(m_y < y_press-10):
            manette_klaxon = KL_AV
        elif(m_y > y_press+10):
            manette_klaxon = KL_AR
        else:
            manette_klaxon = KL_0


    return


def callback_my_canvas_mouse_release(event):
    global robinet_au
    global palette_service; global palette_sens; global cle
    global mouse_owner
    global manette_frein
    global manette_freind
    global manette_acc_0_100
    global manette_acc2_0_100
    global manette_klaxon
    global motor_on; global motor2_on

    m_x = int(event.x);
    m_y = int(event.y);

    if(mouse_owner=="robinet_au"):
        robinet_au ^= 1
        ##motor_on = 0    #les moteurs sont juste mis au ralentit maintenant
        ##motor2_on = 0
        #if(m_x>x_press):
        #    robinet_au = 1
        #else:
        #    robinet_au = 0

    elif(mouse_owner == "cle"):
        v = cle
        if(v == RETIRE):
             v = CLE_OFF
        elif(v == CLE_OFF):
            if(m_x < x_press):
                v = CLE_ON
            if(m_x > x_press):
                v = RETIRE
        elif(v == CLE_ON):
            if(m_x > x_press):
                v = CLE_OFF
        cle = v
        #if(cle != CLE_ON):   # uu a verifier
        #     motor_on = 0    #
        #     motor2_on = 0   #

    elif(mouse_owner=="service"):
        v = palette_service
        print("service: v="+str(v))
        if(v==RETIRE):
            v=RETRAIT
        elif(v==RETRAIT):
            #if(abs(m_y-y_press)>abs(m_x-x_press)):
            if(abs(m_y-y_press)>75):
                v=RETIRE
            elif(m_x<x_press):
                v=NEUTRE
            elif(m_x>x_press):
                v=ISO
        elif(v==SERVICE):
            if(m_x<x_press):
                v=NEUTRE
            if(m_x>x_press):
                v=ISO
        elif(v==NEUTRE):
            if(m_y<y_press):
                v=SERVICE
            if(m_y>y_press):
                v=RETRAIT
        elif(v==ISO):
            if(m_y<y_press):
                v=SERVICE
            if(m_y>y_press):
                v=RETRAIT
        palette_service = v

    elif(mouse_owner=="sens"):     #a mettre ds mouse move pr plus de rapidite
        #if(vit==0.0 and manette_acc_0_100==0):
        if(m_y<y_press):
            if(palette_sens == AR):
                palette_sens = ARRET
            elif(palette_sens == ARRET):
                palette_sens = AV
        else:
            if(palette_sens == AV):
                palette_sens = ARRET
            elif(palette_sens == ARRET):
                palette_sens = AR

    elif(mouse_owner=="frein"):
        manette_frein=RIEN

    elif(mouse_owner=="freind"):
        manette_freind=RIEN

    elif(mouse_owner=="klaxon"):
        manette_klaxon=KL_0

    mouse_owner=""
    return


def callback_my_canvas_mouse_press_right(event):

    m_x = int(event.x);
    m_y = int(event.y);

    return


def callback_my_canvas_mouse_release_right(event):
    return





# mapping du pave numerique
# 7 8 9  -->  ACC1 ACC1+2 ACC2
# 4 5 6  -->  DEC1 DEC1+2 DEC2
# 1 2 3  -->  DESS AU     AV
# 0 0 .  -->  SERR------- AR
#
# * --> phares
# l --> lssf
# espace --> arret immediat
#
def cb_key(event):
    global manette_frein
    global manette_acc_0_100
    global manette_acc2_0_100
    global palette_sens
    global robinet_au
    #global palette_service
    #global motor_on
    global lssf; global lssf_dcpt
    global pahres
    global pwm_inv_sens
    global dcc_inv_sens

    k = repr(event.char)
    #print("pressed " + k)  #return 'a' with '

    if(k[1]=="2"):
        robinet_au ^= 1

    if(k[1]=="3"):
        if(vit==0.0 and manette_acc_0_100==0):
            v = palette_sens
            if(v==AR):
                v=ARRET
            elif(v==ARRET):
                v=AV
            palette_sens = v

    if(k[1]=="."):
        if(vit==0.0 and manette_acc_0_100==0):
            v = palette_sens
            if(v==AV):
                v=ARRET
            elif(v==ARRET):
                v=AR
            palette_sens = v

    if(k[1]=="9"):
        v = manette_acc_0_100
        v = (v//5) * 5 + 5    #(v//5)*5 pour ne garder que multiples de 5
        if(v > 100):
            v = 100
        manette_acc_0_100 = v
    if(k[1]=="6"):
        v = manette_acc_0_100
        v = (v//5) * 5 - 5
        if(v < 0):
            v = 0
        manette_acc_0_100 = v
    if(k[1]=="7"):
        v = manette_acc2_0_100
        v = (v//5) * 5 + 5
        if(v > 100):
            v = 100
        manette_acc2_0_100 = v
    if(k[1]=="4"):
        v = manette_acc2_0_100
        v = (v//5) * 5 - 5
        if(v < 0):
            v = 0
        manette_acc2_0_100 = v
    if(k[1]=="8"):
        v = manette_acc_0_100
        v = (v//5) * 5 + 5
        if(v > 100):
            v = 100
        manette_acc_0_100 = v
        manette_acc2_0_100 = v
    if(k[1]=="5"):
        v = manette_acc_0_100
        v = (v//5) * 5 - 5
        if(v < 0):
            v = 0
        manette_acc_0_100 = v
        manette_acc2_0_100 = v

    if(k[1]=="1"):
        manette_frein=DESS
    if(k[1]=="0"):
        manette_frein=SERR

    if(k[1]=="l"):
        if(lssf == LSSF_0):
            lssf_dcpt = 50
            lssf = LSSF_CLI
        if(lssf == LSSF_1):
            lssf = LSSF_0

    if(k[1]==" "):    #arret immediat loco miniature avec la barre d'espace
        robinet_au = 1
        vit = 0

    if(k[1]=="*"):
        phares ^= 1

    if(k[1]=="i"):
        pwm_inv_sens ^= 1
        dcc_inv_sens ^= 1

    return


def cb_key_release(event):
    global manette_frein

    k = repr(event.char)
    #print("released " + k)

    if(k[1]=="1" or k[1]=="0"):
        manette_frein = RIEN

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# GUI management
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

top = Tk()
top.title(TITLE)

row = 0

img_pupitre = PhotoImage(file = fichier_pupitre)
img_dx = img_pupitre.width()
img_dy = img_pupitre.height()

my_canvas = Canvas(top, bg = "#cccccc", height = img_dy+20, width = img_dx)
my_canvas.grid(row = row, column = 0, columnspan = 4)
row=row+1

my_canvas.bind("<Button-1>"       , callback_my_canvas_mouse_press)
my_canvas.bind("<ButtonRelease-1>", callback_my_canvas_mouse_release)
my_canvas.bind("<B1-Motion>"       , callback_my_canvas_mouse_move)
my_canvas.bind("<Button-3>"        , callback_my_canvas_mouse_press_right)
my_canvas.bind("<ButtonRelease-3>", callback_my_canvas_mouse_release_right)
top.bind("<Key>", cb_key)
top.bind("<KeyRelease>", cb_key_release)



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# MAIN
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

if(serial_ok):
    try:
        ser = serial.Serial(SZ_SERIALPORT,115200,timeout=10,stopbits=serial.STOPBITS_TWO)
        ser.setDTR(False)
        print("INFO:  Ouverture du port serie " + SZ_SERIALPORT + " OK")
    except:
        serial_ok=0
        print("ERROR: Ne peut pas ouvrir le port serie " + SZ_SERIALPORT)

#myFont = Font(family="Times New Roman", size=12)
#text.configure(font=myFont)
#myFont.configure(size=14)
draw_first()
draw_update()

# Thread on enumeration checking
daemon_bool = True
def daemon_func():
    global cpt_err
    next_call = time.time()
    while daemon_bool:
        print("time="+str(datetime.datetime.now())+" cpt_err="+str(cpt_err)+" x="+str(x_press)+" y="+str(y_press))
        try:
            timer_1s_cb()
            next_call += 0.1
            sleep_duration = next_call - time.time()
            if(sleep_duration <= 0):
                print("ERROR: non accurate time !!")
                cpt_err += 1
                #time.sleep(1)
                next_call = time.time()
            else:
                time.sleep(sleep_duration)
        except RuntimeError:
            None


daemon_thread = threading.Thread(target=daemon_func)
daemon_thread.daemon = True
daemon_thread.start()

top.mainloop()

if(serial_ok):
    ser.close()
daemon_bool = False
print("ByeBye")
time.sleep(1)




# QUELQUES NOTES SUR LA PWM POUR ARDUINO UNO R3:

# /*
# * T0A --> PWM D6
# * T0B --> PWM D5
# * T1A --> PWM D9
# * T1B --> PWM D10
# * T2A --> PWM D11
# * T2B --> PWM D3
# *
# * analogWrite(pin, value);  // 0->0% 1->2/256% 255->256/256%
# * T0 aussi utilise pour millis ... (pas de pb pour utiliser pwm en //)
# * T2 peut etre utilise par tone (dans ce cas pas de pwm avec T2)
# *     tone(pin,value,[duration ms])  //31Hz-65kHz
# *     notone()
# * T1 peut etre utilise par servo (dans ce cas pas de pwm avec T1) servos sur 12 pins 2-13
# *     #include <Servo.h>
# *     Servo aServo  or   Servo tServo[12]
# *     aServo.attach(pin_number)
# *     aServo.writeMicroseconds(uS) // aServo.write(0-180)
# *     plus besoin d'utiliser refresh
# */

# EXEMPLE DE CODE ARDUINO DE BASE supportant juste PWM (sens et vitesse)
# QUI FONCTIONNE AVEC CETTE CABINE:
# Pour le code plus complet avec DCC et manette, regarder le fichier cabine.ino le plus recent
#
# #define PIN_PWM  6
# #define PIN_SENS 5
#
# void setup()
# {
#    pinMode(PIN_PWM , OUTPUT); digitalWrite(PIN_PWM , LOW);
#    pinMode(PIN_SENS, OUTPUT); digitalWrite(PIN_SENS, LOW);
#
#    pinMode(13, OUTPUT);
#    digitalWrite(13, HIGH);
#    Serial.begin(115200);
#    while (!Serial) { }
#    digitalWrite(13, LOW);
# }
#
# int step=0;
# int sens;
# int pwm_vit;
#
# void loop()
# {
#     int rx;
#
#     rx = Serial.read();
#     if(rx==-1) return;
#
#     if(step==0)
#     {
#         if(rx=='s') {}
#         if(rx=='p') step=1;
#         if(rx=='0') Serial.write(analogRead(A0)/4);   //0-1023->0-255
#         if(rx=='1') Serial.write(analogRead(A1)/4);   //0-1023->0-255
#         if(rx=='e') { Serial.write(0xff); Serial.write(0xff); } //not supported
#         if(rx=='k') step=3;
#         if(rx=='d') step=4;
#     }
#     else if(step==1)
#     {
#         sens = rx;
#         step = 2;
#     }
#     else if(step==2)
#     {
#         pwm_vit = rx;
#         step = 0;
#
#         if(sens)
#             digitalWrite(PIN_SENS, HIGH);
#         else
#             digitalWrite(PIN_SENS, LOW);
#
#         analogWrite(PIN_PWM, pwm_vit);
#     }
#     else if(step==3)
#     {
#         step = 0;
#     }
#     else if(step==4)
#     {
#          step = 5;
#     }
#     else if(step==5)
#     {
#          step = 6;
#     }
#     else if(step==6)
#     {
#          step = 7;
#     }
#     else if(step==7)
#     {
#          step = 0;
#     }
# }
