# MiniCab : GUI du testeur DCC par le reseau
#
# History:
# 2019/02/24: Delmas-Begue Ulysse: adaptation du reseau pour s'interfcaer avec D17
#                                  configuration par defaut pour D17
# 2017/03/22: Delmas-Begue Ulysse: creation
#
# Notes:
# License: GPLv2
# This application works on Linux/Windows and Python3.x/2.x
#
# TBD
#

TITLE = "MINI-CAB DCC NET TEST v20190226 UDB"

use_network   = 1
use_local     = 0
server_ip_adr = "192.168.4.1"
server_port   = 1234

dcc_adr = 3
sens = 1
cran = 0  #
max_cran = 100 # 100 pour D17, 28 pour fdcc2017
fct_43210 = 0



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# IMPORT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

import os
import time
import sys
import subprocess
import threading
import datetime
import socket
try:
    # for Python 3
    from tkinter import *
    print("Python 3.x interpreter detected")
except ImportError:
    # for Python2
    from Tkinter import *
    print("Python 2.x interpreter detected")
from math import *
import tkinter.messagebox



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# NETWORK FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

def net_connect():
    if(use_network==0):
        return

    if(use_local):
        adr = socket.gethostbyname(socket.gethostname())
    else:
        adr = server_ip_adr

    s.connect((adr, server_port))           #uu: mettre une touche d'inversion ex: I
    print("NETWORK: connected")
    welcome = s.recv(1024).decode()
    print("NETWORK: server string: \"" + welcome + "\"")
    s.send("cabine netv1 test".encode())
    return

net_connect()

def net_disconnect():
    if(use_network==0):
        return
    s.close()
    return

def net_update():
    if(use_network==0):
        return

    # adr
    adr_cb()
    msg = "a" + str(dcc_adr)

    # sens
    if(sens):
        msg += " s+"
    else:
        msg += " s-"

    # vit
    msg += " v" + str(cran)

    # fct (F0 si au moins un des boutons des phares est actif)
    if(fct_43210 & 1):
        msg += " f0+"
    else:
        msg += " f0-"
    if(fct_43210 & 2):
        msg += " f1+"
    else:
        msg += " f1-"
    if(fct_43210 & 4):
        msg += " f2+"
    else:
        msg += " f2-"
    if(fct_43210 & 8):
        msg += " f3+"
    else:
        msg += " f3-"
    if(fct_43210 & 16):
        msg += " f4+"
    else:
        msg += " f4-"

    msg += "\n"

    # send msg
    print("NETWORK: send: " + msg)
    s.send(msg.encode())

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# CALLBACKS
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

def sens_cb():
    global sens
    global c_sens
    if(sens == 1):
        sens = 0; c_sens.deselect()
    else:
        sens = 1; c_sens.select()
    net_update()
    print("sens="+str(sens)+"\n")
    return

def pot_cb(val):
    global cran
    global s_pot
    cran = int(s_pot.get())
    net_update()
    print("cran_0_"+str(max_cran)+"="+str(cran)+"\n")  #or val
    return

def f0_cb():
    global fct_43210
    global c_f0
    if(fct_43210&1):
        fct_43210 ^= 1; c_f0.deselect()
    else:
        fct_43210 ^= 1; c_f0.select()
    net_update()
    print("fct_43210="+str(fct_43210)+"\n")
    return

def f1_cb():
    global fct_43210
    global c_f1
    if(fct_43210&2):
        fct_43210 ^= 2; c_f1.deselect()
    else:
        fct_43210 ^= 2; c_f1.select()
    net_update()
    print("fct_43210="+str(fct_43210)+"\n")
    return

def f2_cb():
    global fct_43210
    global c_f2
    if(fct_43210&4):
        fct_43210 ^= 4; c_f2.deselect()
    else:
        fct_43210 ^= 4; c_f2.select()
    net_update()
    print("fct_43210="+str(fct_43210)+"\n")
    return

def f3_cb():
    global fct_43210
    global c_f3
    if(fct_43210&8):
        fct_43210 ^= 8; c_f3.deselect()
    else:
        fct_43210 ^= 8; c_f3.select()
    net_update()
    print("fct_43210="+str(fct_43210)+"\n")
    return

def f4_cb():
    global fct_43210
    global c_f4
    if(fct_43210&16):
        fct_43210 ^= 16; c_f4.deselect()
    else:
        fct_43210 ^= 16; c_f4.select()
    net_update()
    print("fct_43210="+str(fct_43210)+"\n")
    return

def adr_cb():
    global dcc_adr
    s = e_adr.get()
    try:
        a = int(s)
    except:
        a = 0
    dcc_adr = a


#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# GUI management
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

top = Tk()
top.title(TITLE)

row = 0

l_adr = Label(top, text = "adr") #font=("Helvetica", 16), fg = "#800000")
e_adr = Entry(top)
l_adr.grid(row = row, column = 0, sticky = W)
e_adr.grid(row = row, column = 1, sticky = W + E + N + S)
e_adr.delete(0, 1000); e_adr.insert(0, "3")

row += 1

c_sens = tkinter.Checkbutton(top, text="sens AV", command = sens_cb)
c_sens.grid(row = row, column = 0, sticky = W)
c_sens.select()

#s_pot = Scale(top, from_=0, to=28, orient=VERTICAL, tickinterval=5, command = pot_cb, length=200)
#s_pot.grid(row = 0, column = 1, rowspan=6, sticky = W + E + N + S)
s_pot = Scale(top, from_=0, to=max_cran, orient=HORIZONTAL, tickinterval=5, command = pot_cb, length=200)
s_pot.grid(row = row, column = 1, sticky = W + E + N + S)
s_pot.set(0)

row += 1

c_f0 = tkinter.Checkbutton(top, text="sens F0/FL", command = f0_cb)
c_f0.grid(row = row, column = 0, sticky = W)
c_f0.deselect()
row += 1
c_f1 = tkinter.Checkbutton(top, text="sens F1", command = f1_cb)
c_f1.grid(row = row, column = 0, sticky = W)
c_f1.deselect()
row += 1
c_f2 = tkinter.Checkbutton(top, text="sens F2", command = f2_cb)
c_f2.grid(row = row, column = 0, sticky = W)
c_f2.deselect()
row += 1
c_f3 = tkinter.Checkbutton(top, text="sens F3", command = f3_cb)
c_f3.grid(row = row, column = 0, sticky = W)
c_f3.deselect()
row += 1
c_f4 = tkinter.Checkbutton(top, text="sens F4", command = f4_cb)
c_f4.grid(row = row, column = 0, sticky = W)
c_f4.deselect()



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# MAIN
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

top.mainloop()

daemon_bool = False
print("ByeBye")
time.sleep(1)
