# Pupitre d'y8000
#
# History:
# 2019/02/24: Delmas-Begue Ulysse: adaptation du reseau pour s'interfcaer avec D17
#                                  configuration par defaut pour D17
# 2017/03/22: Delmas-Begue Ulysse: rajout du reseau pour s'interfacer avec fdcc2017
# 2016/12/19: Delmas-Begue Ulysse: rajout DCC et phares
# 2016/12/09: Delmas-Begue Ulysse: verification ok (interface, clavier, serial)
# 2016/12/09: Delmas-Begue Ulysse: creation
#
# Notes:
# License: GPLv2
# Photo du pupitre: http://www.encabine.com
# This application works on Linux/Windows and Python3.x/2.x
#
# TBD
# - pente + vent ?
# - sonds (+klaxon)

TITLE = "FDCC Pupitre Y8000 v20190226 UDB"

use_serial    = 0
SZ_SERIALPORT = "\\\\.\\COM4"

use_network   = 1
use_local     = 0
server_ip_adr = "192.168.4.1"
server_port   = 1234

# caracteristique (surement qq erreurs)
#
# nb             | 375
# masse          | 35t
# vmax           | 30kmph petit regime, 60kmph grand regime
# diesel         | Poyaud V12 205kW = 266ch
# - ralenti      | ?
# - nominal max  | ?
# - transmission | hydraulique
# - reservoire   | 700 l
# - conso        | ?
# freinage       | sabot / frein moteur
#

# pupitre:
#      CF     |  press huile   cmf alerte/alerte   start | vbat trm
#  CG         |  bat                               stop  |
#      RP-RE  |  neutre        vit                 ?     |
#
# manettes:
# acc-frein_direct  av->ar 100% 0% (fd- neutre fd+)
# frein automatique av->ar surcharge desserage marche serrage urgence
# sens              av->ar avant point neutre arriere
# regime            g->d   grand regime, point mort, petit regime
#
# boutons
#  frontal | gauche | droite | projecteurs | gauche | droite | cabine | pupitre | ? | gauche | droite | projecteurs | gauche | droite | frontal
#  FANAL                     |             | FEUX ROUGES     |                      | FEUX ROUGES     |             |                     FANAL
#  signalisation avant                                       |                      |                                     signalisation arriere
#

# parametres pwm pour commander une locomotive via carte Arduino
pwm_vit_max    = 60   # vitesse correspondante a pwm_max
pwm_max        = 255  # valeur de la pwm(0-255) a la vitesse max
pwm_min        = 1    # valeur de la pwm(0-255) pour demarrer la loco
pwm_inv_sens   = 0    # invertion du sens (0=ne pas inverser, 1=inverser)

# parametres dcc pour commander une locomotive via carte Arduino
dcc_adr        = 3    # adresse de la locomotive
dcc_vit_max    = 60   # vitesse correspondante a pwm_max
dcc_max        = 28   # valeur du cran DCC 0-28 a la vitesse max
dcc_min        = 1    # valeur du cran DCC 0-28 pour demarrer la loco
dcc_inv_sens   = 0    # invertion du sens (0=ne pas inverser, 1=inverser)
net_max        = 100  # valeur a envoyer par le reseau a la vitesse max (100 pour D17, 28 pour fdcc2017)
net_min        = 1    # valeur a envoyer par le reseau pour demarrer la loco

# parametres de simulation (a adapter suivant les besoins)
# 1er profil (a vide)
vmax              = 60      # kmph
vmax_petit_regime = 30      # kmph
t_0_100_mot       = 10.0    # s
t_100_0_fr        = 5.0     # s
t_100_0_frd       = 5.0     # s  a vide
t_100_0_mot       = 15.0    # s (frein moteur due au couplage hydraulique)
t_100_0_erre      = 30.0    # s
t_press_3b5_5b    = 4.0     # s
t_press_au_5b_3b5 = 1.0     # s
t_press_0b_5b_fd  = 3.0     # s
rpm_ralenti       = 500     # tr/min
rpm_nominal       = 2000.0  # tr/min
slow_aig_trm_inc  = 40.0    # tr/min /tic
# 2eme profile (avec des wagons)
t2_0_100_mot      = 20.0   # s
t2_100_0_frd      = 10.0   # s
t2_100_0_mot      = 20.0   # s (frein moteur due au couplage hydraulique)


# a l'init
profil = 1                 # choix du profil (1 ou 2)
motor_on  = 0              #
vit = 0.0                  # kmph (a l'init)
pression_rp_0_10  = 8.0    # bars (a l'init mais bouge pas)
pression_cg_0_5   = 0.0    # bars (a l'init)
pression_cfd_0_5  = 0.0    # bars (a l'init) frein direct
pression_cf_0_5   = 5.0    # bars (a l'init)
robinet_au = 0             # 0=ferme, 1=ouvert donc AU
manette_acc_0_100 = 0      # %

AV       = 2
ARRET    = 3
AR       = 4
palette_sens = ARRET
mem_sens = AV

GRAND    = 2
ARRET    = 3   #redefine
PETIT    = 4
manette_regime = ARRET

DESS     = 2
RIEN     = 0
SERR     = 1
manette_frein = RIEN

KL_AV    = 1
KL_0     = 0
KL_AR    = 2
manette_klaxon  = KL_0
manette_klaxon2 = KL_0

LSSF_0   = 0
LSSF_1   = 1
LSSF_CLI = 2
lssf = LSSF_0
lssf_dcpt = LSSF_0

line_help = 1     # lignes d'aides (1=afficher en vert, 0=ne pas afficher)

# ecrasement de certains parametres(machine operationnelle directement)
palette_sens      = ARRET
pression_cg_0_5   = 5.0
pression_cf_0_5   = 0.0


#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# IMPORT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

import os
import time
import sys
import subprocess
import threading
import datetime
import socket
try:
    # for Python 3
    from tkinter import *
    print("Python 3.x interpreter detected")
except ImportError:
    # for Python2
    from Tkinter import *
    print("Python 2.x interpreter detected")
from math import *
PI = 3.14

ser = 0
serial_ok = 1
if(use_serial):
    try:
        import serial
    except ImportError:
        serial_ok = 0
        print("Cannot load serial lib")
else:
    serial_ok = 0


#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# DRAW FCTs
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

# parametres du pupitre
fichier_pupitre = "y8000_80.png"
img_pupitre = 0
img_dx = 0
img_dy = 0

x_aig_vit  = 471; y_aig_vit  = 123; d_aig_vit  = 27
x_aig_trm  = 682; y_aig_trm  =  96; d_aig_trm  = 19

x_aig_rp = 337; y_aig_rp = 131; d_aig_rp = 15
x_aig_cg = 264; y_aig_cg = 105; d_aig_cg = 30
x_aig_re = 337; y_aig_re = 131; d_aig_re = 15
x_aig_cf = 336; y_aig_cf = 86 ; d_aig_cf = 15

xc_boule_acc   = 113; yc_boule_acc   = 305
x0_boule_acc   = 30; y0_boule_acc   = 264
x100_boule_acc = 123; y100_boule_acc = 165

xc_boule_acc2  = 895; yc_boule_acc2  = 295
x0_boule_acc2  = 999; y0_boule_acc2  = 264
x100_boule_acc2= 820; y100_boule_acc2= 165

xc_boule_fr    = 200; yc_boule_fr     = 240
xdess_boule_fr = 215; ydess_boule_fr  = 190
x0_boule_fr    = 192; y0_boule_fr     = 207
xserr_boule_fr = 180; yserr_boule_fr  = 255
b_fr  = 15

xc_boule_fr2   = 787; yc_boule_fr2    = 240
xdess_boule_fr2= 747; ydess_boule_fr2 = 190
x0_boule_fr2   = 790; y0_boule_fr2    = 207
xserr_boule_fr2= 880; yserr_boule_fr2 = 255
b_fr2  = 15

xc_sens   = 308; yc_sens   = 230
xav_sens  = 310; yav_sens  = 170
x0_sens   = 308; y0_sens   = 195
xarr_sens = 306; yarr_sens = 240

xc_sens2   = 661; yc_sens2   = 224
xav_sens2  = 645; yav_sens2  = 170
x0_sens2   = 680; y0_sens2   = 195
xarr_sens2 = 700; yarr_sens2 = 240

xc_regime  = 497; yc_regime  = 248
xgr_regime = 442; ygr_regime = 216
x0_regime  = 507; y0_regime  = 193
xpt_regime = 572; ypt_regime = 216

xc_klaxon   =-25; yc_klaxon   = 340
xav_klaxon  = 20; yav_klaxon  = 320
x0_klaxon   = 25; y0_klaxon   = 340
xarr_klaxon = 20; yarr_klaxon = 360

xc_klaxon2   = 1036+25; yc_klaxon2   = 340
xav_klaxon2  = 1036-20; yav_klaxon2  = 320
x0_klaxon2   = 1036-25; y0_klaxon2   = 340
xarr_klaxon2 = 1036-20; yarr_klaxon2 = 360

xc_au = 457; yc_au = 176
x_au = 502; y_au  = 203

x_lssf       = 20     ; y_lssf       = 20
xc_ack_lssf  = 20     ; yc_ack_lssf  = 20
x_lssf2      = 1036-20; y_lssf2      = 20
xc_ack_lssf2 = 1036-20; yc_ack_lssf2 = 20

xc_neutre = 408; yc_neutre = 143; neutre = 1; old_neutre = -1; oval_neutre = 0

xc_voy_alerte = 486; yc_voy_alerte = 76; alerte = 0; old_alerte = -1; oval_alerte = 0
xc_cmd_alerte = 457; yc_cmd_alerte = 76

y_btn = 275
x_btn = [361,379,398, 427, 456,475,494,513,532,551,570, 600, 627,647,665]
v_btn = [  0,  0,  0,   0,   0,  0,  0,  0,  0,  0,  0,   0,   0,  0,  0]
rect_btn = []
redraw_btn = 1

line_aig_vit=0; line_aig_trm=0; line_aig_rp=0; line_aig_cg=0; line_aig_cf=0; line_aig_re=0
oval_au=0
line_cmd_sens=0; oval_cmd_sens=0
line_cmd_sens2=0; oval_cmd_sens2=0
line_cmd_regime=0; oval_cmd_regime=0
line_cmd_fr=0; oval_cmd_fr=0
line_cmd_fr2=0; oval_cmd_fr2=0
line_cmd_acc = 0; oval_cmd_acc = 0
line_cmd_acc2 = 0; oval_cmd_acc2 = 0
line_cmd_klaxon=0; oval_cmd_klaxon=0
line_cmd_klaxon2=0; oval_cmd_klaxon2=0
oval_lssf = 0; oval_lssf2 = 0
line_volt_bat = 0

zefont = ("Times New Roman", 6)
filename_pupitre = 0
dbg_txt = ""

#creation du pupitre (image, aiguilles, manettes)
def draw_first():
    global filename_pupitre   #necessaire car si local, image perdue a la sortie de la fonction
    global dbg_txt
    global line_aig_vit; global line_aig_trm;
    global line_aig_rp; global line_aig_cg; global line_aig_cf; global line_aig_re
    global oval_au;
    global line_cmd_sens; global oval_cmd_sens
    global line_cmd_sens2; global oval_cmd_sens2
    global line_cmd_regime; global oval_cmd_regime
    global line_cmd_acc; global oval_cmd_acc
    global line_cmd_acc2; global oval_cmd_acc2
    global line_cmd_fr; global oval_cmd_fr
    global line_cmd_fr2; global oval_cmd_fr2
    global line_cmd_klaxon; global oval_cmd_klaxon
    global line_cmd_klaxon2; global oval_cmd_klaxon2
    global oval_lssf; global oval_lssf2; global oval_alerte; global oval_neutre
    global line_volt_bat;
    global rect_btn

    #my_canvas.create_rectangle(x, y, x + 100, y +200 , fill = "#000000"  , outline = "#ffffff")

    dbg_txt = my_canvas.create_text(10, img_dy+10, text = "DBG: .............", fill = "#ff0000", anchor = "nw")

    # image pupitre
    #filename_pupitre = PhotoImage(file = image_du_pupitre)
    image_pupitre = my_canvas.create_image(0, 0, image=img_pupitre, anchor = "nw")   #anchor=NE,

    # aiguilles
    line_aig_vit = my_canvas.create_line(0,0,0,0,width=3,fill="#ffffff")
    line_aig_trm = my_canvas.create_line(0,0,0,0,width=2,fill="#ffffff")
    line_aig_rp  = my_canvas.create_line(0,0,0,0,width=2,fill="#ff0000")
    line_aig_cg  = my_canvas.create_line(0,0,0,0,width=2,fill="#ffffff")
    line_aig_cf  = my_canvas.create_line(0,0,0,0,width=2,fill="#ffffff")
    line_aig_re  = my_canvas.create_line(0,0,0,0,width=2,fill="#ffffff")

    # bargraphes
    line_volt_bat = my_canvas.create_line(625,101,634,85,width=2,fill="#ffffff")

    # avertisseur
    line_cmd_klaxon = my_canvas.create_line(0,0,0,0,width=7,fill="#444444")
    oval_cmd_klaxon = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    if(line_help):
        my_canvas.create_line(x0_klaxon,y0_klaxon,xav_klaxon,yav_klaxon,width=1,fill="#00ff00")
        my_canvas.create_line(x0_klaxon,y0_klaxon,xarr_klaxon,yarr_klaxon,width=1,fill="#00ff00")
    # avertisseur (2eme commande)
    line_cmd_klaxon2 = my_canvas.create_line(0,0,0,0,width=7,fill="#444444")
    oval_cmd_klaxon2 = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    if(line_help):
        my_canvas.create_line(x0_klaxon2,y0_klaxon2,xav_klaxon2,yav_klaxon2,width=1,fill="#00ff00")
        my_canvas.create_line(x0_klaxon2,y0_klaxon2,xarr_klaxon2,yarr_klaxon2,width=1,fill="#00ff00")

    # manette d'acceleration (0%=SERR 15%=NEUTRE 30%=DESS 40%=taction0 100%=traction100
    line_cmd_acc = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    oval_cmd_acc = my_canvas.create_oval(0,0,0,0, fill = "#111111", outline = "black")
    if(line_help):
        line_dbg_acc = my_canvas.create_line(x0_boule_acc,y0_boule_acc,x100_boule_acc,y100_boule_acc,width=1,fill="#00ff00")
    dx = x100_boule_acc - x0_boule_acc; dy = -(y100_boule_acc - y0_boule_acc);
    x = x0_boule_acc; y = y0_boule_acc
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)
    x = x0_boule_acc + int(0.15 * float(dx)); y = y0_boule_acc - int(0.15 * float(dy))
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)
    x = x0_boule_acc + int(0.30 * float(dx)); y = y0_boule_acc - int(0.30 * float(dy))
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)
    x = x0_boule_acc + int(0.40 * float(dx)); y = y0_boule_acc - int(0.40 * float(dy))
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)
    x = x0_boule_acc + dx; y = y0_boule_acc - dy
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)

    # 2eme manette d'acceleration (0%=SERR 15%=NEUTRE 30%=DESS 40%=taction0 100%=traction100
    line_cmd_acc2 = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    oval_cmd_acc2 = my_canvas.create_oval(0,0,0,0, fill = "#111111", outline = "black")
    if(line_help):
        line_dbg_acc2 = my_canvas.create_line(x0_boule_acc2,y0_boule_acc2,x100_boule_acc2,y100_boule_acc2,width=1,fill="#00ff00")
    dx = x100_boule_acc2 - x0_boule_acc2; dy = -(y100_boule_acc2 - y0_boule_acc2);
    x = x0_boule_acc2; y = y0_boule_acc2
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)
    if(line_help):
        my_canvas.create_text(x+5, y-5, text="S", fill="#00ff00", anchor = "w")
    x = x0_boule_acc2 + int(0.15 * float(dx)); y = y0_boule_acc2 - int(0.15 * float(dy))
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)
    if(line_help):
            my_canvas.create_text(x+5, y-5, text="N", fill="#00ff00", anchor = "w")
    x = x0_boule_acc2 + int(0.30 * float(dx)); y = y0_boule_acc2 - int(0.30 * float(dy))
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)
    if(line_help):
            my_canvas.create_text(x+5, y-5, text="D", fill="#00ff00", anchor = "w")
    x = x0_boule_acc2 + int(0.40 * float(dx)); y = y0_boule_acc2 - int(0.40 * float(dy))
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)
    if(line_help):
            my_canvas.create_text(x+5, y-5, text="0%", fill="#00ff00", anchor = "w")
    x = x0_boule_acc2 + dx; y = y0_boule_acc2 - dy
    my_canvas.create_rectangle(x-1, y-1, x+1, y+1, fill='#00ff00', outline='#00ff00', width=1)
    if(line_help):
            my_canvas.create_text(x+5, y-5, text="100%", fill="#00ff00", anchor = "w")
    #font=("Times", 10, "bold") ("Purisa", 12) ("Times", 10, "bold")  ("Helvetica", 10, "bold italic")    ("Symbol", 8)      #or zefont = ("Times", 10, "bold")    font=zefont

    # manette de freinage
    line_cmd_fr = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    oval_cmd_fr = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    if(line_help):
            my_canvas.create_line(x0_boule_fr,y0_boule_fr,xserr_boule_fr,yserr_boule_fr,width=1,fill="#00ff00")
            my_canvas.create_line(x0_boule_fr,y0_boule_fr,xdess_boule_fr,ydess_boule_fr,width=1,fill="#00ff00")
    # 2eme manette de freinage
    line_cmd_fr2 = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    oval_cmd_fr2 = my_canvas.create_oval(0,0,0,0,fill = "#111111", outline = "black")
    if(line_help):
        my_canvas.create_line(x0_boule_fr2,y0_boule_fr2,xserr_boule_fr2,yserr_boule_fr2,width=1,fill="#00ff00")
        my_canvas.create_line(x0_boule_fr2,y0_boule_fr2,xdess_boule_fr2,ydess_boule_fr2,width=1,fill="#00ff00")
        my_canvas.create_text(xdess_boule_fr2+5, ydess_boule_fr2-5, text="D", fill="#00ff00", anchor = "w")
        my_canvas.create_text(x0_boule_fr2+5   , y0_boule_fr2-5   , text="N", fill="#00ff00", anchor = "w")
        my_canvas.create_text(xserr_boule_fr2+5, yserr_boule_fr2-5, text="S", fill="#00ff00", anchor = "w")

    # manette de sens
    line_cmd_sens  = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    oval_cmd_sens = my_canvas.create_oval(0,0,0,0, fill = "#111111", outline = "black")
    if(line_help):
        my_canvas.create_line(x0_sens,y0_sens,xav_sens,yav_sens,width=1,fill="#00ff00")
        my_canvas.create_line(x0_sens,y0_sens,xarr_sens,yarr_sens,width=1,fill="#00ff00")
    # 2eme manette de sens
    line_cmd_sens2  = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    oval_cmd_sens2 = my_canvas.create_oval(0,0,0,0, fill = "#111111", outline = "black")
    if(line_help):
        my_canvas.create_line(x0_sens2,y0_sens2,xav_sens2,yav_sens2,width=1,fill="#00ff00")
        my_canvas.create_line(x0_sens2,y0_sens2,xarr_sens2,yarr_sens2,width=1,fill="#00ff00")
        my_canvas.create_text(xav_sens2+5 , yav_sens2-5 , text="AV", fill="#00ff00", anchor = "w")
        my_canvas.create_text(x0_sens2+5  , y0_sens2-5  , text="0" , fill="#00ff00", anchor = "w")
        my_canvas.create_text(xarr_sens2+5, yarr_sens2-5, text="AR", fill="#00ff00", anchor = "w")

    # manette de regime
    line_cmd_regime  = my_canvas.create_line(0,0,0,0,width=10,fill="#888888")
    oval_cmd_regime = my_canvas.create_oval(0,0,0,0, fill = "#111111", outline = "black")
    if(line_help):
        my_canvas.create_line(x0_regime,y0_regime,xpt_regime,ypt_regime,width=1,fill="#00ff00")
        my_canvas.create_line(x0_regime,y0_regime,xgr_regime,ygr_regime,width=1,fill="#00ff00")
        my_canvas.create_text(xgr_regime, ygr_regime+10, text="GR", fill="#00ff00")
        my_canvas.create_text(x0_regime , y0_regime+10 , text="0" , fill="#00ff00")
        my_canvas.create_text(xpt_regime, ypt_regime+10, text="PT", fill="#00ff00")

    # robinet d'arret d'urgence
    oval_au = my_canvas.create_oval(xc_au,yc_au,x_au,y_au,fill="#ff0000")  #a masquer si pas au

    #lssf
    oval_lssf  = my_canvas.create_oval(x_lssf -12,y_lssf -12,x_lssf +12,y_lssf +12,fill="#ffdd00")
    oval_lssf2 = my_canvas.create_oval(x_lssf2-12,y_lssf2-12,x_lssf2+12,y_lssf2+12,fill="#ffdd00")

    # alerte
    oval_alerte = my_canvas.create_oval(xc_voy_alerte-10,yc_voy_alerte-10,xc_voy_alerte+10,yc_voy_alerte+10, fill = "#ffffff", outline = "black")

    # neutre
    oval_neutre = my_canvas.create_oval(xc_neutre-10,yc_neutre-10,xc_neutre+10,yc_neutre+10, fill = "#11c0ff", outline = "black")

    #boutons
    for i in range(0,len(v_btn)):
        rect_btn.append(my_canvas.create_rectangle(x_btn[i]-5, y_btn-5, x_btn[i]+5, y_btn+5, fill='#888888', outline='#000000', width=1))

    return

# valeurs
aig_val_vit_0_210 = 0.0
aig_val_trm_0_2500 = 0.0
slow_aig_val_trm_0_2500 = 0.0
aig_val_rp_0_10 = 8.0
aig_val_re_0_10 = 0.0
aig_val_cg_0_10 = 0.0
aig_val_cf_0_10 = 5.0

old_aig_val_vit_0_210 = -1.0   #sauvegarde des valeurs entre 2 appels pour
old_aig_val_trm_0_2500 = -1.0 #ne pas redessiner et ainsi gagner en performances
old_aig_val_rp_0_10 = -1.0
old_aig_val_re_0_10 = -1.0
old_aig_val_cg_0_10 = -1.0
old_aig_val_cf_0_10 = -1.0
old_robinet_au = -1
old_palette_sens = -1
old_manette_regime = -1
old_manette_acc_0_100 = -1.0
old_manette_frein = -1
old_manette_klaxon = -1
old_manette_klaxon2 = -1
old_lssf = -1
draw_update_cpt = 0

#mise a jour des aiguilles et position des manettes
def draw_update():
    global my_canvas
    global line_aig_vit; global line_aig_trm;
    global line_cmd_acc; global oval_cmd_acc; global old_manette_acc_0_100
    global line_cmd_acc2; global oval_cmd_acc2;
    global line_aig_rp; global line_aig_cg; global line_aig_re; global line_aig_cf
    global line_cmd_fr; global oval_cmd_fr; global oval_au
    global line_cmd_fr2; global oval_cmd_fr2;
    global old_aig_val_vit_0_210; global old_aig_val_trm_0_2500;
    global old_aig_val_rp_0_10; global old_aig_val_cg_0_10; global old_aig_val_cf_0_10; global old_aig_val_re_0_10
    global old_robinet_au;
    global old_palette_sens; global old_manette_frein;
    global old_manette_klaxon; global line_cmd_klaxon; global oval_cmd_klaxon; global old_manette_regime;
    global old_manette_klaxon2; global line_cmd_klaxon2; global oval_cmd_klaxon2;
    global old_lssf
    global draw_update_cpt
    global oval_alerte; global old_alerte
    global oval_neutre; global old_neutre
    global redraw_btn; global rect_btn

    draw_update_cpt += 1

    #mise a jour des aiguilles
    if(aig_val_vit_0_210 != old_aig_val_vit_0_210):
        old_aig_val_vit_0_210 = aig_val_vit_0_210
        angle = 207.0 - (207.0-(-17.0))*(aig_val_vit_0_210/80.0)
        x = int( float(x_aig_vit) + float(d_aig_vit) * cos(radians(angle)) )
        y = int( float(y_aig_vit) - float(d_aig_vit) * sin(radians(angle)) )
        my_canvas.coords(line_aig_vit, x_aig_vit, y_aig_vit, x, y)

    if(aig_val_trm_0_2500 != old_aig_val_trm_0_2500):
        old_aig_val_trm_0_2500 = aig_val_trm_0_2500
        val = aig_val_trm_0_2500
        angle = 205.0 - (220.0+20.0)*(val/3000.0)
        x = int( float(x_aig_trm) + float(d_aig_trm) * cos(radians(angle)) )
        y = int( float(y_aig_trm) - float(d_aig_trm) * sin(radians(angle)) )
        my_canvas.coords(line_aig_trm, x_aig_trm, y_aig_trm, x, y)

    if(aig_val_rp_0_10 != old_aig_val_rp_0_10):
        angle = 225.0 - (225.0-(-38.0))*(aig_val_rp_0_10/10.0)
        x = int( float(x_aig_rp) + float(d_aig_rp) * cos(radians(angle)) )
        y = int( float(y_aig_rp) - float(d_aig_rp) * sin(radians(angle)) )
        my_canvas.coords(line_aig_rp, x_aig_rp, y_aig_rp, x, y)
        old_aig_val_rp_0_10 = aig_val_rp_0_10

    if(aig_val_re_0_10 != old_aig_val_re_0_10):
        angle = 225.0 - (225.0-(-25.0))*(aig_val_re_0_10/10.0)
        x = int( float(x_aig_re) + float(d_aig_re) * cos(radians(angle)) )
        y = int( float(y_aig_re) - float(d_aig_re) * sin(radians(angle)) )
        my_canvas.coords(line_aig_re, x_aig_re, y_aig_re, x, y)
        old_aig_val_re_0_10 = aig_val_re_0_10

    if(aig_val_cg_0_10 != old_aig_val_cg_0_10):
        angle = 225.0 - (225.0-(-40.0))*(aig_val_cg_0_10/10.0)
        x = int( float(x_aig_cg) + float(d_aig_cg) * cos(radians(angle)) )
        y = int( float(y_aig_cg) - float(d_aig_cg) * sin(radians(angle)) )
        my_canvas.coords(line_aig_cg, x_aig_cg, y_aig_cg, x, y)
        old_aig_val_cg_0_10 = aig_val_cg_0_10

    if(aig_val_cf_0_10 != old_aig_val_cf_0_10):
        angle = 220.0 - (220.0-(-200.0))*(aig_val_cf_0_10/10.0)
        x = int( float(x_aig_cf) + float(d_aig_cf) * cos(radians(angle)) )
        y = int( float(y_aig_cf) - float(d_aig_cf) * sin(radians(angle)) )
        my_canvas.coords(line_aig_cf, x_aig_cf, y_aig_cf, x, y)
        old_aig_val_cf_0_10 = aig_val_cf_0_10

    #robinet AU
    if(robinet_au != old_robinet_au):
        if(robinet_au==0):
            my_canvas.itemconfigure(oval_au,state=HIDDEN)
        else:
            my_canvas.itemconfigure(oval_au,state=NORMAL)
        old_robinet_au = robinet_au

    # alerte
    if(old_alerte != alerte):
        old_alerte = alerte
        if(alerte==0):
            my_canvas.itemconfigure(oval_alerte,state=HIDDEN)
        else:
            my_canvas.itemconfigure(oval_alerte,state=NORMAL)

    # neutre
    if(old_neutre != neutre):
        old_neutre = neutre
        if(neutre==0):
            my_canvas.itemconfigure(oval_neutre,state=HIDDEN)
        else:
            my_canvas.itemconfigure(oval_neutre,state=NORMAL)

    #palette sens
    if(palette_sens != old_palette_sens):
        old_palette_sens = palette_sens
        if(palette_sens == AV):
            x = xav_sens ; y=yav_sens
            x2= xav_sens2; y2=yav_sens2
        if(palette_sens == ARRET):
            x = x0_sens ; y=y0_sens
            x2= x0_sens2; y2=y0_sens2
        if(palette_sens == AR):
            x = xarr_sens;  y=yarr_sens
            x2= xarr_sens2; y2=yarr_sens2
        my_canvas.coords(line_cmd_sens, xc_sens, yc_sens, x, y)
        my_canvas.coords(oval_cmd_sens, x-15, y-15, x+15, y+15)
        my_canvas.coords(line_cmd_sens2, xc_sens2, yc_sens2, x2, y2)
        my_canvas.coords(oval_cmd_sens2, x2-15, y2-15, x2+15, y2+15)

    # manette regime
    if(manette_regime != old_manette_regime):
        old_manette_regime = manette_regime
        if(manette_regime == GRAND):
            x = xgr_regime ; y=ygr_regime
        if(manette_regime == ARRET):
            x = x0_regime; y=y0_regime
        if(manette_regime == PETIT):
            x = xpt_regime;  y=ypt_regime
        my_canvas.coords(line_cmd_regime, xc_regime, yc_regime, x, y)
        my_canvas.coords(oval_cmd_regime, x-15, y-15, x+15, y+15)

    # manette frein
    if(manette_frein != old_manette_frein):
        old_manette_frein = manette_frein
        x=y=0
        if(manette_frein==DESS):
            x  = xdess_boule_fr ; y  = ydess_boule_fr
            x2 = xdess_boule_fr2; y2 = ydess_boule_fr2
        if(manette_frein==RIEN):
            x  = x0_boule_fr ; y  = y0_boule_fr
            x2 = x0_boule_fr2; y2 = y0_boule_fr2
        if(manette_frein==SERR):
            x  = xserr_boule_fr ; y  = yserr_boule_fr
            x2 = xserr_boule_fr2; y2 = yserr_boule_fr2
        my_canvas.coords(line_cmd_fr , xc_boule_fr, yc_boule_fr, x, y)
        my_canvas.coords(oval_cmd_fr , x-b_fr, y-b_fr, x+b_fr, y+b_fr)
        my_canvas.coords(line_cmd_fr2, xc_boule_fr2, yc_boule_fr2, x2, y2)
        my_canvas.coords(oval_cmd_fr2, x2-b_fr2, y2-b_fr2, x2+b_fr2, y2+b_fr2)

    #manettes d'acceleration
    if(old_manette_acc_0_100 != manette_acc_0_100):
        old_manette_acc_0_100 = manette_acc_0_100
        x = x0_boule_acc - (x0_boule_acc - x100_boule_acc) * manette_acc_0_100 / 100
        y = y0_boule_acc - (y0_boule_acc - y100_boule_acc) * manette_acc_0_100 / 100
        my_canvas.coords(line_cmd_acc, xc_boule_acc, yc_boule_acc, x, y)
        my_canvas.coords(oval_cmd_acc, x-15, y-15, x+15, y+15)

        #2eme manettes d'acceleration
        x = x0_boule_acc2 - (x0_boule_acc2 - x100_boule_acc2) * manette_acc_0_100 / 100
        y = y0_boule_acc2 - (y0_boule_acc2 - y100_boule_acc2) * manette_acc_0_100 / 100
        my_canvas.coords(line_cmd_acc2, xc_boule_acc2, yc_boule_acc2, x, y)
        my_canvas.coords(oval_cmd_acc2, x-15, y-15, x+15, y+15)

    #avertisseur
    if(manette_klaxon != old_manette_klaxon):
        old_manette_klaxon = manette_klaxon
        if(manette_klaxon==KL_AV):
            x = xav_klaxon;
            y = yav_klaxon
        elif(manette_klaxon==KL_AR):
            x = xarr_klaxon
            y = yarr_klaxon
        else:
            x = x0_klaxon
            y = y0_klaxon
        my_canvas.coords(line_cmd_klaxon, xc_klaxon, yc_klaxon, x, y)
        my_canvas.coords(oval_cmd_klaxon, x-15, y-15, x+15, y+15)

    #avertisseur (2eme commande)
    if(manette_klaxon2 != old_manette_klaxon2):
        old_manette_klaxon2 = manette_klaxon2
        if(manette_klaxon2==KL_AV):
            x = xav_klaxon2;
            y = yav_klaxon2
        elif(manette_klaxon2==KL_AR):
            x = xarr_klaxon2
            y = yarr_klaxon2
        else:
            x = x0_klaxon2
            y = y0_klaxon2
        my_canvas.coords(line_cmd_klaxon2, xc_klaxon2, yc_klaxon2, x, y)
        my_canvas.coords(oval_cmd_klaxon2, x-15, y-15, x+15, y+15)

    #lssf
    if(old_lssf!=lssf or lssf==LSSF_CLI):
        old_lssf=lssf
        aff = 0
        if(lssf==LSSF_1):
            aff = 1
        if(lssf==LSSF_CLI and draw_update_cpt&4):
            aff = 1
        if(aff == 0):
            my_canvas.itemconfigure(oval_lssf ,state=HIDDEN)
            my_canvas.itemconfigure(oval_lssf2,state=HIDDEN)
        else:
            my_canvas.itemconfigure(oval_lssf ,state=NORMAL)
            my_canvas.itemconfigure(oval_lssf2,state=NORMAL)

    #boutons
    if(redraw_btn):
        redraw_btn = 0
        for i in range(0, len(v_btn)):
            if(v_btn[i] == 0):
                my_canvas.itemconfigure(rect_btn[i] ,state=HIDDEN)
            else:
                my_canvas.itemconfigure(rect_btn[i] ,state=NORMAL)

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# SIMU FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

# les parametres de simulations sont maintenant au debut du programme
vit_sens = 0.0    #vit mais avec un signe +=AV, -=AR


def simu():
    global vit; global vit_sens; global mem_sens
    global aig_val_vit_0_210
    global pression_cg_0_5; global pression_cf_0_5; global pression_cfd_0_5
    global aig_val_rp_0_10; global aig_val_re_0_10; global aig_val_cg_0_10; global aig_val_cf_0_10
    global dbg_txt
    global lssf_dcpt; global robinet_au; global neutre

    mode = ""

    #entrees
    if(palette_sens == ARRET and manette_regime == ARRET):
        neutre = 1  #en fait neutre = point mort ici
    else:
        neutre = 0

    if(manette_acc_0_100<=40 or motor_on==0 or manette_regime==ARRET or palette_sens==ARRET or robinet_au):
        traction_0_100 = 0.0
    else:
        traction_0_100 = float(manette_acc_0_100 - 40) * 100.0 / 60.0

    freind = RIEN
    if(manette_acc_0_100<7):
        freind = SERR
    elif(manette_acc_0_100>22):
        freind = DESS

    vmax2 = 0
    if(manette_regime == GRAND):
        vmax2 = vmax
    elif(manette_regime == PETIT):
        vmax2 = vmax_petit_regime
    v_cible = vmax2 * traction_0_100 / 100.0
    t_cible=0

    if(lssf == LSSF_CLI):
        if(lssf_dcpt==0):
            robinet_au = 1
        else:
            lssf_dcpt -= 1

    #arret
    if(traction_0_100==0 and vit==0.0):
        mode = "ARRET "

    #marche sur l'erre
    elif(traction_0_100==0 or motor_on==0 or v_cible<vit or robinet_au==1):
        mode = "ERRE "
        dv = (100.0/t_100_0_erre)/10
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        t_cible = vit / (10*dv)

    #frein moteur due au couplage hydraulique)
    if(traction_0_100!=0 and v_cible<vit and t_100_0_mot!=0.0):
        mode = "FR-MOT "
        if(profil==1):
            dv = (100.0/t_100_0_mot)/10.0
        if(profil==2):
            dv = (100.0/t2_100_0_mot)/10.0
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        t_cible = vit / (10*dv)

   #acceleration
    elif(v_cible>vit and palette_sens!=ARRET):
        if(profil==1):
            dv = (100.0/t_0_100_mot)/10.0     * traction_0_100 / 100.0
        if(profil==2):
            dv = (100.0/t2_0_100_mot)/10.0    * traction_0_100 / 100.0
        v=vit+dv
        if(v>v_cible):
            vit=v_cible
        else:
            vit=v
        mode = "ACC "
        print("ACC dv="+str(dv))
        t_cible = (v_cible-vit) / (10*dv)

    #precalcul des pressions
    #manette frein -> cg
    dp = (1.5 / t_press_3b5_5b) / 10.0
    if(manette_frein == SERR):
        p = pression_cg_0_5 - dp
        if(p < 0.0):
            p = 0.0
        pression_cg_0_5 = p
    if(manette_frein == DESS):
        p = pression_cg_0_5 + dp
        if(p > 5.0):
            p = 5.0
        pression_cg_0_5 = p

    #manette frein direct -> cfd
    dp = (5.0 / t_press_0b_5b_fd) / 10.0
    if(freind == SERR):
        p = pression_cfd_0_5 + dp
        if(p > 5.0):
            p = 5.0
        pression_cfd_0_5 = p
    if(freind == DESS):
        p = pression_cfd_0_5 - dp
        if(p < 0.0):
            p = 0.0
        pression_cfd_0_5 = p

    #robinet au -> cg
    if(robinet_au):
        dp = (1.5 / t_press_au_5b_3b5) / 10.0
        p = pression_cg_0_5 - dp
        if(p < 0.0):
            p = 0.0
        pression_cg_0_5 = p

    #cg=5 -> cf=0, cg=3.5 -> cf=5
    p =  (5.0 / 1.5) * (5.0 - pression_cg_0_5)
    if(p > 5.0):
        p = 5.0
    if(p < 0.0):
        p = 0.0
    pression_cf_0_5 = p

    fr_0_100 = 100.0 * (pression_cf_0_5 / 5.0)
    aig_val_rp_0_10 = pression_rp_0_10
    aig_val_cg_0_10 = pression_cg_0_5
    aig_val_re_0_10 = pression_cg_0_5
    aig_val_cf_0_10 = pression_cf_0_5

    #freinage pneumatique (sabots + disques)
    if(fr_0_100>0.0):
        mode += "FR-AUTO "
        dv = (100.0/t_100_0_fr)/10.0    *   fr_0_100 / 100.0
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        t_cible = vit / (10*dv)
        v_cible = 0

    #frein_direct
    if(pression_cfd_0_5 > pression_cf_0_5):
        mode += "FR-DIR "
        if(profil == 1):
            dv = (100.0/t_100_0_frd)/10.0    *   (pression_cfd_0_5 - pression_cf_0_5) / 5.0
        if(profil == 2):
            dv = (100.0/t2_100_0_frd)/10.0    *   (pression_cfd_0_5 - pression_cf_0_5) / 5.0
        v = vit - dv
        if(v<0.0):
            v=0.0
        vit = v
        t_cible = vit / (10*dv)
        v_cible = 0
        aig_val_cf_0_10 = pression_cfd_0_5

    #vitesse combinee avec sens
    if(palette_sens==AV):
        vit_sens = vit
        mem_sens = AV
    if(palette_sens==AR):
        vit_sens = -vit
        mem_sens = AR
    if(palette_sens==ARRET):
        vit_sens = 0

    #sorties
    aig_val_vit_0_210 = vit

    if(mode==""):
        mode = "STABLE"

    zetxt = "VIT=" + str(round(vit_sens,2)) + "kmph"
    zetxt += " VCIBLE=" + str(round(v_cible,2)) + "kmph"
    zetxt += " TCIBLE=" + str(round(t_cible,2)) + "s"
    zetxt += " ACC1=" + str(round(manette_acc_0_100,0)) + "%"
    zetxt += " ACC=" + str(round(traction_0_100,0)) + "%"
    zetxt += " RPM=" + str(round(aig_val_trm_0_2500,0)) + "trpm"
    zetxt += " MODE=" + mode
    zetxt += " PROFIL=" + str(profil)

    print(zetxt)
    my_canvas.itemconfigure(dbg_txt,text=zetxt)

    return


def simu_rpm():
    global aig_val_trm_0_2500; global slow_aig_val_trm_0_2500

    if(motor_on==0):
        slow_aig_val_trm_0_2500 = 0

    elif( manette_regime==ARRET or palette_sens==ARRET or manette_acc_0_100<40 or robinet_au==1):
        slow_aig_val_trm_0_2500 = rpm_ralenti

    else:
        traction_0_100 = float(manette_acc_0_100 - 40) * 100.0 / 60.0
        slow_aig_val_trm_0_2500 = rpm_ralenti + traction_0_100/100.0  * (rpm_nominal - rpm_ralenti)

    #pour que l'aiguille bouge lentement surtaout au l'allumage et arret du moteur
    if(slow_aig_val_trm_0_2500 > aig_val_trm_0_2500):
        aig_val_trm_0_2500 += slow_aig_trm_inc
        if(aig_val_trm_0_2500 > slow_aig_val_trm_0_2500):
            aig_val_trm_0_2500 = slow_aig_val_trm_0_2500
    if(slow_aig_val_trm_0_2500 < aig_val_trm_0_2500):
        aig_val_trm_0_2500 -= slow_aig_trm_inc
        if(aig_val_trm_0_2500 < slow_aig_val_trm_0_2500):
            aig_val_trm_0_2500 = slow_aig_val_trm_0_2500

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# SERIAL FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

#mise a jour de l'arduino avec les nouvelles valeurs
#
# protocol:
# s s s s s s s s p <0..sens> <vit0-255> k <0..KL_AV.KL_AR> 0         e
#                                                           |<ana0>   |<e0> <e1>
#
#  e0:   0 AU FRD-DESS FRD-SERR FRA-DESS FRA-SERR SENS_AV SENS-AR
MSK_AU       = 64
MSK_FRD_DESS = 32; MSK_GR_REGIME = 32   #comme le frein direct est integre au manipulateur,
MSK_FRD_SERR = 16; MSK_PT_REGIME = 16   #on peut reutiliser cette entree pour le regime
MSK_FRA_DESS = 8                        #ne simule pas la surcharge et l'urgence
MSK_FRA_SERR = 4
MSK_SENS_AV  = 2
MSK_SENS_AR  = 1
#  e1:   0  0        0        0        0        0  KL_AV    KL_AR
MSK_KL_AV = 2
MSK_KL_AR = 1
#  a0: 0-255 -> acc

first = 1; old_e1 = 0; old_e0 = 0; old_a0 = 0

def serial_update():
    global ser; global serial_ok
    global manette_acc_0_100; global palette_sens; global manette_klaxon; global manette_regime
    global robinet_au; global manette_frein;
    global first; global old_e1; global old_e0; global old_a0

    if(serial_ok==0):
        return

    try:
        nb_rx=0
        while(ser.inWaiting()):
            nb_rx+=1
            rb = ser.read(1)
            if(nb_rx==1): a0 = rb[0]
            if(nb_rx==2): e0 = rb[0]
            if(nb_rx==3): e1 = rb[0]
            #print("RX:" + str(rb[0]))

        if(nb_rx==3 and first==0):

            if(abs(old_a0 - a0)>5):
                manette_acc_0_100 = 100 * a0 / 255
                if(a0<5):
                    manette_acc_0_100 = 0
                if(a0>250):
                    manette_acc_0_100 = 100
                old_a0 = a0

            e0 ^= 255; e1 ^= 255;

            if((old_e0 & MSK_AU)==0 and (e0 & MSK_AU)):
                robinet_au ^= 1

            msk = MSK_SENS_AV | MSK_SENS_AR
            if((old_e0 ^ e0)& msk):
                if(e0 & MSK_SENS_AV):
                    palette_sens = AV
                elif(e0 & MSK_SENS_AR):
                    palette_sens = AR
                else:
                    palette_sens = ARRET

            msk = MSK_FRA_DESS | MSK_FRA_SERR
            if((old_e0 ^ e0)& msk):
                if(e0 & MSK_FRA_DESS):
                    manette_frein = DESS
                elif(e0 & MSK_FRA_SERR):
                    manette_frein = SERR
                else:
                    manette_frein = RIEN

            msk = MSK_GR_REGIME | MSK_PT_REGIME
            if((old_e0 ^ e0)& msk):
                if(e0 & MSK_GR_REGIME):
                    manette_regime = GRAND
                elif(e0 & MSK_PT_REGIME):
                    manette_regime = PETIT
                else:
                    manette_regime = ARRET

            msk =  MSK_KL_AV | MSK_KL_AR
            if((old_e1 ^ e1)& msk):
                if(e1 & MSK_KL_AV):
                    manette_klaxon = KL_AV
                elif(e1 & MSK_KL_AR):
                    manette_klaxon = KL_AR
                else:
                    manette_klaxon = KL_0

            old_e1 = e1
            old_e0 = e0

        first = 0

        # 1) synchros
        data = ord('s')
        for i in range(0,8):
            ser.write(bytes([data]))

        # 2) pwm command (sens + vit)
        ser.write(bytes([ord('p')]))
        #sens
        if(mem_sens == AV):
            sens = 1      #sens +
        else:
            sens = 0      #sens -
        sens ^= pwm_inv_sens
        ser.write(bytes([sens]))
        #print("TX:"+str(sens))
        #vit
        if(vit==0.0):
            v = 0
        else:
            v = pwm_min + int(float(pwm_max - pwm_min) * (vit / pwm_vit_max))
            if(v > pwm_max):
                v = pwm_max
        ser.write(bytes([v]))
        #print("TX:"+str(v))

        # 3) dcc command (adr + sens + vit + fct)
        ser.write(bytes([ord('d')]))
        #adresse
        ser.write(bytes([dcc_adr]))
        #sens
        if(mem_sens == AV):
            sens = 1      #sens +
        else:
            sens = 0      #sens -
        sens ^= dcc_inv_sens
        ser.write(bytes([sens]))
        #vit
        if(vit==0.0):
            v = 0
        else:
            v = dcc_min + int(float(dcc_max - dcc_min) * (vit / dcc_vit_max))
            if(v > dcc_max):
                v = dcc_max
        ser.write(bytes([v]))
        # fct (F0 si au moins un des boutons des phares est actif)
        fct = 0
        if(v_btn[0]):  #frontal av
            fct = 1
        if(v_btn[1]):  #gauche av
            fct = 1
        if(v_btn[2]):  #droite av
            fct = 1
        if(v_btn[3]):  #projo av
            fct = 1
        if(v_btn[4]):  #rouge gauche av
            fct = 1
        if(v_btn[5]):  #rouge droite av
            fct = 1
        if(v_btn[9]):  #rouge gauche ar
            fct = 1
        if(v_btn[10]):  #rouge droite ar
            fct = 1
        if(v_btn[11]):  #projo ar
            fct = 1
        if(v_btn[12]):  #gauche ar
            fct = 1
        if(v_btn[13]):  #droite ar
            fct = 1
        if(v_btn[14]):  #frontal ar
            fct = 1
        ser.write(bytes([fct]))

        # 3) cmd klaxon (plus utilise car le timer 2 est utilise par le DCC)
        #ser.write(bytes([ord('k')]))
        #data = 0
        #if(manette_klaxon  == KL_AV):  # 0.0.0.0.0.0.KLAV.KLAR
        #    data |= 2
        #if(manette_klaxon  == KL_AR):  # 0.0.0.0.0.0.KLAV.KLAR
        #    data |= 1
        #if(manette_klaxon2 == KL_AV):  # 0.0.0.0.0.0.KLAV.KLAR
        #    data |= 2
        #if(manette_klaxon2 == KL_AR):  # 0.0.0.0.0.0.KLAV.KLAR
        #    data |= 1
        #ser.write(bytes([data]))
        #print("TX:"+str(data))

        # 4) read analog 0
        ser.write(bytes([ord('0')]))

        # 5) read boutons
        ser.write(bytes([ord('e')]))

    except:
        serial_ok=0
        print("ERROR: Perte de la liaison serie")

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# NETWORK FCT
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

def net_connect():
    if(use_network==0):
        return

    if(use_local):
        adr = socket.gethostbyname(socket.gethostname())
    else:
        adr = server_ip_adr

    s.connect((adr, server_port))           #uu: mettre une touche d'inversion ex: I
    print("NETWORK: connected")
    welcome = s.recv(1024).decode()
    print("NETWORK: server string: \"" + welcome + "\"")
    s.send("cabine netv1 y8000".encode())
    return

net_connect()

def net_disconnect():
    if(use_network==0):
        return
    s.close()
    return

old_msg = ""

def net_update():
    global old_msg

    if(use_network==0):
        return

    # adr
    msg = "a" + str(dcc_adr)

    # sens
    if(mem_sens == AV):
        sens = 1      #sens +
    else:
        sens = 0      #sens -
    sens ^= dcc_inv_sens
    if(sens):
        msg += " s+"
    else:
        msg += " s-"

    # vit
    if(vit==0.0):
        v = 0
    else:
        v = net_min + int(float(net_max - net_min) * (vit / dcc_vit_max))
        if(v > net_max):
            v = net_max
    msg += " v" + str(v)

    # fct (F0 si au moins un des boutons des phares est actif)
    fct = 0
    if(v_btn[0]):  #frontal av
        fct = 1
    if(v_btn[1]):  #gauche av
        fct = 1
    if(v_btn[2]):  #droite av
        fct = 1
    if(v_btn[3]):  #projo av
        fct = 1
    if(v_btn[4]):  #rouge gauche av
        fct = 1
    if(v_btn[5]):  #rouge droite av
        fct = 1
    if(v_btn[9]):  #rouge gauche ar
        fct = 1
    if(v_btn[10]):  #rouge droite ar
        fct = 1
    if(v_btn[11]):  #projo ar
        fct = 1
    if(v_btn[12]):  #gauche ar
        fct = 1
    if(v_btn[13]):  #droite ar
        fct = 1
    if(v_btn[14]):  #frontal ar
        fct = 1
    if(fct):
        msg += " f0+"
    else:
        msg += " f0-"

    msg += "\n"

    # send msg
    if(msg != old_msg):
        print("NETWORK: send: " + msg)
        s.send(msg.encode())
        old_msg = msg

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# CALLBACKS
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

cpt_avr = 0
cpt_err = 0

def timer_1s_cb():
    global cpt_avr
    #global aig_val_vit_0_210; global aig_val_trm_0_2500
    #global aig_val_rp_0_10; global aig_val_cg_0_10; global aig_val_cf_0_10
    cpt_avr += 1
    ##print("----------------------------------")
    ##print("cpt_avr = " + str(cpt_avr));
    #aig_val_vit_0_210 = float(s_pot.get())
    #aig_val_trm_0_2500 = float(s_pot.get())
    #aig_val_rp_0_10 = float(s_pot.get())
    #aig_val_cg_0_10 = float(s_pot.get())
    #aig_val_cf_0_10 = float(s_pot.get())

    simu()
    simu_rpm()

    serial_update()
    net_update()

    draw_update()


mouse_owner = ""
x_press = 0
y_press = 0
mem_manette_acc_0_100 = 0

def callback_my_canvas_mouse_press(event):
    global mouse_owner; global x_press; global y_press
    global motor_on;
    global lssf; global alerte
    global mem_manette_acc_0_100
    global v_btn; global redraw_btn

    m_x = int(event.x); x_press = m_x
    m_y = int(event.y); y_press = m_y
    #print("mouse press at " + str(m_x) + " " + str(m_y))

    if(m_x>xc_au and m_x<x_au-10 and m_y>yc_au and m_y<y_au):
        mouse_owner = "robinet_au"
        return

    if(m_x>xc_cmd_alerte-10 and m_x<xc_cmd_alerte+10 and m_y>yc_cmd_alerte-10 and m_y<yc_cmd_alerte+10):
        alerte ^= 1

    if(m_x>x0_sens-15 and m_x<x0_sens+15 and m_y>yav_sens-15 and m_y<yarr_sens+15):
        mouse_owner = "sens"
    if(m_x>xav_sens2-15 and m_x<xarr_sens2+15 and m_y>yav_sens2-15 and m_y<yarr_sens2+15):
        mouse_owner = "sens"

    if(m_x>xgr_regime-15 and m_x<xpt_regime+15 and m_y>y0_regime-15 and m_y<ypt_regime+15):
        mouse_owner = "regime"

    if(m_x>xserr_boule_fr-15 and m_x<xdess_boule_fr+15 and m_y>ydess_boule_fr-15 and m_y<yserr_boule_fr+15):
        mouse_owner = "frein"
    if(m_x>xdess_boule_fr2-15 and m_x<xserr_boule_fr2+15 and m_y>ydess_boule_fr2-15 and m_y<yserr_boule_fr2+15):
        mouse_owner = "frein"

    if(m_x>x0_klaxon-20 and m_x<x0_klaxon+20 and m_y>y0_klaxon-20 and m_y<y0_klaxon+20):
        mouse_owner = "klaxon"

    if(m_x>x0_klaxon2-20 and m_x<x0_klaxon2+20 and m_y>y0_klaxon2-20 and m_y<y0_klaxon2+20):
        mouse_owner = "klaxon2"

    if(m_x>534-15 and m_x<534+15 and m_y>87-15 and m_y<87+15):
        if(manette_acc_0_100<40):
            motor_on = 1

    elif(m_x>534-15 and m_x<534+15 and m_y>117-15 and m_y<117+15):
        motor_on = 0

    if(lssf == LSSF_CLI):
        if(m_x>xc_ack_lssf-20 and m_x<xc_ack_lssf+20 and m_y>yc_ack_lssf-20 and m_y<yc_ack_lssf+20):
            lssf = LSSF_1
        if(m_x>xc_ack_lssf2-20 and m_x<xc_ack_lssf2+20 and m_y>yc_ack_lssf2-20 and m_y<yc_ack_lssf2+20):
            lssf = LSSF_1

    if(m_x<143 and m_y>145 and m_y<285):
        x1 = x0_boule_acc - (x0_boule_acc - x100_boule_acc) * manette_acc_0_100 / 100
        y1 = y0_boule_acc - (y0_boule_acc - y100_boule_acc) * manette_acc_0_100 / 100
        if(m_x>x1-20 and m_x<x1+20 and m_y>y1-20 and m_y<y1+20):
            mouse_owner = "acc"
            mem_manette_acc_0_100 = manette_acc_0_100

    if(m_x>800 and m_y>145 and m_y<285):
        x1 = x0_boule_acc2 - (x0_boule_acc2 - x100_boule_acc2) * manette_acc_0_100 / 100
        y1 = y0_boule_acc2 - (y0_boule_acc2 - y100_boule_acc2) * manette_acc_0_100 / 100
        if(m_x>x1-20 and m_x<x1+20 and m_y>y1-20 and m_y<y1+20):
            mouse_owner = "acc"
            mem_manette_acc_0_100 = manette_acc_0_100

    if(m_y > y_btn-5 and m_y < y_btn+5):
        for i in range(0, len(x_btn)):
            x = x_btn[i]
            if(m_x > x-5 and m_x < x+5):
                v_btn[i] ^= 1
                redraw_btn = 1

    return


def callback_my_canvas_mouse_move(event):
    global manette_frein;
    global manette_klaxon
    global manette_klaxon2
    global manette_acc_0_100
    m_x = int(event.x);
    m_y = int(event.y);

    #print("mouse move at " + str(m_x) + " " + str(m_y) + " owner=" + mouse_owner)

    if(mouse_owner=="acc"):
        dy = y_press - m_y   #positif ds le sens des accelerations
        dy_100 = y0_boule_acc - y100_boule_acc
        v = mem_manette_acc_0_100 + 100 * dy / dy_100
        if(v<0):
            v=0
        if(v>100):
            v=100
        manette_acc_0_100=v

    elif(mouse_owner=="frein"):
        if(m_y < y_press-20):   #desserage
            manette_frein=DESS
        elif(m_y > y_press+20): #serrage
            manette_frein=SERR
        else:
            manette_frein=RIEN

    elif(mouse_owner == "klaxon"):
        if(m_y < y_press-10):
            manette_klaxon = KL_AV
        elif(m_y > y_press+10):
            manette_klaxon = KL_AR
        else:
            manette_klaxon = KL_0

    elif(mouse_owner == "klaxon2"):
        if(m_y < y_press-10):
            manette_klaxon2 = KL_AV
        elif(m_y > y_press+10):
            manette_klaxon2 = KL_AR
        else:
            manette_klaxon2 = KL_0

    return


def callback_my_canvas_mouse_release(event):
    global robinet_au
    global palette_sens;
    global mouse_owner
    global manette_frein
    global manette_klaxon; global manette_klaxon2
    global motor_on;
    global manette_acc_0_100
    global manette_regime

    m_x = int(event.x);
    m_y = int(event.y);

    if(mouse_owner=="robinet_au"):
        robinet_au ^= 1
        manette_acc_0_100 = 0   #le moteur est juste mis au ralentit maintenant

    elif(mouse_owner=="sens"):
        #if(vit==0.0 and manette_acc_0_100==0):
        v=palette_sens
        if(m_y<(y_press-10)):
            if(v == AR):
                v = ARRET
            elif(v == ARRET):
                v = AV
        elif(m_y>(y_press+10)):
            if(v == AV):
                v = ARRET
            elif(v == ARRET):
                v = AR
        palette_sens=v

    elif(mouse_owner=="regime"):
        v=manette_regime
        if(m_x<(x_press-10)):
            if(v == PETIT):
                v = ARRET
            elif(v == ARRET):
                v = GRAND
        elif(m_x>(x_press+10)):
            if(v == GRAND):
                v = ARRET
            elif(v == ARRET):
                v = PETIT
        manette_regime=v

    #elif(mouse_owner=="frein"):
    #    manette_frein=RIEN

    elif(mouse_owner=="klaxon"):
        manette_klaxon=KL_0

    elif(mouse_owner=="klaxon2"):
        manette_klaxon2=KL_0

    mouse_owner=""
    return


def callback_my_canvas_mouse_press_right(event):

    m_x = int(event.x);
    m_y = int(event.y);

    return


def callback_my_canvas_mouse_release_right(event):
    return


# mapping du pave numerique
# 7 8 9  -->  T+  FA-DESS AV
# 4 5 6  -->  T-  FA-SERR AR
# 1 2 3  -->  GR  PT      KL
# 0 0 .  -->  AU  AU      KL
def cb_key(event):
    global robinet_au
    global manette_acc_0_100
    global manette_frein
    global palette_sens
    global manette_regime
    global manette_klaxon
    global lssf; global lssf_dcpt
    global vit
    global profil
    global pwm_inv_sens
    global dcc_inv_sens


    k = repr(event.char)
    #print("pressed " + k)  #return 'a' with '

    if(k[1]=="0"):
        robinet_au ^= 1

    elif(k[1]=="7"):
        v = manette_acc_0_100
        v = (v//5) * 5 + 5    #(v//5)*5 pour ne garder que multiples de 5
        if(v > 100):
            v = 100
        manette_acc_0_100 = v
    elif(k[1]=="4"):
        v = manette_acc_0_100
        v = (v//5) * 5 - 5
        if(v < 0):
            v = 0
        manette_acc_0_100 = v

    elif(k[1]=="8"):
        v = manette_frein
        if(v==SERR):
            v=RIEN
        elif(v==RIEN):
            v=DESS
        manette_frein=v
    elif(k[1]=="5"):
        v = manette_frein
        if(v==DESS):
            v=RIEN
        elif(v==RIEN):
            v=SERR
        manette_frein=v

    elif(k[1]=="9"):
        v = palette_sens
        if(v==AR):
            v=ARRET
        elif(v==ARRET):
            v=AV
        palette_sens = v

    elif(k[1]=="6"):
        v = palette_sens
        if(v==AV):
            v=ARRET
        elif(v==ARRET):
            v=AR
        palette_sens = v

    elif(k[1]=="1"):
        v = manette_regime
        if(v==PETIT):
            v=ARRET
        elif(v==ARRET):
            v=GRAND
        manette_regime = v

    elif(k[1]=="2"):
        v = manette_regime
        if(v==GRAND):
            v=ARRET
        elif(v==ARRET):
            v=PETIT
        manette_regime = v

    elif(k[1]=="3"):
        manette_klaxon = KL_AV
    elif(k[1]=="."):
        manette_klaxon = KL_AR

    elif(k[1]=="l"):
        if(lssf == LSSF_0):
            lssf_dcpt = 50
            lssf = LSSF_CLI
        if(lssf == LSSF_1):
            lssf = LSSF_0

    elif(k[1]==" "):    #arret immediat loco miniature avec la barre d'espace
        robinet_au = 1
        vit = 0

    elif(k[1]=="p"):
        profil += 1
        if(profil > 2):
            profil = 1

    elif(k[1]=="i"):
        pwm_inv_sens ^= 1
        dcc_inv_sens ^= 1

    return


def cb_key_release(event):
    global manette_klaxon

    k = repr(event.char)
    #print("released " + k)

    if(k[1]=="3"):
        manette_klaxon = KL_0
    elif(k[1]=="."):
        manette_klaxon = KL_0

    return



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# GUI management
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

top = Tk()
top.title(TITLE)

row = 0

#e_v_div_mbps = Entry(top)
#b_reset      = Button(top, text = "reset", command = reset_cb)
#l_sec        = Label(top, text = "0 sec", font=("Helvetica", 16), fg = "#800000")
#l_v_div.grid(row = row, column = 0, sticky = W + E + N + S)
#e_v_div_mbps.grid(row = row, column = 1, sticky = W + E + N + S)
#b_reset.grid(row = row, column = 3, sticky = W + E + N + S)
#l_sec.grid(row = row, column = 1, sticky = W + E + N + S)
#e_v_div_mbps.delete(0, 1000); e_v_div_mbps.insert(0, "100")

img_pupitre = PhotoImage(file = fichier_pupitre)
img_dx = img_pupitre.width()
img_dy = img_pupitre.height()

my_canvas = Canvas(top, bg = "#cccccc", height = img_dy+20, width = img_dx)
my_canvas.grid(row = row, column = 0, columnspan = 4)
row=row+1

#---------------------------------------------------------------------------------------

#l_pot = Label(top, text ="Pot")
##s_pot = Scale(top, from_=0, to=210, orient=HORIZONTAL, tickinterval=10)
##s_pot = Scale(top, from_=0, to=2500, orient=HORIZONTAL, tickinterval=100)
#s_pot = Scale(top, from_=0, to=10, orient=HORIZONTAL, tickinterval=1)
#s_pot.grid(row=row,column=0,sticky=W+E+N+S)
#s_pot.grid(row=row,column=1,columnspan=4,sticky=W+E+N+S)
#s_pot.set(0) #s_pot.get()
#row=row+1

#---------------------------------------------------------------------------------------

#l_0 = Label(top, text ="----------------------")
#l_1 = Label(top, text ="----------------------")
#l_2 = Label(top, text ="----------------------")
#l_3 = Label(top, text ="----------------------")

#l_0.grid(row = row, column = 0)
#l_1.grid(row = row, column = 1)
#l_2.grid(row = row, column = 2)
#l_3.grid(row = row, column = 3)
#row = row + 1

my_canvas.bind("<Button-1>"       , callback_my_canvas_mouse_press)
my_canvas.bind("<ButtonRelease-1>", callback_my_canvas_mouse_release)
my_canvas.bind("<B1-Motion>"       , callback_my_canvas_mouse_move)
my_canvas.bind("<Button-3>"        , callback_my_canvas_mouse_press_right)
my_canvas.bind("<ButtonRelease-3>", callback_my_canvas_mouse_release_right)
top.bind("<Key>", cb_key)
top.bind("<KeyRelease>", cb_key_release)



#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/
# MAIN
#/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/

if(serial_ok):
    try:
        ser = serial.Serial(SZ_SERIALPORT,115200,timeout=10,stopbits=serial.STOPBITS_TWO)
        ser.setDTR(False)
        print("INFO:  Ouverture du port serie " + SZ_SERIALPORT + " OK")
    except:
        serial_ok=0
        print("ERROR: Ne peut pas ouvrir le port serie " + SZ_SERIALPORT)

#myFont = Font(family="Times New Roman", size=12)
#text.configure(font=myFont)
#myFont.configure(size=14)
draw_first()
draw_update()

# Thread on enumeration checking
daemon_bool = True
def daemon_func():
    global cpt_err
    next_call = time.time()
    while daemon_bool:
        print("time="+str(datetime.datetime.now())+" cpt_err="+str(cpt_err)+" x="+str(x_press)+" y="+str(y_press))
        try:
            timer_1s_cb()
            next_call += 0.1
            sleep_duration = next_call - time.time()
            if(sleep_duration <= 0):
                print("ERROR: non accurate time !!")
                cpt_err += 1
                #time.sleep(1)
                next_call = time.time()
            else:
                time.sleep(sleep_duration)
        except RuntimeError:
            None


daemon_thread = threading.Thread(target=daemon_func)
daemon_thread.daemon = True
daemon_thread.start()

top.mainloop()

if(serial_ok):
    ser.close()
net_disconnect()
daemon_bool = False
print("ByeBye")
time.sleep(1)



# QUELQUES NOTES SUR LA PWM POUR ARDUINO UNO R3:

# /*
# * T0A --> PWM D6
# * T0B --> PWM D5
# * T1A --> PWM D9
# * T1B --> PWM D10
# * T2A --> PWM D11
# * T2B --> PWM D3
# *
# * analogWrite(pin, value);  // 0->0% 1->2/256% 255->256/256%
# * T0 aussi utilise pour millis ... (pas de pb pour utiliser pwm en //)
# * T2 peut etre utilise par tone (dans ce cas pas de pwm avec T2)
# *     tone(pin,value,[duration ms])  //31Hz-65kHz
# *     notone()
# * T1 peut etre utilise par servo (dans ce cas pas de pwm avec T1) servos sur 12 pins 2-13
# *     #include <Servo.h>
# *     Servo aServo  or   Servo tServo[12]
# *     aServo.attach(pin_number)
# *     aServo.writeMicroseconds(uS) // aServo.write(0-180)
# *     plus besoin d'utiliser refresh
# */

# EXEMPLE DE CODE ARDUINO DE BASE supportant juste PWM (sens et vitesse)
# QUI FONCTIONNE AVEC CETTE CABINE:
# Pour le code plus complet avec DCC et manette, regarder le fichier cabine.ino le plus recent
#
# #define PIN_PWM  6
# #define PIN_SENS 5
#
# void setup()
# {
#    pinMode(PIN_PWM , OUTPUT); digitalWrite(PIN_PWM , LOW);
#    pinMode(PIN_SENS, OUTPUT); digitalWrite(PIN_SENS, LOW);
#
#    pinMode(13, OUTPUT);
#    digitalWrite(13, HIGH);
#    Serial.begin(115200);
#    while (!Serial) { }
#    digitalWrite(13, LOW);
# }
#
# int step=0;
# int sens;
# int pwm_vit;
#
# void loop()
# {
#     int rx;
#
#     rx = Serial.read();
#     if(rx==-1) return;
#
#     if(step==0)
#     {
#         if(rx=='s') {}
#         if(rx=='p') step=1;
#         if(rx=='0') Serial.write(analogRead(A0)/4);   //0-1023->0-255
#         if(rx=='1') Serial.write(analogRead(A1)/4);   //0-1023->0-255
#         if(rx=='e') { Serial.write(0xff); Serial.write(0xff); } //not supported
#         if(rx=='k') step=3;
#         if(rx=='d') step=4;
#     }
#     else if(step==1)
#     {
#         sens = rx;
#         step = 2;
#     }
#     else if(step==2)
#     {
#         pwm_vit = rx;
#         step = 0;
#
#         if(sens)
#             digitalWrite(PIN_SENS, HIGH);
#         else
#             digitalWrite(PIN_SENS, LOW);
#
#         analogWrite(PIN_PWM, pwm_vit);
#     }
#     else if(step==3)
#     {
#         step = 0;
#     }
#     else if(step==4)
#     {
#          step = 5;
#     }
#     else if(step==5)
#     {
#          step = 6;
#     }
#     else if(step==6)
#     {
#          step = 7;
#     }
#     else if(step==7)
#     {
#          step = 0;
#     }
# }
